@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,8 +21,7 @@
  * the two.
  */
 
-#ifndef _ABOUT_H_
-#define _ABOUT_H_
+#pragma once
 
 #include <QStringList>
 
@@ -34,4 +33,15 @@
  */
 QStringList libraryDependencies(void);
 
-#endif /* _ABOUT_H_ */
+/*!
+ * @brief Compare newest available version and application version.
+ *
+ * @param[in] vStr1 Version string.
+ * @param[in] vStr2 Version string.
+ * @retval -2 if vStr1 does not contain a suitable version string
+ * @retval -1 if vStr1 is less than vStr2
+ * @retval  0 if vStr1 is equal to vStr2
+ * @retval  1 if vStr1 is greater than vStr2
+ * @retval  2 if vStr2 does not contain a suitable version string
+ */
+int compareVersionStrings(const QString &vStr1, const QString &vStr2);
