@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,8 +21,7 @@
  * the two.
  */
 
-#ifndef _CLI_H_
-#define _CLI_H_
+#pragma once
 
 #include <QSet>
 #include <QString>
@@ -59,6 +58,10 @@
 #define SER_GET_MSG_IDS "get-msg-ids"
 #define SER_FIND_DATABOX "find-databox"
 
+#define SER_EXPORT_MSG "export-msg"
+#define SER_EXPORT_MSGS "export-msgs"
+#define SER_IMPORT_MSG "import-msg"
+#define SER_IMPORT_MSGS "import-msgs"
 
 // set of return error values
 enum cli_error {
@@ -86,5 +89,3 @@ extern const QSet<QString> serviceSet;
 */
 int runService(const QString &lParam,
     const QString &service, const QString &sParam);
-
-#endif /* _CLI_H_ */
