@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -23,11 +23,13 @@
 
 #include "src/common.h" /* ISDS_CONNECT_TIMEOUT_MS */
 #include "src/cli/cli_login.h"
+#include "src/datovka_shared/identifiers/account_id.h"
+#include "src/datovka_shared/log/log.h"
+#include "src/global.h"
 #include "src/io/account_db.h"
 #include "src/io/isds_helper.h"
 #include "src/io/isds_login.h"
-#include "src/log/log.h"
-#include "src/settings/preferences.h"
+#include "src/settings/prefs_specific.h"
 
 /*!
  * @brief Performs a ISDS log-in operation.
@@ -39,24 +41,22 @@
  * @return True on successful login.
  */
 static
-bool logInCLI(IsdsSessions &isdsSessions, AcntSettings &acntSettings,
+bool logInCLI(IsdsSessions &isdsSessions, AcntData &acntSettings,
     const QString &pwd, const QString &key)
 {
 	if (!acntSettings.isValid()) {
 		return false;
 	}
 
-	{
-		const QString userName(acntSettings.userName());
-		if (userName.isEmpty()) {
-			Q_ASSERT(0);
-			return false;
-		}
-		/* Create clean session if session doesn't exist. */
-		if (!isdsSessions.holdsSession(userName)) {
-			isdsSessions.createCleanSession(userName,
-			    ISDS_CONNECT_TIMEOUT_MS);
-		}
+	const QString userName(acntSettings.userName());
+	if (Q_UNLIKELY(userName.isEmpty())) {
+		Q_ASSERT(0);
+		return false;
+	}
+	/* Create clean session if session doesn't exist. */
+	if (!isdsSessions.holdsSession(userName)) {
+		isdsSessions.createCleanSession(userName,
+		    ISDS_CONNECT_TIMEOUT_MS);
 	}
 
 	enum IsdsLogin::ErrorCode errCode;
@@ -123,6 +123,15 @@ bool logInCLI(IsdsSessions &isdsSessions, AcntSettings &acntSettings,
 				return false;
 			}
 			break;
+		case IsdsLogin::EC_NO_MEP:
+			logErrorNL(
+			    "Missing MEP communication key for account '%s'.",
+			    acntSettings.accountName().toUtf8().constData());
+			return false;
+			break;
+		case IsdsLogin::EC_MEP_PARTIAL_SUCCESS:
+			/* Waiting for mobile key confirmation. */
+			break;
 		default:
 			logErrorNL(
 			    "Received log-in error code %d for account '%s'.",
@@ -133,15 +142,16 @@ bool logInCLI(IsdsSessions &isdsSessions, AcntSettings &acntSettings,
 		}
 	} while (errCode != IsdsLogin::EC_OK);
 
-	if (errCode != IsdsLogin::EC_OK) {
+	if (Q_UNLIKELY(errCode != IsdsLogin::EC_OK)) {
 		Q_ASSERT(0);
 		return false;
 	}
 
+	isdsSessions.setLoggedIn(userName);
 	return true;
 }
 
-bool connectToIsdsCLI(IsdsSessions &isdsSessions, AcntSettings acntSettings,
+bool connectToIsdsCLI(IsdsSessions &isdsSessions, AcntData acntSettings,
     const QString &pwd, const QString &key)
 {
 	if (!logInCLI(isdsSessions, acntSettings, pwd, key)) {
@@ -150,37 +160,60 @@ bool connectToIsdsCLI(IsdsSessions &isdsSessions, AcntSettings acntSettings,
 
 	/* Logged in. */
 
-	const QString userName(acntSettings.userName());
-	if (userName.isEmpty()) {
+	const AcntId acntId(acntSettings.userName(), acntSettings.isTestAccount());
+	if (Q_UNLIKELY(!acntId.isValid())) {
 		Q_ASSERT(0);
 		return false;
 	}
 
-	IsdsHelper::getOwnerInfoFromLogin(userName);
-	IsdsHelper::getUserInfoFromLogin(userName);
-	IsdsHelper::getPasswordInfoFromLogin(userName);
+	if (Q_UNLIKELY(!IsdsHelper::getOwnerInfoFromLogin(acntId))) {
+		logWarningNL(
+		    "Couldn't download and update box owner information for user '%s' of account '%s'.",
+		    acntId.username().toUtf8().constData(),
+		    acntSettings.accountName().toUtf8().constData());
+	}
+	if (Q_UNLIKELY(!IsdsHelper::getUserInfoFromLogin(acntId))) {
+		logWarningNL(
+		    "Couldn't download and update user information for user '%s' of account '%s'.",
+		    acntId.username().toUtf8().constData(),
+		    acntSettings.accountName().toUtf8().constData());
+	}
+	const QString dbID = GlobInstcs::accntDbPtr->dbId(
+	    AccountDb::keyFromLogin(acntId.username()));
+	if (Q_UNLIKELY(!IsdsHelper::getDtInfo(acntId, dbID))) {
+		logWarningNL(
+		    "Couldn't download and update long term storage information for user '%s' of account '%s'.",
+		    acntId.username().toUtf8().constData(),
+		    acntSettings.accountName().toUtf8().constData());
+	}
+	if (Q_UNLIKELY(!IsdsHelper::getPasswordInfoFromLogin(acntId))) {
+		logWarningNL(
+		    "Couldn't download and update password information for user '%s' of account '%s'.",
+		    acntId.username().toUtf8().constData(),
+		    acntSettings.accountName().toUtf8().constData());
+	}
 
 	/* Check password expiration. */
 	if (!acntSettings._pwdExpirDlgShown()) {
 		/* Notify only once. */
 		acntSettings._setPwdExpirDlgShown(true);
 
-		int daysTo = globAccountDbPtr->pwdExpiresInDays(
-		    AccountDb::keyFromLogin(userName),
+		int daysTo = GlobInstcs::accntDbPtr->pwdExpiresInDays(
+		    AccountDb::keyFromLogin(acntId.username()),
 		    PWD_EXPIRATION_NOTIFICATION_DAYS);
 
 		if (daysTo >= 0) {
 			logWarningNL(
 			    "Password for user '%s' of account '%s' expires in %d days.",
-			    userName.toUtf8().constData(),
+			    acntId.username().toUtf8().constData(),
 			    acntSettings.accountName().toUtf8().constData(),
 			    daysTo);
 		}
 	}
 
 	/* Set longer time-out. */
-	isdsSessions.setSessionTimeout(userName,
-	    globPref.isds_download_timeout_ms);
+	isdsSessions.setSessionTimeout(acntId.username(),
+	    PrefsSpecific::isdsDownloadTimeoutMs(*GlobInstcs::prefsPtr));
 
 	return true;
 }
