@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -155,7 +155,8 @@ void openssl_mutexes_init(void)
 	}
 
 	for (i = 0; i < openssl_mutex_count; ++i) {
-		mutex = new(std::nothrow) QMutex(QMutex::NonRecursive);
+		/* Default in Qt5 is QMutex::NonRecursive. */
+		mutex = new (::std::nothrow) QMutex();
 		openssl_mutexes.append(mutex);
 	}
 
