@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,31 +21,22 @@
  * the two.
  */
 
-
-#ifndef _CRYPTO_THREADS_H_
-#define _CRYPTO_THREADS_H_
-
+#pragma once
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-
 /*!
  * @brief Initialise cryptographic back-end for multi-threaded use.
  */
 void crypto_init_threads(void);
 
-
 /*!
  * @brief Perform clean-up of thread-related structures.
  */
 void crypto_cleanup_threads(void);
 
-
 #ifdef __cplusplus
 } /* extern "C" */
 #endif
-
-
-#endif /* _CRYPTO_THREADS_H_ */
