@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,28 +21,32 @@
  * the two.
  */
 
-#include <algorithm> /* std::sort */
+#include <algorithm> /* ::std::sort, ::std::upper_bound */
 #include <QPainter>
+#include <QPainterPath>
 
+#include "src/datovka_shared/json/basic.h"
+#include "src/datovka_shared/localisation/localisation.h"
+#include "src/datovka_shared/log/log.h"
 #include "src/delegates/tag_item.h"
 #include "src/dimensions/dimensions.h"
-#include "src/localisation/localisation.h"
-#include "src/log/log.h"
 
 TagItem::TagItem(void)
-    : TagDb::TagEntry()
+    : Json::TagEntry()
 {
 }
 
-TagItem::TagItem(const TagDb::TagEntry &entry)
-    : TagDb::TagEntry(entry)
+TagItem::TagItem(const Json::TagEntry &entry)
+    : Json::TagEntry(entry)
 {
 }
 
-int TagItem::paint(class QPainter *painter,
-    const QStyleOptionViewItem &option) const
+int TagItem::paint(QPainter *painter, const QStyleOptionViewItem &option) const
 {
-	Q_ASSERT(0 != painter);
+	if (Q_UNLIKELY(Q_NULLPTR == painter)) {
+		Q_ASSERT(0);
+		return 0;
+	}
 
 	const QRect &rect(option.rect);
 
@@ -71,11 +75,11 @@ int TagItem::paint(class QPainter *painter,
 
 	painter->translate(margin, 0);
 
-	QColor rectColour("#" + colour);
-	if (!rectColour.isValid()) {
+	QColor rectColour("#" + colour());
+	if (Q_UNLIKELY(!rectColour.isValid())) {
 		logWarningNL("Invalid tag colour '%s'. Using 'ffffff'",
-		    colour.toUtf8().constData());
-		rectColour = QColor("#ffffff");
+		    colour().toUtf8().constData());
+		rectColour = QColor("#ffffff"); /* Should be valid now. */
 		Q_ASSERT(rectColour.isValid());
 	}
 
@@ -87,7 +91,7 @@ int TagItem::paint(class QPainter *painter,
 	/* TODO -- Obtain foreground colour. */
 	painter->setPen(
 	    QPen(adjustForegroundColour(Qt::black, rectColour), 1));
-	painter->drawText(drawnRect, Qt::AlignCenter, name);
+	painter->drawText(drawnRect, Qt::AlignCenter, name());
 
 	painter->restore();
 
@@ -101,7 +105,12 @@ QSize TagItem::sizeHint(const QStyleOptionViewItem &option) const
 	int height = QFontMetrics(font).height();
 	const int padding = Dimensions::padding(height);
 
-	int width = QFontMetrics(font).width(name);
+	/*
+	 * Stopped using deprecated QFontMetrics::width().
+	 * https://kdepepo.wordpress.com/2019/08/05/about-deprecation-of-qfontmetricswidth/
+	 * https://doc.qt.io/qt-5/qfontmetrics-obsolete.html#width
+	 */
+	int width = QFontMetrics(font).boundingRect(name()).width();
 	width += 2 * padding;
 
 	height += 2 * padding;
@@ -132,25 +141,30 @@ TagItemList::TagItemList(void)
 {
 }
 
-TagItemList::TagItemList(const QList<TagDb::TagEntry> &tagList)
+TagItemList::TagItemList(const Json::TagEntryList &tagList)
     : QList<TagItem>()
 {
-	foreach (const TagDb::TagEntry entry, tagList) {
+	for (const Json::TagEntry &entry : tagList) {
 		QList<TagItem>::append(TagItem(entry));
 	}
 }
 
-void TagItemList::paint(class QPainter *painter,
+TagItemList::TagItemList(const QList<TagItem> &tagList)
+    : QList<TagItem>(tagList)
+{
+}
+
+void TagItemList::paint(QPainter *painter,
     const QStyleOptionViewItem &option) const
 {
-	if (0 == painter) {
+	if (Q_UNLIKELY(Q_NULLPTR == painter)) {
 		Q_ASSERT(0);
 		return;
 	}
 
 	painter->save();
 
-	foreach (const TagItem &tag, *this) {
+	for (const TagItem &tag : *this) {
 		int width = tag.paint(painter, option);
 		painter->translate(width, 0);
 	}
@@ -163,7 +177,7 @@ QSize TagItemList::sizeHint(const QStyleOptionViewItem &option) const
 	int width = 0;
 	const int margin = Dimensions::margin(option);
 
-	foreach (const TagItem &tag, *this) {
+	for (const TagItem &tag : *this) {
 		width += tag.sizeHint(option).width();
 		width += 2 * margin;
 	}
@@ -173,18 +187,86 @@ QSize TagItemList::sizeHint(const QStyleOptionViewItem &option) const
 	return QSize(width, 1);
 }
 
+bool TagItemList::updateTag(const Json::TagEntry &entry)
+{
+	int pos = indexOfId(entry.id());
+	if (pos < 0) {
+		return false;
+	}
+
+	this->operator[](pos) = TagItem(entry);
+	return true;
+}
+
+bool TagItemList::deleteTags(const Json::Int64StringList &ids)
+{
+	if (this->isEmpty()) {
+		return false;
+	}
+
+	bool removed = false;
+
+	for (qint64 id : ids) {
+		int pos = indexOfId(id);
+		if (pos < 0) {
+			continue;
+		}
+
+		this->removeAt(pos);
+		removed = true;
+		if (this->isEmpty()) {
+			break;
+		}
+	}
+
+	return removed;
+}
+
 /*!
  * @brief Used for sorting tag lists.
+ *
+ * @param[in] a First tag item.
+ * @param[in] b Second tag item.
+ * @return True if \a a comes before \a b.
  */
-class TagItemLess {
-public:
-	bool operator()(const TagItem &a, const TagItem &b) const
-	{
-		return Localisation::stringCollator.compare(a.name, b.name) < 0;
-	}
-};
+static
+bool tagItemLessName(const TagItem &a, const TagItem &b)
+{
+	return Localisation::stringCollator.compare(a.name(), b.name()) < 0;
+}
 
 void TagItemList::sortNames(void)
 {
-	::std::sort(this->begin(), this->end(), TagItemLess());
+	::std::sort(this->begin(), this->end(), tagItemLessName);
+}
+
+TagItemList::iterator TagItemList::upperBoundName(const TagItem &item)
+{
+	return ::std::upper_bound(this->begin(), this->end(), item, tagItemLessName);
+}
+
+int TagItemList::indexOfId(qint64 tagId) const
+{
+	for (int i = 0; i < this->size(); ++i) {
+		if (this->at(i).id() == tagId) {
+			return i;
+		}
+	}
+
+	return -1;
+}
+
+bool TagItemList::containsId(qint64 tagId) const
+{
+	/*
+	 * Cannot use ::std::lower_bound here because the list is not sorted
+	 * according to identifiers.
+	 */
+	for (const TagItem &item : *this) {
+		if (item.id() == tagId) {
+			return true;
+		}
+	}
+
+	return false;
 }
