@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,10 +21,13 @@
  * the two.
  */
 
-#ifndef _DIMENSIONS_H_
-#define _DIMENSIONS_H_
+#pragma once
 
 #include <QStyleOptionViewItem>
+#include <QVector>
+
+class QTableView; /* Forward declaration. */
+class QSplitter; /* Forward declaration. */
 
 /*!
  * @brief Defines some basic dimensions.
@@ -61,18 +64,22 @@ public:
 
 	/*!
 	 * @brief Return size of available space on desktop.
+	 *
+	 * @param[in] widget Widget used to determine the screen.
 	 * @note This excludes some control elements (e.g. dock, menu bar,
 	 *     task bar). This may not work when using X11.
 	 */
 	static
-	QRect availableScreenSize(void);
+	QRect availableScreenSize(const QWidget *widget);
 
 	/*!
 	 * @brief Return screen size.
+	 *
+	 * @param[in] widget Widget used to determine the screen.
 	 * @return Screen size.
 	 */
 	static
-	QRect screenSize(void);
+	QRect screenSize(const QWidget *widget);
 
 	/*!
 	 * @brief Returns size of dialogue specified as the ratio of the
@@ -110,6 +117,93 @@ public:
 	static
 	QRect windowOnScreenDimensions(const QWidget *widget);
 
+	/*!
+	 * @brief Returns preferred size of dialogue.
+	 *     If the screen is too small or the screen cannot be determined
+	 *     or \a newSize is invalid then \a dfltSize is returned.
+	 *
+	 * @param[in] widget Window widget.
+	 * @param[in] newSize New size.
+	 * @param[in] dfltSize Default/current size.
+	 * @return Dialogue window size.
+	 */
+	static
+	QSize dialogueSize(const QWidget *widget, const QSize &newSize,
+	    const QSize &dfltSize);
+
+	/*!
+	 * @brief Returns table column widths as ration to the table width.
+	 *
+	 * @param[in] table Table view.
+	 * @return Respective column to table width ratios.
+	 */
+	static
+	QVector<double> relativeTableColumnWidths(const QTableView *table);
+
+	/*!
+	 * @brief Set table columns width.
+	 *
+	 * @param[in,out] table Table view.
+	 * @param[in] relativeWidths Relative column widths.
+	 */
+	static
+	void setRelativeTableColumnWidths(QTableView *table,
+	    const QVector<double> &relativeWidths);
+
+	/*!
+	 * @brief Returns table column widths.
+	 *
+	 * @param[in] table Table view.
+	 * @return Table column widths.
+	 */
+	static
+	QVector<qint64> tableColumnWidths(const QTableView *table);
+
+	/*!
+	 * @brief Set table columns width.
+	 *
+	 * @param[in,out] table Table view.
+	 * @param[in] widths Column widths.
+	 */
+	static
+	void setTableColumnWidths(QTableView *table, const QVector<qint64> &widths);
+
+	/*!
+	 * @brief Get splitter size.
+	 *
+	 * @param[in,out] splitter Splitter.
+	 */
+	static
+	int splitterSize(const QSplitter *splitter);
+
+	/*!
+	 * @brief Set splitter size.
+	 *
+	 * @param[in,out] splitter Splitter.
+	 * @param[in] paneSize First pane size.
+	 */
+	static
+	void setSplitterSize(QSplitter *splitter, int paneSize);
+
+	/*!
+	 * @brief Returns table colum sort order.
+	 *
+	 * @param[in] table Table view.
+	 * @return Column index and sort order value.
+	 */
+	static
+	QVector<int> tableColumnSortOrder(const QTableView *table);
+
+	/*!
+	 * @brief Set table column sort order.
+	 *
+	 * @param[in,out] table Table view.
+	 * @param[in] columnAndOrder Index and sort order value.
+	 */
+	static
+	void setTableColumnSortOrder(QTableView *table,
+	    const QVector<int> &columnAndOrder);
+
 private:
 	/*!
 	 * @brief Constructor.
@@ -133,5 +227,3 @@ private:
 	static
 	const QRect m_dfltWinRect; /*!< Default window rectangle. */
 };
-
-#endif /* _DIMENSIONS_H_ */
