@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -24,20 +24,29 @@
 #include <QFileDialog>
 #include <QMessageBox>
 
-#include "src/common.h"
+#include "src/datovka_shared/identifiers/account_id.h"
+#include "src/datovka_shared/log/log.h"
+#include "src/dimensions/dimensions.h"
+#include "src/global.h"
 #include "src/gui/dlg_account_from_db.h"
-#include "src/log/log.h"
+#include "src/io/message_db_set.h"
+#include "src/settings/account.h"
+#include "src/settings/accounts.h"
+#include "src/settings/preferences.h"
+#include "src/settings/prefs_specific.h"
 #include "ui_dlg_account_from_db.h"
 
 DlgCreateAccountFromDb::DlgCreateAccountFromDb(QWidget *parent)
     : QDialog(parent),
-    m_ui(new (std::nothrow) Ui::DlgCreateAccountFromDb)
+    m_ui(new (::std::nothrow) Ui::DlgCreateAccountFromDb)
 {
 	m_ui->setupUi(this);
+	/* Tab order is defined in UI file. */
+
 	m_ui->info->setText(tr(
-	    "A new account will be created according to the name and the content of the database file. "
-	    "This account will operate over the selected database. "
-	    "Should such an account or database file already exist in Datovka then the association will fail. "
+	    "A new data box will be created according to the name and the content of the database file. "
+	    "This data box will operate over the selected database. "
+	    "Should such a data box or database file already exist in Datovka then the association will fail. "
 	    "During the association no database file copy is created nor is the content of the database file modified. "
 	    "Nevertheless, we strongly advice you to back-up all important files before associating a database file. "
 	    "In order for the association to succeed you will need an active connection to the ISDS server."));
@@ -117,29 +126,21 @@ QStringList getDatabaseFilesFromLocation(bool fromDirectory,
 /*!
  * @brief Create accounts from supplied database files.
  *
- * @param[in,out] accountModel Account model to add account data into.
+ * @param[in,out] regularAccounts Regular account container to add account into.
  * @param[in]     filePathList List of database file paths.
  * @param[in]     parent Parent widget.
- * @return List of user names of newly created accounts.
+ * @return List of account identifiers of newly created accounts.
  */
 static
-QStringList createAccountsFromDatabaseFiles(AccountModel &accountModel,
+QList<AcntId> createAccountsFromDatabaseFiles(AccountsMap &regularAccounts,
     const QStringList &filePathList, QWidget *parent = Q_NULLPTR)
 {
 	if (filePathList.isEmpty()) {
-		return QStringList();
+		return QList<AcntId>();
 	}
 
-	QStringList accountUserNames;
-	const int accountCount = accountModel.rowCount();
-	for (int i = 0; i < accountCount; ++i) {
-		const QModelIndex index(accountModel.index(i, 0));
-		const QString userName(accountModel.userName(index));
-		Q_ASSERT(!userName.isEmpty());
-		accountUserNames.append(userName);
-	}
-
-	QStringList createdAccountUserNames;
+	QList<AcntId> createdAccountIds;
+	QSet<AcntId> processedAcntIds;
 
 	foreach (const QString &filePath, filePathList) {
 		const QString dbFileName(QFileInfo(filePath).fileName());
@@ -149,10 +150,10 @@ QStringList createAccountsFromDatabaseFiles(AccountModel &accountModel,
 		QString errMsg;
 
 		/* Split and check the database file name. */
-		if (!isValidDatabaseFileName(dbFileName, dbUserName,
+		if (!MessageDbSet::isValidDbFileName(dbFileName, dbUserName,
 		    dbYearFlag, dbTestingFlag, errMsg)) {
 			QMessageBox::warning(parent,
-			    DlgCreateAccountFromDb::tr("Create account: %1")
+			    DlgCreateAccountFromDb::tr("Create data box: %1")
 			        .arg(dbUserName),
 			    DlgCreateAccountFromDb::tr("File") + ": " +
 			        filePath + "\n\n" + errMsg,
@@ -160,21 +161,23 @@ QStringList createAccountsFromDatabaseFiles(AccountModel &accountModel,
 			continue;
 		}
 
+		const AcntId acntId(dbUserName, dbTestingFlag);
+
+		/* Skip other year databases of same account. */
+		if (!processedAcntIds.contains(acntId)) {
+			processedAcntIds.insert(acntId);
+		} else {
+			continue;
+		}
+
 		/* Check whether account already exists. */
-		bool exists = false;
-		for (int j = 0; j < accountCount; ++j) {
-			if (accountUserNames.at(j) == dbUserName) {
-				exists = true;
-				break;
-			}
-		}
-		if (exists) {
+		if (regularAccounts.acntIds().contains(acntId)) {
 			errMsg = DlgCreateAccountFromDb::tr(
-			    "Account with user name '%1' and its message database already exist. "
-			    "New account was not created and selected database file was not associated with this account.")
+			    "Data box with username '%1' and its message database already exist. "
+			    "New data box was not created and selected database file was not associated with this data box.")
 			        .arg(dbUserName);
 			QMessageBox::warning(parent,
-			    DlgCreateAccountFromDb::tr("Create account: %1")
+			    DlgCreateAccountFromDb::tr("Create data box: %1")
 			        .arg(dbUserName),
 			    DlgCreateAccountFromDb::tr("File") + ": " +
 			        filePath + "\n\n" + errMsg,
@@ -182,7 +185,7 @@ QStringList createAccountsFromDatabaseFiles(AccountModel &accountModel,
 			continue;
 		}
 
-		AcntSettings itemSettings;
+		AcntData itemSettings;
 		itemSettings.setTestAccount(dbTestingFlag);
 		itemSettings.setAccountName(dbUserName);
 		itemSettings.setUserName(dbUserName);
@@ -190,30 +193,31 @@ QStringList createAccountsFromDatabaseFiles(AccountModel &accountModel,
 		itemSettings.setPassword("");
 		itemSettings.setRememberPwd(false);
 		itemSettings.setSyncWithAll(false);
-		itemSettings.setDbDir(absoluteDirPath(filePath));
-		accountModel.addAccount(itemSettings);
+		itemSettings.setDbDir(absoluteDirPath(filePath),
+		    GlobInstcs::iniPrefsPtr->confDir());
+		regularAccounts.addAccount(itemSettings);
 		errMsg = DlgCreateAccountFromDb::tr(
-		        "Account with name '%1' has been created (user name '%1').")
+		        "Data box with name '%1' has been created (username '%1').")
 		        .arg(dbUserName) + " " +
 		    DlgCreateAccountFromDb::tr(
-		        "This database file has been set as the actual message database for this account. "
-		        "You'll probably have to modify the account properties in order to log in to the ISDS server correctly.");
+		        "This database file has been set as the actual message database for this data box. "
+		        "You'll probably have to modify the data-box properties in order to log in to the ISDS server correctly.");
 
 		QMessageBox::information(parent,
-		    DlgCreateAccountFromDb::tr("Create account: %1")
+		    DlgCreateAccountFromDb::tr("Create data box: %1")
 		        .arg(dbUserName),
 		    DlgCreateAccountFromDb::tr("File") + ": " +
 		        filePath + "\n\n" + errMsg,
 		    QMessageBox::Ok);
 
-		createdAccountUserNames.append(dbUserName);
+		createdAccountIds.append(acntId);
 	}
 
-	return createdAccountUserNames;
+	return createdAccountIds;
 }
 
-QStringList DlgCreateAccountFromDb::createAccount(AccountModel &accountModel,
-    QString &lastImportDir, QWidget *parent)
+QList<AcntId> DlgCreateAccountFromDb::createAccount(
+    AccountsMap &regularAccounts, QString &lastImportDir, QWidget *parent)
 {
 	QStringList dbFilePathList;
 
@@ -228,10 +232,10 @@ QStringList DlgCreateAccountFromDb::createAccount(AccountModel &accountModel,
 		break;
 	default:
 		/* Do nothing. */
-		return QStringList();
+		return QList<AcntId>();
 	}
 
-	return createAccountsFromDatabaseFiles(accountModel, dbFilePathList,
+	return createAccountsFromDatabaseFiles(regularAccounts, dbFilePathList,
 	    parent);
 }
 
@@ -239,7 +243,24 @@ enum DlgCreateAccountFromDb::Action DlgCreateAccountFromDb::chooseAction(
     QWidget *parent)
 {
 	DlgCreateAccountFromDb dlg(parent);
-	if (QDialog::Accepted == dlg.exec()) {
+
+	const QString dlgName("account_from_db");
+	const QSize dfltSize = dlg.size();
+	{
+		const QSize newSize = Dimensions::dialogueSize(&dlg,
+		    PrefsSpecific::dlgSize(*GlobInstcs::prefsPtr, dlgName),
+		    dfltSize);
+		if (newSize.isValid()) {
+			dlg.resize(newSize);
+		}
+	}
+
+	int ret = dlg.exec();
+
+	PrefsSpecific::setDlgSize(*GlobInstcs::prefsPtr, dlgName,
+	    dlg.size(), dfltSize);
+
+	if (QDialog::Accepted == ret) {
 		return dlg.m_ui->directory->isChecked() ? ACT_FROM_DIRECTORY :
 		    ACT_FROM_FILES;
 	} else {
