@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -24,19 +24,24 @@
 #include <QFileDialog>
 #include <QMessageBox>
 
+#include "src/datovka_shared/identifiers/account_id.h"
+#include "src/datovka_shared/log/log.h"
+#include "src/dimensions/dimensions.h"
+#include "src/global.h"
 #include "src/gui/dlg_change_directory.h"
 #include "src/io/message_db_set.h"
-#include "src/log/log.h"
 #include "src/settings/accounts.h"
 #include "src/settings/preferences.h"
+#include "src/settings/prefs_specific.h"
 #include "ui_dlg_change_directory.h"
 
 DlgChangeDirectory::DlgChangeDirectory(const QString &currentDir,
     QWidget *parent)
     : QDialog(parent),
-    m_ui(new (std::nothrow) Ui::DlgChangeDirectory)
+    m_ui(new (::std::nothrow) Ui::DlgChangeDirectory)
 {
 	m_ui->setupUi(this);
+	/* Tab order is defined in UI file. */
 
 	m_ui->newPath->setText("");
 	m_ui->currentPath->setText(currentDir);
@@ -62,10 +67,10 @@ DlgChangeDirectory::~DlgChangeDirectory(void)
 	delete m_ui;
 }
 
-bool DlgChangeDirectory::changeDataDirectory(const QString &userName,
+bool DlgChangeDirectory::changeDataDirectory(const AcntId &acntId,
     MessageDbSet *dbSet, QWidget *parent)
 {
-	if (Q_UNLIKELY(userName.isEmpty() || (dbSet == Q_NULLPTR))) {
+	if (Q_UNLIKELY((!acntId.isValid()) || (dbSet == Q_NULLPTR))) {
 		Q_ASSERT(0);
 		return false;
 	}
@@ -74,12 +79,12 @@ bool DlgChangeDirectory::changeDataDirectory(const QString &userName,
 	QString newDirPath;
 
 	/* Get current settings. */
-	const AcntSettings &itemSettings(globAccounts[userName]);
+	const AcntData itemSettings(GlobInstcs::acntMapPtr->acntData(acntId));
 
 	QString oldDbDir(itemSettings.dbDir());
 	if (oldDbDir.isEmpty()) {
 		/* Set default directory name. */
-		oldDbDir = globPref.confDir();
+		oldDbDir = GlobInstcs::iniPrefsPtr->confDir();
 	}
 
 	if (!chooseAction(oldDbDir, newDirPath, action, parent)) {
@@ -91,7 +96,7 @@ bool DlgChangeDirectory::changeDataDirectory(const QString &userName,
 		return false;
 	}
 
-	return relocateDatabase(userName, dbSet, oldDbDir, newDirPath, action,
+	return relocateDatabase(acntId, dbSet, oldDbDir, newDirPath, action,
 	    parent);
 }
 
@@ -118,7 +123,24 @@ bool DlgChangeDirectory::chooseAction(const QString &currentDir,
     QString &newDir, enum Action &action, QWidget *parent)
 {
 	DlgChangeDirectory dlg(currentDir, parent);
-	if (QDialog::Accepted == dlg.exec()) {
+
+	const QString dlgName("change_directory");
+	const QSize dfltSize = dlg.size();
+	{
+		const QSize newSize = Dimensions::dialogueSize(&dlg,
+		    PrefsSpecific::dlgSize(*GlobInstcs::prefsPtr, dlgName),
+		    dfltSize);
+		if (newSize.isValid()) {
+			dlg.resize(newSize);
+		}
+	}
+
+	int ret = dlg.exec();
+
+	PrefsSpecific::setDlgSize(*GlobInstcs::prefsPtr, dlgName,
+	    dlg.size(), dfltSize);
+
+	if (QDialog::Accepted == ret) {
 		newDir = dlg.m_ui->newPath->text();
 		if (dlg.m_ui->moveDataRadioButton->isChecked()) {
 			action = ACT_MOVE;
@@ -133,66 +155,70 @@ bool DlgChangeDirectory::chooseAction(const QString &currentDir,
 	}
 }
 
-bool DlgChangeDirectory::relocateDatabase(const QString &userName,
+bool DlgChangeDirectory::relocateDatabase(const AcntId &acntId,
     MessageDbSet *dbSet, const QString &oldDir, const QString &newDir,
     enum Action action, QWidget *parent)
 {
-	if (Q_UNLIKELY(userName.isEmpty()) || (dbSet == Q_NULLPTR)) {
+	if (Q_UNLIKELY((!acntId.isValid()) || (dbSet == Q_NULLPTR))) {
 		Q_ASSERT(0);
 		return false;
 	}
 
 	/* Get current settings. */
-	AcntSettings &itemSettings(globAccounts[userName]);
+	AcntData itemSettings(GlobInstcs::acntMapPtr->acntData(acntId));
 
 	switch (action) {
 	case ACT_MOVE:
 		/* Move account database into new directory. */
 		if (dbSet->moveToLocation(newDir)) {
-			itemSettings.setDbDir(newDir);
+			itemSettings.setDbDir(newDir,
+			    GlobInstcs::iniPrefsPtr->confDir());
 
 			logInfo("Database files for '%s' have been moved from '%s' to '%s'.\n",
-			    userName.toUtf8().constData(),
-			    oldDir.toUtf8().constData(),
-			    newDir.toUtf8().constData());
+			    acntId.username().toUtf8().constData(),
+			    QDir::toNativeSeparators(oldDir).toUtf8().constData(),
+			    QDir::toNativeSeparators(newDir).toUtf8().constData());
 
 			QMessageBox::information(parent,
-			    tr("Change data directory for current account"),
+			    tr("Change data directory for current data box"),
 			    tr("Database files for '%1' have been successfully moved to\n\n'%2'.")
-			        .arg(userName).arg(newDir),
+			        .arg(acntId.username()).arg(QDir::toNativeSeparators(newDir)),
 			    QMessageBox::Ok);
 
+			GlobInstcs::acntMapPtr->updateAccount(itemSettings);
 			return true;
 		} else {
 			QMessageBox::critical(parent,
-			    tr("Change data directory for current account"),
+			    tr("Change data directory for current data box"),
 			    tr("Database files for '%1' could not be moved to\n\n'%2'.")
-			        .arg(userName).arg(newDir),
+			        .arg(acntId.username()).arg(QDir::toNativeSeparators(newDir)),
 			    QMessageBox::Ok);
 		}
 		break;
 	case ACT_COPY:
 		/* Copy account database into new directory. */
 		if (dbSet->copyToLocation(newDir)) {
-			itemSettings.setDbDir(newDir);
+			itemSettings.setDbDir(newDir,
+			    GlobInstcs::iniPrefsPtr->confDir());
 
 			logInfo("Database files for '%s' have been copied from '%s' to '%s'.\n",
-			    userName.toUtf8().constData(),
-			    oldDir.toUtf8().constData(),
-			    newDir.toUtf8().constData());
+			    acntId.username().toUtf8().constData(),
+			    QDir::toNativeSeparators(oldDir).toUtf8().constData(),
+			    QDir::toNativeSeparators(newDir).toUtf8().constData());
 
 			QMessageBox::information(parent,
-			    tr("Change data directory for current account"),
+			    tr("Change data directory for current data box"),
 			    tr("Database files for '%1' have been successfully copied to\n\n'%2'.")
-			        .arg(userName).arg(newDir),
+			        .arg(acntId.username()).arg(QDir::toNativeSeparators(newDir)),
 			    QMessageBox::Ok);
 
+			GlobInstcs::acntMapPtr->updateAccount(itemSettings);
 			return true;
 		} else {
 			QMessageBox::critical(parent,
-			    tr("Change data directory for current account"),
+			    tr("Change data directory for current data box"),
 			    tr("Database files for '%1' could not be copied to\n\n'%2'.")
-			        .arg(userName).arg(newDir),
+			        .arg(acntId.username()).arg(QDir::toNativeSeparators(newDir)),
 			    QMessageBox::Ok);
 		}
 		break;
@@ -200,24 +226,26 @@ bool DlgChangeDirectory::relocateDatabase(const QString &userName,
 		/* Create a new account database into new directory. */
 		if (dbSet->reopenLocation(newDir, MessageDbSet::DO_YEARLY,
 		        MessageDbSet::CM_CREATE_EMPTY_CURRENT)) {
-			itemSettings.setDbDir(newDir);
+			itemSettings.setDbDir(newDir,
+			    GlobInstcs::iniPrefsPtr->confDir());
 
 			logInfo("Database files for '%s' have been created in '%s'.\n",
-			    userName.toUtf8().constData(),
-			    newDir.toUtf8().constData());
+			    acntId.username().toUtf8().constData(),
+			    QDir::toNativeSeparators(newDir).toUtf8().constData());
 
 			QMessageBox::information(parent,
-			    tr("Change data directory for current account"),
+			    tr("Change data directory for current data box"),
 			    tr("New database files for '%1' have been successfully created in\n\n'%2'.")
-			        .arg(userName).arg(newDir),
+			        .arg(acntId.username()).arg(QDir::toNativeSeparators(newDir)),
 			    QMessageBox::Ok);
 
+			GlobInstcs::acntMapPtr->updateAccount(itemSettings);
 			return true;
 		} else {
 			QMessageBox::critical(parent,
-			    tr("Change data directory for current account"),
+			    tr("Change data directory for current data box"),
 			    tr("New database files for '%1' could not be created in\n\n'%2'.")
-			        .arg(userName).arg(newDir),
+			        .arg(acntId.username()).arg(QDir::toNativeSeparators(newDir)),
 			    QMessageBox::Ok);
 		}
 		break;
