@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,12 +21,12 @@
  * the two.
  */
 
-#ifndef _DLG_CHANGE_DIRECTORY_H_
-#define _DLG_CHANGE_DIRECTORY_H_
+#pragma once
 
 #include <QDialog>
 #include <QString>
 
+class AcntId; /* Forward declaration. */
 class MessageDbSet; /* Forward declaration. */
 
 namespace Ui {
@@ -58,13 +58,13 @@ public:
 	/*!
 	 * @brief Asks the user for the new database location and relocates it.
 	 *
-	 * @param[in] userName User name identifying related account.
+	 * @param[in] acntId Account identifier.
 	 * @param[in] dbSet Affected database set.
 	 * @param[in] parent Parent widget.
 	 * @return True on success.
 	 */
 	static
-	bool changeDataDirectory(const QString &userName, MessageDbSet *dbSet,
+	bool changeDataDirectory(const AcntId &acntId, MessageDbSet *dbSet,
 	    QWidget *parent = Q_NULLPTR);
 
 private slots:
@@ -100,7 +100,7 @@ private:
 	/*!
 	 * @brief Performs a database relocation.
 	 *
-	 * @param[in] userName User name identifying related account.
+	 * @param[in] acntId Account identifier.
 	 * @param[in] dbSet Affected database set.
 	 * @param[in] oldDir Old location.
 	 * @param[in] newDir New location.
@@ -109,11 +109,9 @@ private:
 	 * @return True on success.
 	 */
 	static
-	bool relocateDatabase(const QString &userName, MessageDbSet *dbSet,
+	bool relocateDatabase(const AcntId &acntId, MessageDbSet *dbSet,
 	    const QString &oldDir, const QString &newDir, enum Action action,
 	    QWidget *parent = Q_NULLPTR);
 
 	Ui::DlgChangeDirectory *m_ui; /*!< UI generated from UI file. */
 };
-
-#endif /* _DLG_CHANGE_DIRECTORY_H_ */
