@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -27,83 +27,129 @@
 #include <QPushButton>
 #include <QTextStream>
 
+#include "src/datovka_shared/identifiers/account_id.h"
+#include "src/datovka_shared/settings/prefs.h"
+#include "src/datovka_shared/utility/date_time.h"
 #include "src/delegates/tag_item.h"
+#include "src/dimensions/dimensions.h"
+#include "src/global.h"
 #include "src/gui/dlg_correspondence_overview.h"
-#include "src/gui/dlg_msg_box_informative.h"
+#include "src/gui/dlg_msg_box_detail.h"
+#include "src/gui/styles.h"
+#include "src/identifiers/message_id.h"
 #include "src/io/exports.h"
 #include "src/io/filesystem.h"
+#include "src/io/message_db_set.h"
+#include "src/io/tag_container.h"
+#include "src/json/tag_assignment_hash.h"
+#include "src/json/tag_message_id.h"
 #include "src/settings/accounts.h"
-#include "src/settings/preferences.h"
+#include "src/settings/prefs_specific.h"
+#include "ui_dlg_correspondence_overview.h"
 
 #define CSV_LITERAL QStringLiteral("CSV")
 #define HTML_LITERAL QStringLiteral("HTML")
 
 DlgCorrespondenceOverview::DlgCorrespondenceOverview(const MessageDbSet &dbSet,
-    const QString &dbId, const QString &userName, TagDb &tagDb, QWidget *parent)
+    const QString &dbId, const AcntId &acntId, TagContainer &tagCont,
+    QWidget *parent)
     : QDialog(parent),
+    m_ui(new (::std::nothrow) Ui::DlgCorrespondenceOverview),
     m_messDbSet(dbSet),
     m_dbId(dbId),
-    m_tagDb(tagDb),
+    m_tagCont(tagCont),
     m_exportedMsgs()
 {
-	setupUi(this);
+	m_ui->setupUi(this);
+	/* Tab order is defined in UI file. */
 
-	Q_ASSERT(!userName.isEmpty());
+	Q_ASSERT(acntId.isValid());
 
-	this->accountName->setText(globAccounts[userName].accountName() +
-	    QStringLiteral(" (") + userName + QStringLiteral(")"));
+	m_ui->accountName->setText(
+	    GlobInstcs::acntMapPtr->acntData(acntId).accountName() +
+	    QStringLiteral(" (") + acntId.username() + QStringLiteral(")"));
 
-	this->toCalendarWidget->setMinimumDate(this->fromCalendarWidget->selectedDate());
+	m_ui->toCalendarWidget->setMinimumDate(
+	    m_ui->fromCalendarWidget->selectedDate());
 
-	QDate currentDate(QDate().currentDate());
-	this->toCalendarWidget->setMaximumDate(currentDate);
-	this->fromCalendarWidget->setMaximumDate(currentDate);
+	{
+		const QDate currentDate(QDate::currentDate());
+		m_ui->toCalendarWidget->setMaximumDate(currentDate);
+		m_ui->fromCalendarWidget->setMaximumDate(currentDate);
+	}
 
-	this->outputFormatComboBox->addItem(CSV_LITERAL);
-	this->outputFormatComboBox->addItem(HTML_LITERAL);
+	m_ui->outputFormatComboBox->addItem(CSV_LITERAL);
+	m_ui->outputFormatComboBox->addItem(HTML_LITERAL);
 
-	connect(this->outputFormatComboBox, SIGNAL(currentIndexChanged(QString)),
-	    this, SLOT(reftectOverviewTypeChange(QString)));
+	/* Days in week and week number format. */
+	GuiStyles::setCalendarStyle(m_ui->fromCalendarWidget);
+	GuiStyles::setCalendarStyle(m_ui->toCalendarWidget);
 
-	connect(this->fromCalendarWidget, SIGNAL(clicked(QDate)),
-	    this, SLOT(reftectCalendarChange()));
+	connect(m_ui->outputFormatComboBox, SIGNAL(currentIndexChanged(int)),
+	    this, SLOT(reflectOverviewTypeChange(int)));
 
-	connect(this->toCalendarWidget, SIGNAL(clicked(QDate)),
-	    this, SLOT(reftectCalendarChange()));
+	connect(m_ui->fromCalendarWidget, SIGNAL(clicked(QDate)),
+	    this, SLOT(reflectCalendarChange()));
 
-	connect(this->sentCheckBox, SIGNAL(stateChanged(int)),
+	connect(m_ui->toCalendarWidget, SIGNAL(clicked(QDate)),
+	    this, SLOT(reflectCalendarChange()));
+
+	connect(m_ui->sentCheckBox, SIGNAL(stateChanged(int)),
 	    this, SLOT(checkMsgTypeSelection()));
 
-	connect(this->receivedCheckBox, SIGNAL(stateChanged(int)),
+	connect(m_ui->receivedCheckBox, SIGNAL(stateChanged(int)),
 	    this, SLOT(checkMsgTypeSelection()));
 
-	this->groupBox->setEnabled(false);
+	m_ui->groupBox->setEnabled(false);
 
-	updateExportedMsgList(this->fromCalendarWidget->selectedDate(),
-	    this->toCalendarWidget->selectedDate());
+	updateExportedMsgList(m_ui->fromCalendarWidget->selectedDate(),
+	    m_ui->toCalendarWidget->selectedDate());
 	updateOkButtonActivity();
 }
 
-void DlgCorrespondenceOverview::exportData(const MessageDbSet &dbSet,
-    const QString &dbId, const QString &userName, TagDb &tagDb,
-    QString &exportCorrespondDir, QWidget *parent)
+DlgCorrespondenceOverview::~DlgCorrespondenceOverview(void)
 {
-	if (userName.isEmpty()) {
+	delete m_ui;
+}
+
+void DlgCorrespondenceOverview::exportData(const MessageDbSet &dbSet,
+    const QString &dbId, const AcntId &acntId, TagContainer &tagCont,
+    Prefs &prefs, QWidget *parent)
+{
+	if (Q_UNLIKELY(!acntId.isValid())) {
 		Q_ASSERT(0);
 		return;
 	}
 
-	DlgCorrespondenceOverview dlg(dbSet, dbId, userName, tagDb, parent);
-	if (QDialog::Accepted != dlg.exec()) {
+	DlgCorrespondenceOverview dlg(dbSet, dbId, acntId, tagCont, parent);
+
+	const QString dlgName("correspondence_overview");
+	const QSize dfltSize = dlg.size();
+	{
+		const QSize newSize = Dimensions::dialogueSize(&dlg,
+		    PrefsSpecific::dlgSize(*GlobInstcs::prefsPtr, dlgName),
+		    dfltSize);
+		if (newSize.isValid()) {
+			dlg.resize(newSize);
+		}
+	}
+
+	int ret = dlg.exec();
+
+	PrefsSpecific::setDlgSize(*GlobInstcs::prefsPtr, dlgName,
+	    dlg.size(), dfltSize);
+
+	if (QDialog::Accepted != ret) {
 		return;
 	}
 
-	dlg.exportChosenData(userName, exportCorrespondDir);
+	dlg.exportChosenData(acntId, prefs);
 }
 
-void DlgCorrespondenceOverview::reftectOverviewTypeChange(const QString &text)
+void DlgCorrespondenceOverview::reflectOverviewTypeChange(int index)
 {
-	this->groupBox->setEnabled(text == HTML_LITERAL);
+	m_ui->groupBox->setEnabled(
+	    m_ui->outputFormatComboBox->itemText(index) == HTML_LITERAL);
 }
 
 void DlgCorrespondenceOverview::checkMsgTypeSelection(void)
@@ -111,23 +157,24 @@ void DlgCorrespondenceOverview::checkMsgTypeSelection(void)
 	updateOkButtonActivity();
 }
 
-void DlgCorrespondenceOverview::reftectCalendarChange(void)
+void DlgCorrespondenceOverview::reflectCalendarChange(void)
 {
-	this->toCalendarWidget->setMinimumDate(
-	    this->fromCalendarWidget->selectedDate());
+	m_ui->toCalendarWidget->setMinimumDate(
+	    m_ui->fromCalendarWidget->selectedDate());
+	GuiStyles::setCalendarStyle(m_ui->toCalendarWidget);
 
-	updateExportedMsgList(this->fromCalendarWidget->selectedDate(),
-	    this->toCalendarWidget->selectedDate());
+	updateExportedMsgList(m_ui->fromCalendarWidget->selectedDate(),
+	    m_ui->toCalendarWidget->selectedDate());
 	updateOkButtonActivity();
 }
 
 void DlgCorrespondenceOverview::updateOkButtonActivity(void)
 {
 	/* Enabled the button if there are some messages to be exported. */
-	this->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(
-	    (this->sentCheckBox->isChecked() &&
+	m_ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(
+	    (m_ui->sentCheckBox->isChecked() &&
 	     (m_exportedMsgs.sentDmIDs.count() > 0)) ||
-	    (this->receivedCheckBox->isChecked() &&
+	    (m_ui->receivedCheckBox->isChecked() &&
 	     (m_exportedMsgs.receivedDmIDs.count() > 0)));
 }
 
@@ -135,36 +182,35 @@ void DlgCorrespondenceOverview::updateExportedMsgList(const QDate &fromDate,
     const QDate &toDate)
 {
 	m_exportedMsgs.sentDmIDs = m_messDbSet.msgsDateInterval(fromDate,
-	    toDate, MSG_SENT);
+	    toDate, MessageDb::TYPE_SENT);
 	m_exportedMsgs.receivedDmIDs = m_messDbSet.msgsDateInterval(fromDate,
-	    toDate, MSG_RECEIVED);
+	    toDate, MessageDb::TYPE_RECEIVED);
 
-	this->sentCntLabel->setText(QStringLiteral("(") + tr("messages: ") +
+	m_ui->sentCntLabel->setText(QStringLiteral("(") + tr("messages: ") +
 	    QString::number(m_exportedMsgs.sentDmIDs.count()) +
 	    QStringLiteral(")"));
-	this->receivedCntLabel->setText(QStringLiteral("(") + tr("messages: ") +
+	m_ui->receivedCntLabel->setText(QStringLiteral("(") + tr("messages: ") +
 	    QString::number(m_exportedMsgs.receivedDmIDs.count()) +
 	    QStringLiteral(")"));
 }
 
-QString DlgCorrespondenceOverview::msgCsvEntry(
-    const MessageDb::MsgId &mId) const
+QString DlgCorrespondenceOverview::msgCsvEntry(const MsgId &mId) const
 {
-	if (!mId.isValid()) {
+	if (Q_UNLIKELY(!mId.isValid())) {
 		Q_ASSERT(0);
 		return QString();
 	}
 
-	const MessageDb *messageDb = m_messDbSet.constAccessMessageDb(
-	    mId.deliveryTime);
+	MessageDb *messageDb = m_messDbSet.constAccessMessageDb(
+	    mId.deliveryTime());
 	Q_ASSERT(Q_NULLPTR != messageDb);
 
-	QStringList messageItems(messageDb->getMsgForCsvExport(mId.dmId));
+	QStringList messageItems(messageDb->getMessageForCsvExport(mId.dmId()));
 	if (messageItems.empty()) {
 		return QString();
 	}
 
-	QString content(QString::number(mId.dmId));
+	QString content(QString::number(mId.dmId()));
 
 	for (int i = 0; i < messageItems.count(); ++i) {
 		content += QStringLiteral(",") + messageItems.at(i);
@@ -187,11 +233,11 @@ QString tagHtmlString(const TagItemList &tagList, bool useColours)
 
 	if (!useColours) {
 		foreach (const TagItem &tag, tagList) {
-			tagStrings.append(tag.name);
+			tagStrings.append(tag.name());
 		}
 	} else {
 		foreach (const TagItem &tag, tagList) {
-			QColor bgCol(QStringLiteral("#") + tag.colour);
+			QColor bgCol(QStringLiteral("#") + tag.colour());
 			QColor textCol(
 			    TagItem::adjustForegroundColour(Qt::black, bgCol));
 			QString tagStr(
@@ -201,7 +247,7 @@ QString tagHtmlString(const TagItemList &tagList, bool useColours)
 			tagStr += textCol.name();
 			tagStr += QStringLiteral(";\">");
 			tagStr += QStringLiteral("&nbsp;");
-			tagStr += tag.name;
+			tagStr += tag.name();
 			tagStr += QStringLiteral("&nbsp;");
 			tagStr += QStringLiteral("</span>");
 			tagStrings.append(tagStr);
@@ -214,23 +260,34 @@ QString tagHtmlString(const TagItemList &tagList, bool useColours)
 /*!
  * @brief Creates tag-related HTML entry.
  *
- * @param[in] tagDb Tag database.
- * @param[in] userName User name identifying account.
- * @param[in] msgId Message identifier.
+ * @param[in] tagCont Tag database/client.
+ * @param[in] acntId Account identifier.
+ * @param[in] dmId Message identifier.
  * @param[in] useColours True if coloured tags should be generated.
  * @return Tag entry if some tags found. Return empty string on error or when
  *     no tags found.
  */
 static
-QString tagHtmlEntry(TagDb &tagDb, const QString &userName, qint64 msgId,
+QString tagHtmlEntry(TagContainer &tagCont, const AcntId &acntId, qint64 dmId,
     bool useColours)
 {
-	if (userName.isEmpty() || (msgId < 0)) {
+	if (Q_UNLIKELY((!acntId.isValid()) || (dmId < 0))) {
 		Q_ASSERT(0);
 		return QString();
 	}
 
-	TagItemList tagList(tagDb.getMessageTags(userName, msgId));
+	TagItemList tagList;
+	{
+		const Json::TagMsgId msgId(
+		    acntId.testing() ? Isds::Type::BOOL_TRUE : Isds::Type::BOOL_FALSE,
+		    dmId);
+		Json::TagMsgIdList msgIds({msgId});
+		Json::TagAssignmentHash assignments;
+		if (Q_UNLIKELY(!tagCont.getMessageTags(msgIds, assignments))) {
+			return QString();
+		}
+		tagList = assignments[msgId];
+	}
 	if (tagList.isEmpty()) {
 		return QString();
 	}
@@ -238,12 +295,12 @@ QString tagHtmlEntry(TagDb &tagDb, const QString &userName, qint64 msgId,
 
 	QStringList tagStrings;
 	foreach (const TagItem &tag, tagList) {
-		tagStrings.append(tag.name);
+		tagStrings.append(tag.name());
 	}
 
 	QString retStr(
 	    QStringLiteral("<table><tr><td><table><tr><td valign=\"top\">")
-	    + QObject::tr("Tags") +
+	    + DlgCorrespondenceOverview::tr("Tags") +
 	    QStringLiteral(":</td><td>"));
 	retStr += tagHtmlString(tagList, useColours);
 	retStr += QStringLiteral("</td></tr></table></td></tr></table>");
@@ -251,19 +308,19 @@ QString tagHtmlEntry(TagDb &tagDb, const QString &userName, qint64 msgId,
 	return retStr;
 }
 
-QString DlgCorrespondenceOverview::msgHtmlEntry(const QString &userName,
-    const MessageDb::MsgId &mId) const
+QString DlgCorrespondenceOverview::msgHtmlEntry(const AcntId &acntId,
+    const MsgId &mId) const
 {
-	if (!mId.isValid()) {
+	if (Q_UNLIKELY(!mId.isValid())) {
 		Q_ASSERT(0);
 		return QString();
 	}
 
-	const MessageDb *messageDb = m_messDbSet.constAccessMessageDb(
-	    mId.deliveryTime);
+	MessageDb *messageDb = m_messDbSet.constAccessMessageDb(
+	    mId.deliveryTime());
 	Q_ASSERT(Q_NULLPTR != messageDb);
 
-	QStringList messageItems(messageDb->getMsgForHtmlExport(mId.dmId));
+	QStringList messageItems(messageDb->getMessageForHtmlExport(mId.dmId()));
 	if (messageItems.empty()) {
 		return QString();
 	}
@@ -273,7 +330,7 @@ QString DlgCorrespondenceOverview::msgHtmlEntry(const QString &userName,
 	                   "<tr><td>")
 	    + QStringLiteral("Id:") +
 	    QStringLiteral("</td><td><b>")
-	    + QString::number(mId.dmId) +
+	    + QString::number(mId.dmId()) +
 	    QStringLiteral("</b></td></tr>"
 	                   "<tr><td>")
 	    + tr("Delivery") +
@@ -292,15 +349,15 @@ QString DlgCorrespondenceOverview::msgHtmlEntry(const QString &userName,
 	    QStringLiteral("</b></i></td></tr><tr><td>")
 	    + tr("Sender") +
 	    QStringLiteral(":</td><td><i>")
-	    + messageItems.at(0) +
+	    + messageItems.at(0) + " (" + messageItems.at(5) + ")" +
 	    QStringLiteral("</i></td></tr><tr><td>")
 	    + tr("Recipient") +
 	    QStringLiteral(":</td><td><i>")
-	    + messageItems.at(1) +
+	    + messageItems.at(1) + " (" + messageItems.at(6) + ")" +
 	    QStringLiteral("</i></td></tr></table></td></tr></table>"));
-	if (this->addTagsCheckBox->checkState() == Qt::Checked) {
-		retStr += tagHtmlEntry(m_tagDb, userName, mId.dmId,
-		    this->colourTagsCheckBox->checkState() == Qt::Checked);
+	if (m_ui->addTagsCheckBox->checkState() == Qt::Checked) {
+		retStr += tagHtmlEntry(m_tagCont, acntId, mId.dmId(),
+		    m_ui->colourTagsCheckBox->checkState() == Qt::Checked);
 	}
 	retStr += QStringLiteral("</div>");
 
@@ -312,7 +369,7 @@ bool DlgCorrespondenceOverview::writeCsvOverview(const QString &fileName) const
 	qDebug("Files are going be be exported to CSV file '%s'.",
 	    fileName.toUtf8().constData());
 
-	if (fileName.isEmpty()) {
+	if (Q_UNLIKELY(fileName.isEmpty())) {
 		Q_ASSERT(0);
 		return false;
 	}
@@ -334,21 +391,23 @@ bool DlgCorrespondenceOverview::writeCsvOverview(const QString &fileName) const
 	    tr("Sender Address") + QStringLiteral(",") +
 	    tr("Recipient") + QStringLiteral(",") +
 	    tr("Recipient Address") + QStringLiteral(",") +
-	    tr("Our file mark") + QStringLiteral(",") +
-	    tr("Our reference number") + QStringLiteral(",") +
-	    tr("Your file mark") + QStringLiteral(",") +
-	    tr("Your reference number") + QStringLiteral("\n");
+	    tr("Sender file mark") + QStringLiteral(",") +
+	    tr("Sender reference number") + QStringLiteral(",") +
+	    tr("Recipient file mark") + QStringLiteral(",") +
+	    tr("Recipient reference number") + QStringLiteral(",") +
+	    tr("Sender data-box ID") + QStringLiteral(",") +
+	    tr("Recipient data-box ID") + QStringLiteral("\n");
 
 	/* Sent messages. */
-	if (this->sentCheckBox->isChecked()) {
-		foreach (const MessageDb::MsgId &mId, m_exportedMsgs.sentDmIDs) {
+	if (m_ui->sentCheckBox->isChecked()) {
+		foreach (const MsgId &mId, m_exportedMsgs.sentDmIDs) {
 			f << msgCsvEntry(mId) + QStringLiteral("\n");
 		}
 	}
 
 	/* Received messages. */
-	if (this->receivedCheckBox->isChecked()) {
-		foreach (const MessageDb::MsgId &mId, m_exportedMsgs.receivedDmIDs) {
+	if (m_ui->receivedCheckBox->isChecked()) {
+		foreach (const MsgId &mId, m_exportedMsgs.receivedDmIDs) {
 			f << msgCsvEntry(mId) + QStringLiteral("\n");
 		}
 	}
@@ -359,13 +418,13 @@ bool DlgCorrespondenceOverview::writeCsvOverview(const QString &fileName) const
 	return true;
 }
 
-bool DlgCorrespondenceOverview::writeHtmlOverview(const QString &userName,
+bool DlgCorrespondenceOverview::writeHtmlOverview(const AcntId &acntId,
     const QString &fileName) const
 {
 	qDebug("Files are going be be exported to HTML file '%s'.",
 	    fileName.toUtf8().constData());
 
-	if (fileName.isEmpty()) {
+	if (Q_UNLIKELY(fileName.isEmpty())) {
 		Q_ASSERT(0);
 		return false;
 	}
@@ -380,7 +439,12 @@ bool DlgCorrespondenceOverview::writeHtmlOverview(const QString &userName,
 	 * Always use UTF-8 for this HTML file as this encoding is hard-wired
 	 * into its header.
 	 */
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+	/* UTF-8 is the default encoding in Qt-6 but we want to make sure. */
+	f.setEncoding(QStringConverter::Utf8);
+#else /* < Qt-6.0 */
 	f.setCodec("UTF-8");
+#endif /* >= Qt-6.0 */
 	/* Generate HTML header. */
 	f << QStringLiteral("<!DOCTYPE html\n"
 	    "   PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" "
@@ -405,34 +469,34 @@ bool DlgCorrespondenceOverview::writeHtmlOverview(const QString &userName,
 	    "<table><tr><td>\n")
 	    + tr("From date:") +
 	    QStringLiteral("</td><td>")
-	    + this->fromCalendarWidget->selectedDate().toString("dd.MM.yyyy") +
+	    + m_ui->fromCalendarWidget->selectedDate().toString(Utility::dateDisplayFormat) +
 	    QStringLiteral("</td></tr><tr><td>")
 	    + tr("To date:") +
 	    QStringLiteral("</td><td>")
-	    + this->toCalendarWidget->selectedDate().toString("dd.MM.yyyy") +
+	    + m_ui->toCalendarWidget->selectedDate().toString(Utility::dateDisplayFormat) +
 	    QStringLiteral("</td></tr><tr><td>")
 	    + tr("Generated:") +
 	    QStringLiteral("</td><td>")
-	    + QDateTime().currentDateTime().toString("dd.MM.yyyy hh:mm:ss") +
+	    + QDateTime().currentDateTime().toString(Utility::dateTimeDisplayFormat) +
 	    QStringLiteral("</td></tr></table>\n");
 
 	/* Sent messages. */
-	if (this->sentCheckBox->isChecked()) {
+	if (m_ui->sentCheckBox->isChecked()) {
 		f << QStringLiteral("<h2>") << tr("Sent")
 		    << QStringLiteral("</h2>\n");
 
-		foreach (const MessageDb::MsgId &mId, m_exportedMsgs.sentDmIDs) {
-			f << msgHtmlEntry(userName, mId);
+		foreach (const MsgId &mId, m_exportedMsgs.sentDmIDs) {
+			f << msgHtmlEntry(acntId, mId);
 		}
 	}
 
 	/* Received messages. */
-	if (this->receivedCheckBox->isChecked()) {
+	if (m_ui->receivedCheckBox->isChecked()) {
 		f << QStringLiteral("<h2>") << tr("Received")
 		    << QStringLiteral("</h2>\n");
 
-		foreach (const MessageDb::MsgId &mId, m_exportedMsgs.receivedDmIDs) {
-			f << msgHtmlEntry(userName, mId);
+		foreach (const MsgId &mId, m_exportedMsgs.receivedDmIDs) {
+			f << msgHtmlEntry(acntId, mId);
 		}
 	}
 
@@ -444,18 +508,18 @@ bool DlgCorrespondenceOverview::writeHtmlOverview(const QString &userName,
 	return true;
 }
 
-QString DlgCorrespondenceOverview::exportOverview(const QString &userName,
+QString DlgCorrespondenceOverview::exportOverview(const AcntId &acntId,
     const QString &dir, QString &summary)
 {
 	QString exportDir;
 
 	QString overviewFileName(dir + QDir::separator() + tr("Overview") +
 	    QStringLiteral("--") +
-	    this->fromCalendarWidget->selectedDate().toString(Qt::ISODate) +
+	    m_ui->fromCalendarWidget->selectedDate().toString(Qt::ISODate) +
 	    QStringLiteral("--") +
-	    this->toCalendarWidget->selectedDate().toString(Qt::ISODate));
+	    m_ui->toCalendarWidget->selectedDate().toString(Qt::ISODate));
 	overviewFileName +=
-	    (this->outputFormatComboBox->currentText() == HTML_LITERAL) ?
+	    (m_ui->outputFormatComboBox->currentText() == HTML_LITERAL) ?
 	        QStringLiteral(".html") : QStringLiteral(".csv");
 
 	overviewFileName = QFileDialog::getSaveFileName(this,
@@ -469,9 +533,9 @@ QString DlgCorrespondenceOverview::exportOverview(const QString &userName,
 		    exportDir.toUtf8().constData());
 
 		bool writeHtml =
-		    this->outputFormatComboBox->currentText() == HTML_LITERAL;
+		    m_ui->outputFormatComboBox->currentText() == HTML_LITERAL;
 		bool overviewWritten = writeHtml ?
-		    writeHtmlOverview(userName, overviewFileName) :
+		    writeHtmlOverview(acntId, overviewFileName) :
 		    writeCsvOverview(overviewFileName);
 		if (!overviewWritten) {
 			QMessageBox::warning(this,
@@ -511,28 +575,28 @@ void appendError(enum Exports::ExportFileType fileType, qint64 dmId,
 		qWarning("%s",
 		    QString("DZ '%1' export error.").arg(dmId).toUtf8().constData());
 		errList.append(
-		    QObject::tr("Message '%1' does not contain data necessary for ZFO export.")
+		    DlgCorrespondenceOverview::tr("Message '%1' does not contain data necessary for ZFO export.")
 		        .arg(dmId));
 		break;
 	case Exports::ZFO_DELIVERY:
 		qWarning("%s",
 		    QString("DZ '%1' export error").arg(dmId).toUtf8().constData());
 		errList.append(
-		    QObject::tr("Message '%1' does not contain acceptance info data necessary for ZFO export.")
+		    DlgCorrespondenceOverview::tr("Message '%1' does not contain acceptance info data necessary for ZFO export.")
 		        .arg(dmId));
 		break;
 	case Exports::PDF_ENVELOPE:
 		qWarning("%s",
 		    QString("OZ '%1' export error").arg(dmId).toUtf8().constData());
 		errList.append(
-		    QObject::tr("Message '%1' does not contain message envelope data necessary for PDF export.")
+		    DlgCorrespondenceOverview::tr("Message '%1' does not contain message envelope data necessary for PDF export.")
 		        .arg(dmId));
 		break;
 	case Exports::PDF_DELIVERY:
 		qWarning("%s",
 		    QString("DD '%1' export error").arg(dmId).toUtf8().constData());
 		errList.append(
-		    QObject::tr("Message '%1' does not contain acceptance info data necessary for PDF export.")
+		    DlgCorrespondenceOverview::tr("Message '%1' does not contain acceptance info data necessary for PDF export.")
 		        .arg(dmId));
 		break;
 	case Exports::PDF_DELIV_ATTACH:
@@ -550,50 +614,57 @@ void appendError(enum Exports::ExportFileType fileType, qint64 dmId,
  * @param[in] dbSet Database set.
  * @param[in] fileType Type of files to be generated.
  * @param[in] targetPath Location of created files.
- * @param[in] userName Login identifying the account.
+ * @param[in] acntId Account identifier.
  * @param[in] dbId Account database identifier.
  * @param[in,out] lastPath Last used path.
  * @param[out] errList List of error strings.
  */
 static
-int exportMessageData(const QList<MessageDb::MsgId> &mIds,
-    QWidget *parent, const MessageDbSet &dbSet,
-    enum Exports::ExportFileType fileType, const QString &targetPath,
-    const QString &userName, const QString &dbId, QString &lastPath,
-    QStringList &errList)
+int exportMessageData(const QList<MsgId> &mIds, QWidget *parent,
+    const MessageDbSet &dbSet, enum Exports::ExportFileType fileType,
+    const QString &targetPath, const AcntId &acntId, const QString &dbId,
+    QString &lastPath, QStringList &errList)
 {
 	QString errStr;
 
 	int successCnt = 0;
 	Exports::ExportError ret;
 
-	foreach (const MessageDb::MsgId &mId, mIds) {
-		ret = Exports::exportAs(parent, dbSet, fileType, targetPath,
-		    QString(), userName, dbId, mId, false, lastPath, errStr);
+	const QString accountName(
+	    GlobInstcs::acntMapPtr->acntData(acntId).accountName());
+
+	foreach (const MsgId &mId, mIds) {
+		ret = Exports::exportAsGUI(parent, dbSet, fileType, targetPath,
+		    QString(), acntId.username(), accountName, dbId, mId, false,
+		    lastPath, errStr);
 		if (Exports::EXP_SUCCESS == ret) {
 			++successCnt;
 		} else if (Exports::EXP_CANCELED == ret) {
 			break;
 		} else {
-			appendError(fileType, mId.dmId, errList);
+			appendError(fileType, mId.dmId(), errList);
 		}
 	}
 
 	return successCnt;
 }
 
-void DlgCorrespondenceOverview::exportChosenData(const QString &userName,
-    QString &exportCorrespondDir)
+void DlgCorrespondenceOverview::exportChosenData(const AcntId &acntId,
+    Prefs &prefs)
 {
+	QString exportCorrespondDir = PrefsSpecific::acntCorrespondenceDir(
+	    prefs, acntId);
 	QString summaryMsg;
 	QString exportDir;
 	QString lastPath;
 
 	{
 		const QString saveDir(
-		    exportOverview(userName, exportCorrespondDir, summaryMsg));
+		    exportOverview(acntId, exportCorrespondDir, summaryMsg));
 		if (!saveDir.isEmpty()) {
 			exportCorrespondDir = saveDir;
+			PrefsSpecific::setAcntCorrespondenceDir(prefs, acntId,
+			    exportCorrespondDir);
 		}
 	}
 
@@ -603,10 +674,10 @@ void DlgCorrespondenceOverview::exportChosenData(const QString &userName,
 	int successEnvelopePdfCnt = 0;
 	int successDelInfoPdfCnt = 0;
 
-	if (this->exportZfoCheckBox->isChecked() ||
-	    this->exportDeliveryZfoCheckBox->isChecked() ||
-	    this->exportMessageEnvelopePDFCheckBox->isChecked() ||
-	    this->exportDeliveryPDFCheckBox->isChecked()) {
+	if (m_ui->exportZfoCheckBox->isChecked() ||
+	    m_ui->exportDeliveryZfoCheckBox->isChecked() ||
+	    m_ui->exportMessageEnvelopePDFCheckBox->isChecked() ||
+	    m_ui->exportDeliveryPDFCheckBox->isChecked()) {
 		exportDir = QFileDialog::getExistingDirectory(this,
 		    tr("Select directory for export of ZFO/PDF file(s)"),
 		    exportCorrespondDir,
@@ -620,22 +691,24 @@ void DlgCorrespondenceOverview::exportChosenData(const QString &userName,
 			goto finish;
 		} 
 		exportCorrespondDir = exportDir;
+		PrefsSpecific::setAcntCorrespondenceDir(prefs, acntId,
+		    exportCorrespondDir);
 		qDebug("Files are going to be exported to directory '%s'.",
 		    exportDir.toUtf8().constData());
 	} 
 
 	/* Export messages to ZFO. */
-	if (this->exportZfoCheckBox->isChecked()) {
-		if (this->sentCheckBox->isChecked()) {
+	if (m_ui->exportZfoCheckBox->isChecked()) {
+		if (m_ui->sentCheckBox->isChecked()) {
 			successMsgZFOCnt += exportMessageData(
 			    m_exportedMsgs.sentDmIDs, this, m_messDbSet,
-			    Exports::ZFO_MESSAGE, exportDir, userName,
+			    Exports::ZFO_MESSAGE, exportDir, acntId,
 			    m_dbId, lastPath, errorList);
 		}
-		if (this->receivedCheckBox->isChecked()) {
+		if (m_ui->receivedCheckBox->isChecked()) {
 			successMsgZFOCnt += exportMessageData(
 			    m_exportedMsgs.receivedDmIDs, this, m_messDbSet,
-			    Exports::ZFO_MESSAGE, exportDir, userName,
+			    Exports::ZFO_MESSAGE, exportDir, acntId,
 			    m_dbId, lastPath, errorList);
 		}
 		summaryMsg += QStringLiteral("<b>") +
@@ -646,17 +719,17 @@ void DlgCorrespondenceOverview::exportChosenData(const QString &userName,
 	}
 
 	/* Export delivery info ZFO. */
-	if (this->exportDeliveryZfoCheckBox->isChecked()) {
-		if (this->sentCheckBox->isChecked()) {
+	if (m_ui->exportDeliveryZfoCheckBox->isChecked()) {
+		if (m_ui->sentCheckBox->isChecked()) {
 			successDelInfoZFOCnt += exportMessageData(
 			    m_exportedMsgs.sentDmIDs, this, m_messDbSet,
-			    Exports::ZFO_DELIVERY, exportDir, userName,
+			    Exports::ZFO_DELIVERY, exportDir, acntId,
 			    m_dbId, lastPath, errorList);
 		}
-		if (this->receivedCheckBox->isChecked()) {
+		if (m_ui->receivedCheckBox->isChecked()) {
 			successDelInfoZFOCnt += exportMessageData(
 			    m_exportedMsgs.receivedDmIDs, this, m_messDbSet,
-			    Exports::ZFO_DELIVERY, exportDir, userName,
+			    Exports::ZFO_DELIVERY, exportDir, acntId,
 			    m_dbId, lastPath, errorList);
 		}
 		summaryMsg += QStringLiteral("<b>") +
@@ -667,17 +740,17 @@ void DlgCorrespondenceOverview::exportChosenData(const QString &userName,
 	}
 
 	/* Export envelope to PDF. */
-	if (this->exportMessageEnvelopePDFCheckBox->isChecked()) {
-		if (this->sentCheckBox->isChecked()) {
+	if (m_ui->exportMessageEnvelopePDFCheckBox->isChecked()) {
+		if (m_ui->sentCheckBox->isChecked()) {
 			successEnvelopePdfCnt += exportMessageData(
 			    m_exportedMsgs.sentDmIDs, this, m_messDbSet,
-			    Exports::PDF_ENVELOPE, exportDir, userName,
+			    Exports::PDF_ENVELOPE, exportDir, acntId,
 			    m_dbId, lastPath, errorList);
 		}
-		if (this->receivedCheckBox->isChecked()) {
+		if (m_ui->receivedCheckBox->isChecked()) {
 			successEnvelopePdfCnt += exportMessageData(
 			    m_exportedMsgs.receivedDmIDs, this, m_messDbSet,
-			    Exports::PDF_ENVELOPE, exportDir, userName,
+			    Exports::PDF_ENVELOPE, exportDir, acntId,
 			    m_dbId, lastPath, errorList);
 		}
 		summaryMsg += QStringLiteral("<b>") +
@@ -688,17 +761,17 @@ void DlgCorrespondenceOverview::exportChosenData(const QString &userName,
 	}
 
 	/* Export delivery info to PDF. */
-	if (this->exportDeliveryPDFCheckBox->isChecked()) {
-		if (this->sentCheckBox->isChecked()) {
+	if (m_ui->exportDeliveryPDFCheckBox->isChecked()) {
+		if (m_ui->sentCheckBox->isChecked()) {
 			successDelInfoPdfCnt += exportMessageData(
 			    m_exportedMsgs.sentDmIDs, this, m_messDbSet,
-			    Exports::PDF_DELIVERY, exportDir, userName,
+			    Exports::PDF_DELIVERY, exportDir, acntId,
 			    m_dbId, lastPath, errorList);
 		}
-		if (this->receivedCheckBox->isChecked()) {
+		if (m_ui->receivedCheckBox->isChecked()) {
 			successDelInfoPdfCnt += exportMessageData(
 			    m_exportedMsgs.receivedDmIDs, this, m_messDbSet,
-			    Exports::PDF_DELIVERY, exportDir, userName,
+			    Exports::PDF_DELIVERY, exportDir, acntId,
 			    m_dbId, lastPath, errorList);
 		}
 		summaryMsg += QStringLiteral("<b>") +
@@ -722,7 +795,8 @@ finish:
 			detailMsg += errorList.at(i) + QStringLiteral("\n");
 		}
 	}
-	DlgMsgBox::message(this, QMessageBox::Information, tr("Export results"),
+	DlgMsgBoxDetail::message(this, QMessageBox::Information,
+	    tr("Export results"),
 	    tr("Export of correspondence overview finished with these results:"),
 	    summaryMsg, detailMsg, QMessageBox::Ok, QMessageBox::Ok);
 }
