@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -24,11 +24,19 @@
 #include <QCoreApplication>
 #include <QMessageBox>
 
+#include "src/datovka_shared/compat/compiler.h" /* macroStdMove */
+#include "src/datovka_shared/isds/box_interface.h"
+#include "src/datovka_shared/log/log.h"
+#include "src/datovka_shared/worker/pool.h"
+#include "src/dimensions/dimensions.h"
+#include "src/global.h"
 #include "src/gui/dlg_ds_search.h"
+#include "src/gui/icon_container.h"
 #include "src/io/isds_sessions.h"
+#include "src/settings/prefs_specific.h"
 #include "src/views/table_home_end_filter.h"
 #include "src/views/table_space_selection_filter.h"
-#include "src/worker/pool.h"
+#include "src/views/table_tab_ignore_filter.h"
 #include "ui_dlg_ds_search.h"
 
 #define CBOX_TARGET_ALL 0
@@ -36,12 +44,16 @@
 #define CBOX_TARGET_IC 2
 #define CBOX_TARGET_BOX_ID 3
 
+/* Indexes into box type combo box. */
 #define CBOX_TYPE_ALL 0
 #define CBOX_TYPE_OVM 1
 #define CBOX_TYPE_PO 2
 #define CBOX_TYPE_PFO 3
 #define CBOX_TYPE_FO 4
 
+static const QString dlgName("ds_search");
+static const QString databoxTableName("databox_list");
+
 FulltextSearchThread::FulltextSearchThread(DlgDsSearch *dlg)
     : m_dlg(dlg)
 {
@@ -54,22 +66,30 @@ void FulltextSearchThread::run(void)
 		Q_ASSERT(0);
 		return;
 	}
+
+	logDebugLv0NL("Starting full-text search in thread '%p'.",
+	    (void *) QThread::currentThreadId());
+
 	DlgDsSearch::SearchResultFt searchResult(m_dlg->searchDataBoxFulltext());
 
 	emit searchFinished(searchResult.result, searchResult.longErrMsg);
+
+	logDebugLv0NL("Full-text search finished in thread '%p'.",
+	    (void *) QThread::currentThreadId());
 }
 
-DlgDsSearch::DlgDsSearch(const QString &userName, const QString &dbType,
+DlgDsSearch::DlgDsSearch(const AcntId &acntId, const QString &dbType,
     bool dbEffectiveOVM, bool dbOpenAddressing, QStringList *dbIdList,
     QWidget *parent)
     : QDialog(parent),
-    m_ui(new (std::nothrow) Ui::DlgDsSearch),
+    m_ui(new (::std::nothrow) Ui::DlgDsSearch),
     m_fulltextThread(this),
-    m_userName(userName),
+    m_acntId(acntId),
     m_dbType(dbType),
     m_dbEffectiveOVM(dbEffectiveOVM),
     m_dbOpenAddressing(dbOpenAddressing),
     m_contactListProxyModel(this),
+    m_dfltFilerLineStyleSheet(SortFilterProxyModel::invalidFilterEditStyle),
     m_contactTableModel(this),
     m_boxTypeCBoxModel(this),
     m_fulltextCBoxModel(this),
@@ -78,9 +98,16 @@ DlgDsSearch::DlgDsSearch(const QString &userName, const QString &dbType,
     m_showInfoLabel(false)
 {
 	m_ui->setupUi(this);
+	/* Tab order is defined in UI file. */
+
+	m_ui->searchPushButton->setIcon(
+	    IconContainer::construcIcon(IconContainer::ICON_APP_SEARCH));
 
 	/* Set default line height for table views/widgets. */
 	m_ui->contactTableView->setNarrowedLineHeight();
+	m_ui->contactTableView->horizontalHeader()->setDefaultAlignment(Qt::AlignVCenter | Qt::AlignLeft);
+	m_ui->contactTableView->setSelectionMode(
+	    QAbstractItemView::ExtendedSelection);
 	m_ui->contactTableView->setSelectionBehavior(
 	    QAbstractItemView::SelectRows);
 
@@ -88,16 +115,16 @@ DlgDsSearch::DlgDsSearch(const QString &userName, const QString &dbType,
 	    tr("All") + QStringLiteral(" - ") + tr("All types"),
 	    CBOX_TYPE_ALL);
 	m_boxTypeCBoxModel.appendRow(
-	    tr("OVM") + QStringLiteral(" - ") + tr("Orgán veřejné moci"),
+	    tr("OVM") + QStringLiteral(" - ") + tr("Public authority"), /* organ verejne moci */
 	    CBOX_TYPE_OVM);
 	m_boxTypeCBoxModel.appendRow(
-	    tr("PO") + QStringLiteral(" - ") + tr("Právnická osoba"),
+	    tr("PO") + QStringLiteral(" - ") + tr("Legal person"), /* pravnicka osoba */
 	    CBOX_TYPE_PO);
 	m_boxTypeCBoxModel.appendRow(
-	    tr("PFO") + QStringLiteral(" - ") + tr("Podnikající fyzická osoba"),
+	    tr("PFO") + QStringLiteral(" - ") + tr("Self-employed person"), /* podnikajici fyzicka osoba */
 	    CBOX_TYPE_PFO);
 	m_boxTypeCBoxModel.appendRow(
-	    tr("FO") + QStringLiteral(" - ") + tr("Fyzická osoba"),
+	    tr("FO") + QStringLiteral(" - ") + tr("Natural person"), /* fyzicka osoba */
 	    CBOX_TYPE_FO);
 	m_ui->dataBoxTypeCBox->setModel(&m_boxTypeCBoxModel);
 
@@ -108,7 +135,7 @@ DlgDsSearch::DlgDsSearch(const QString &userName, const QString &dbType,
 	    tr("Address") + QStringLiteral(" - ") + tr("Search in address data"),
 	    CBOX_TARGET_ADDRESS);
 	m_fulltextCBoxModel.appendRow(
-	    tr("IC") + QStringLiteral(" - ") + tr("Identification number"),
+	    tr("IČO") + QStringLiteral(" - ") + tr("Identification number"),
 	    CBOX_TARGET_IC);
 	m_fulltextCBoxModel.appendRow(
 	    tr("ID") + QStringLiteral(" - ") + tr("Box identifier"),
@@ -116,7 +143,7 @@ DlgDsSearch::DlgDsSearch(const QString &userName, const QString &dbType,
 	m_ui->fulltextTargetCBox->setModel(&m_fulltextCBoxModel);
 
 	m_contactTableModel.setHeader();
-	m_contactListProxyModel.setSortRole(ROLE_MSGS_DB_PROXYSORT);
+	m_contactListProxyModel.setSortRole(BoxContactsModel::ROLE_PROXYSORT);
 	m_contactListProxyModel.setSourceModel(&m_contactTableModel);
 	{
 		QList<int> columnList;
@@ -127,9 +154,8 @@ DlgDsSearch::DlgDsSearch(const QString &userName, const QString &dbType,
 	}
 	m_ui->contactTableView->setModel(&m_contactListProxyModel);
 
-	connect(m_ui->contactTableView->selectionModel(),
-	    SIGNAL(selectionChanged(QItemSelection, QItemSelection)),
-	    this, SLOT(setFirstColumnActive(QItemSelection, QItemSelection)));
+	m_ui->contactTableView->setSortingEnabled(true);
+	m_ui->contactTableView->sortByColumn(BoxContactsModel::BOX_ID_COL, Qt::AscendingOrder);
 
 	connect(m_ui->textLineEdit, SIGNAL(textChanged(QString)),
 	    this, SLOT(checkInputFields()));
@@ -165,9 +191,12 @@ DlgDsSearch::DlgDsSearch(const QString &userName, const QString &dbType,
 	m_ui->useFulltextCheckBox->setCheckState(Qt::Checked);
 
 	m_ui->contactTableView->installEventFilter(
-	    new TableHomeEndFilter(this));
+	    new (::std::nothrow) TableHomeEndFilter(m_ui->contactTableView));
 	m_ui->contactTableView->installEventFilter(
-	    new TableSpaceSelectionFilter(this));
+	    new (::std::nothrow) TableSpaceSelectionFilter(
+	        BoxContactsModel::CHECKBOX_COL, m_ui->contactTableView));
+	m_ui->contactTableView->installEventFilter(
+	    new (::std::nothrow) TableTabIgnoreFilter(m_ui->contactTableView));
 
 	m_ui->filterLine->setFixedWidth(200);
 	m_ui->filterLine->setToolTip(tr("Enter sought expression"));
@@ -182,39 +211,61 @@ DlgDsSearch::DlgDsSearch(const QString &userName, const QString &dbType,
 DlgDsSearch::~DlgDsSearch(void)
 {
 	m_fulltextThread.wait();
+
+	PrefsSpecific::setDlgTblRelColWidths(*GlobInstcs::prefsPtr,
+	    dlgName, databoxTableName,
+	    Dimensions::relativeTableColumnWidths(m_ui->contactTableView));
+	PrefsSpecific::setDlgTblColSortOrder(*GlobInstcs::prefsPtr,
+	    dlgName, databoxTableName,
+	    Dimensions::tableColumnSortOrder(m_ui->contactTableView));
+
 	delete m_ui;
 }
 
-QStringList DlgDsSearch::search(const QString &userName, const QString &dbType,
+QStringList DlgDsSearch::search(const AcntId &acntId, const QString &dbType,
     bool dbEffectiveOVM, bool dbOpenAddressing, QWidget *parent)
 {
 	QStringList dbIdList;
 
-	DlgDsSearch dlg(userName, dbType, dbEffectiveOVM, dbOpenAddressing,
+	DlgDsSearch dlg(acntId, dbType, dbEffectiveOVM, dbOpenAddressing,
 	    &dbIdList, parent);
+
+	const QSize dfltSize = dlg.size();
+	{
+		const QSize newSize = Dimensions::dialogueSize(&dlg,
+		    PrefsSpecific::dlgSize(*GlobInstcs::prefsPtr, dlgName),
+		    dfltSize);
+		if (newSize.isValid()) {
+			dlg.resize(newSize);
+		}
+	}
+
 	dlg.exec();
 
+	PrefsSpecific::setDlgSize(*GlobInstcs::prefsPtr, dlgName,
+	    dlg.size(), dfltSize);
+
 	return dbIdList;
 }
 
+void DlgDsSearch::showEvent(QShowEvent *event)
+{
+	QDialog::showEvent(event);
+
+	Dimensions::setRelativeTableColumnWidths(m_ui->contactTableView,
+	    PrefsSpecific::dlgTblRelColWidths(*GlobInstcs::prefsPtr,
+	    dlgName, databoxTableName));
+	Dimensions::setTableColumnSortOrder(m_ui->contactTableView,
+	    PrefsSpecific::dlgTblColSortOrder(*GlobInstcs::prefsPtr,
+	    dlgName, databoxTableName));
+}
+
 void DlgDsSearch::enableOkButton(void)
 {
 	m_ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(
 	    m_contactTableModel.somethingChecked());
 }
 
-void DlgDsSearch::setFirstColumnActive(const QItemSelection &selected,
-    const QItemSelection &deselected)
-{
-	Q_UNUSED(deselected);
-
-	if (selected.isEmpty()) {
-		return;
-	}
-	m_ui->contactTableView->selectColumn(BoxContactsModel::CHECKBOX_COL);
-	m_ui->contactTableView->selectRow(selected.first().top());
-}
-
 void DlgDsSearch::checkInputFields(void)
 {
 	if (Qt::Checked == m_ui->useFulltextCheckBox->checkState()) {
@@ -255,6 +306,7 @@ void DlgDsSearch::searchDataBox(void)
 	QCoreApplication::processEvents();
 	if (Qt::Checked == m_ui->useFulltextCheckBox->checkState()) {
 #if 0
+		m_contactTableModel.removeAllRows();
 		displaySearchResultFt(searchDataBoxFulltext());
 		setSearchControlsEnabled(m_ui, true);
 #else
@@ -265,7 +317,7 @@ void DlgDsSearch::searchDataBox(void)
 		/* Just in case the thread was not running. */
 		m_breakDownloadLoop = false;
 		/* Execute search thread. */
-		searchDataBoxFulltextThread();
+		startFulltextSearchThread();
 #endif
 	} else {
 		if (m_ui->iDLineEdit->text() == ID_ISDS_SYS_DATABOX) {
@@ -336,7 +388,7 @@ void DlgDsSearch::makeSearchElelementsVisible(int fulltextState)
 
 	m_ui->searchResultText->setText(QString());
 
-	m_contactTableModel.removeRows(0, m_contactTableModel.rowCount());
+	m_contactTableModel.removeAllRows();
 
 	if (Qt::Checked == fulltextState) {
 		m_ui->labelSearchDescr->setText(
@@ -346,8 +398,14 @@ void DlgDsSearch::makeSearchElelementsVisible(int fulltextState)
 
 		m_ui->contactTableView->setColumnHidden(
 		    BoxContactsModel::POST_CODE_COL, true);
-		m_ui->contactTableView->setColumnHidden(BoxContactsModel::PDZ_COL,
-		    true);
+		m_ui->contactTableView->setColumnHidden(
+		    BoxContactsModel::PUBLIC_COL, true);
+		m_ui->contactTableView->setColumnHidden(
+		    BoxContactsModel::PDZ_COL, true);
+		m_ui->contactTableView->setColumnHidden(
+		    BoxContactsModel::PAYMENT_COL, true);
+		m_ui->contactTableView->setColumnHidden(
+		    BoxContactsModel::PAYMENTS_COL, true);
 
 		m_ui->fulltextTargetLabel->show();
 		m_ui->fulltextTargetCBox->show();
@@ -356,11 +414,11 @@ void DlgDsSearch::makeSearchElelementsVisible(int fulltextState)
 		m_ui->textLineEdit->show();
 	} else {
 		m_ui->labelSearchDescr->setText(
-		    tr("Enter the ID, IČ or at least three letters from the name of the data box you look for:"));
+		    tr("Enter the box ID, identification number (IČO) or at least three letters from the name of the data box you look for:"));
 
 		/* Cannot search for all data box types. */
 		m_boxTypeCBoxModel.setEnabled(CBOX_TYPE_ALL, false);
-		if (m_ui->dataBoxTypeCBox->currentData(CBoxModel::valueRole).toInt() ==
+		if (m_ui->dataBoxTypeCBox->currentData(CBoxModel::ROLE_VALUE).toInt() ==
 		    CBOX_TYPE_ALL) {
 			m_ui->dataBoxTypeCBox->setCurrentIndex(
 			    m_boxTypeCBoxModel.findRow(CBOX_TYPE_OVM));
@@ -368,8 +426,12 @@ void DlgDsSearch::makeSearchElelementsVisible(int fulltextState)
 
 		m_ui->contactTableView->setColumnHidden(
 		    BoxContactsModel::POST_CODE_COL, false);
-		m_ui->contactTableView->setColumnHidden(BoxContactsModel::PDZ_COL,
-		    true);
+		m_ui->contactTableView->setColumnHidden(
+		    BoxContactsModel::PDZ_COL, true);
+		m_ui->contactTableView->setColumnHidden(
+		    BoxContactsModel::PAYMENT_COL, true);
+		m_ui->contactTableView->setColumnHidden(
+		    BoxContactsModel::PAYMENTS_COL, true);
 
 		m_ui->iDLineLabel->show();
 		m_ui->iDLineEdit->show();
@@ -387,12 +449,22 @@ void DlgDsSearch::makeSearchElelementsVisible(int fulltextState)
 
 void DlgDsSearch::filterContact(const QString &text)
 {
+	/* Store style at first invocation. */
+	if (m_dfltFilerLineStyleSheet == SortFilterProxyModel::invalidFilterEditStyle) {
+		m_dfltFilerLineStyleSheet = m_ui->filterLine->styleSheet();
+	}
+
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 15, 0))
+	m_contactListProxyModel.setFilterRegularExpression(
+	    QRegularExpression(QRegularExpression::escape(text),
+	        QRegularExpression::CaseInsensitiveOption));
+#else /* < Qt-5.15 */
 	m_contactListProxyModel.setFilterRegExp(QRegExp(text,
 	    Qt::CaseInsensitive, QRegExp::FixedString));
+#endif /* >= Qt-5.15 */
 	/* Set filter field background colour. */
 	if (text.isEmpty()) {
-		m_ui->filterLine->setStyleSheet(
-		    SortFilterProxyModel::blankFilterEditStyle);
+		m_ui->filterLine->setStyleSheet(m_dfltFilerLineStyleSheet);
 	} else if (m_contactListProxyModel.rowCount() != 0) {
 		m_ui->filterLine->setStyleSheet(
 		    SortFilterProxyModel::foundFilterEditStyle);
@@ -404,26 +476,26 @@ void DlgDsSearch::filterContact(const QString &text)
 
 void DlgDsSearch::initContent(void)
 {
-	QString dbOpenAddressing = "";
-	QString toolTipInfo = "";
+	QString dbOpenAddressing;
+	QString toolTipInfo;
 
 	if ("OVM" != m_dbType && !m_dbEffectiveOVM) {
 		if (m_dbOpenAddressing) {
-			toolTipInfo = tr("Your account is of type")
+			toolTipInfo = tr("Your data box is of type")
 			    + " " + m_dbType + ".\n" +
 			    tr("You have also Post Data Messages activated.\n"
-			    "This means you can only search for accounts of "
-			    "type OVM and accounts that have Post Data "
+			    "This means you can only search for data boxes of "
+			    "type OVM and data boxes that have Post Data "
 			    "Messages delivery activated.\nBecause of this "
 			    "limitation the results of your current search "
 			    "might not contain all otherwise matching"
-			    " databoxes.");
+			    " data boxes.");
 			dbOpenAddressing = " - " +
 			    tr("commercial messages are enabled");
 		} else {
-			toolTipInfo = tr("Your account is of type")
+			toolTipInfo = tr("Your data box is of type")
 			    + " " + m_dbType + ".\n" +
-			    tr("This means you can only search for accounts of "
+			    tr("This means you can only search for data boxes of "
 			    "type OVM.\nThe current search settings will thus "
 			    "probably yield no result.");
 			dbOpenAddressing = " - " +
@@ -432,14 +504,14 @@ void DlgDsSearch::initContent(void)
 		m_showInfoLabel = true;
 	}
 
-	m_ui->accountInfo->setText("<strong>" + m_userName + "</strong>" +
+	m_ui->accountInfo->setText("<strong>" + m_acntId.username() + "</strong>" +
 	    " - " + m_dbType + dbOpenAddressing);
 
 	m_ui->labelSearchInfo->setStyleSheet("QLabel { color: red }");
 	m_ui->labelSearchInfo->setToolTip(toolTipInfo);
 	m_ui->labelSearchInfo->hide();
 
-	m_ui->contactTableView->setColumnWidth(BoxContactsModel::CHECKBOX_COL, 20);
+	m_ui->contactTableView->setSquareColumnWidth(BoxContactsModel::CHECKBOX_COL);
 	m_ui->contactTableView->setColumnWidth(BoxContactsModel::BOX_ID_COL, 60);
 	m_ui->contactTableView->setColumnWidth(BoxContactsModel::BOX_TYPE_COL, 70);
 	m_ui->contactTableView->setColumnWidth(BoxContactsModel::BOX_NAME_COL, 120);
@@ -459,7 +531,7 @@ void DlgDsSearch::checkInputFieldsNormal(void)
 	m_ui->iCLineEdit->setEnabled((true));
 	m_ui->iDLineEdit->setEnabled((true));
 
-	switch (m_ui->dataBoxTypeCBox->currentData(CBoxModel::valueRole).toInt()) {
+	switch (m_ui->dataBoxTypeCBox->currentData(CBoxModel::ROLE_VALUE).toInt()) {
 	case CBOX_TYPE_OVM:
 		m_ui->iCLineEdit->setEnabled(true);
 		m_ui->nameLineLabel->setText(tr("Subject Name:"));
@@ -537,104 +609,75 @@ void DlgDsSearch::checkInputFieldsFulltext(void)
 	    m_ui->textLineEdit->text().size() > 2);
 }
 
+/*!
+ * @brief Converts box type combo box index onto enum Isds::Type::DbType.
+ *
+ * @note ALL is converted onto null type.
+ *
+ * @param[in] cBoxIdx Combo box index.
+ * @return Data box type.
+ */
+static
+enum Isds::Type::DbType typeIdx2DbType(int cBoxIdx)
+{
+	switch (cBoxIdx) {
+	case CBOX_TYPE_ALL: return Isds::Type::BT_NULL; break;
+	case CBOX_TYPE_OVM: return Isds::Type::BT_OVM; break;
+	case CBOX_TYPE_PO: return Isds::Type::BT_PO; break;
+	case CBOX_TYPE_PFO: return Isds::Type::BT_PFO; break;
+	case CBOX_TYPE_FO: return Isds::Type::BT_FO; break;
+	default:
+		Q_ASSERT(0);
+		return Isds::Type::BT_NULL;
+		break;
+	}
+}
+
 DlgDsSearch::SearchResult DlgDsSearch::searchDataBoxNormal(void)
 {
-	enum TaskSearchOwner::BoxType boxType = TaskSearchOwner::BT_OVM;
-	switch (m_ui->dataBoxTypeCBox->currentData(CBoxModel::valueRole).toInt()) {
-	case CBOX_TYPE_FO:
-		boxType = TaskSearchOwner::BT_FO;
-		break;
-	case CBOX_TYPE_PFO:
-		boxType = TaskSearchOwner::BT_PFO;
-		break;
-	case CBOX_TYPE_PO:
-		boxType = TaskSearchOwner::BT_PO;
-		break;
-	case CBOX_TYPE_OVM:
-	default:
-		boxType = TaskSearchOwner::BT_OVM;
-		break;
+	enum Isds::Type::DbType boxType = typeIdx2DbType(
+	    m_ui->dataBoxTypeCBox->currentData(CBoxModel::ROLE_VALUE).toInt());
+	if (boxType == Isds::Type::BT_NULL) {
+		boxType = Isds::Type::BT_OVM; /* TODO -- Is this necessary? */
 	}
 
-	return queryBoxNormal(m_ui->iDLineEdit->text(), boxType,
-	    m_ui->iCLineEdit->text(), m_ui->nameLineEdit->text(),
-	    m_ui->pscLineEdit->text());
-}
-
-DlgDsSearch::SearchResultFt DlgDsSearch::searchDataBoxFulltext(void)
-{
-	enum TaskSearchOwnerFulltext::FulltextTarget target =
-	    TaskSearchOwnerFulltext::FT_ALL;
-	switch (m_ui->fulltextTargetCBox->currentData(CBoxModel::valueRole).toInt()) {
-	case CBOX_TARGET_ALL:
-		target = TaskSearchOwnerFulltext::FT_ALL;
-		break;
-	case CBOX_TARGET_ADDRESS:
-		target = TaskSearchOwnerFulltext::FT_ADDRESS;
-		break;
-	case CBOX_TARGET_IC:
-		target = TaskSearchOwnerFulltext::FT_IC;
-		break;
-	case CBOX_TARGET_BOX_ID:
-	default:
-		target = TaskSearchOwnerFulltext::FT_BOX_ID;
-		break;
-	}
-
-	enum TaskSearchOwnerFulltext::BoxType boxType =
-	    TaskSearchOwnerFulltext::BT_ALL;
-	switch (m_ui->dataBoxTypeCBox->currentData(CBoxModel::valueRole).toInt()) {
-	case CBOX_TYPE_FO:
-		boxType = TaskSearchOwnerFulltext::BT_FO;
-		break;
-	case CBOX_TYPE_PFO:
-		boxType = TaskSearchOwnerFulltext::BT_PFO;
-		break;
-	case CBOX_TYPE_PO:
-		boxType = TaskSearchOwnerFulltext::BT_PO;
-		break;
-	case CBOX_TYPE_OVM:
-		boxType = TaskSearchOwnerFulltext::BT_OVM;
-		break;
-	case CBOX_TYPE_ALL:
-	default:
-		boxType = TaskSearchOwnerFulltext::BT_ALL;
-		break;
-	}
-
-	return queryBoxFulltextAll(target, boxType, m_ui->textLineEdit->text());
-}
-
-void DlgDsSearch::searchDataBoxFulltextThread(void)
-{
-	m_fulltextThread.start();
-}
-
-DlgDsSearch::SearchResult DlgDsSearch::queryBoxNormal(const QString &boxId,
-    enum TaskSearchOwner::BoxType boxType, const QString &ic,
-    const QString &name, const QString &zipCode)
-{
 	m_ui->contactTableView->setEnabled(false);
-	m_contactTableModel.removeRows(0, m_contactTableModel.rowCount());
+	m_contactTableModel.removeAllRows();
 
 	m_ui->searchResultText->setText(totalFoundStr(0));
 
-	TaskSearchOwner::SoughtOwnerInfo soughtInfo(boxId, boxType, ic, name,
-	    name, name, zipCode);
+	Isds::DbOwnerInfo dbOwnerInfo;
+	{
+		Isds::Address address;
+		Isds::PersonName personName;
 
-	TaskSearchOwner *task = new (std::nothrow) TaskSearchOwner(m_userName,
-	    soughtInfo);
+		dbOwnerInfo.setDbID(m_ui->iDLineEdit->text());
+		dbOwnerInfo.setDbType(boxType);
+		dbOwnerInfo.setIc(m_ui->iCLineEdit->text());
+		personName.setFirstName(m_ui->nameLineEdit->text());
+		personName.setLastName(m_ui->nameLineEdit->text());
+		dbOwnerInfo.setPersonName(macroStdMove(personName));
+		dbOwnerInfo.setFirmName(m_ui->nameLineEdit->text());
+		address.setZipCode(m_ui->pscLineEdit->text());
+		dbOwnerInfo.setAddress(macroStdMove(address));
+	}
+
+	TaskSearchOwner *task = new (::std::nothrow) TaskSearchOwner(m_acntId,
+	    dbOwnerInfo);
 	if (Q_UNLIKELY(task == Q_NULLPTR)) {
 		Q_ASSERT(0);
 		m_ui->contactTableView->setEnabled(true);
 		return SearchResult(TaskSearchOwner::SO_ERROR, QString());
 	}
 	task->setAutoDelete(false);
-	globWorkPool.runSingle(task);
+	if (Q_UNLIKELY(!GlobInstcs::workPoolPtr->runSingle(task))) {
+		delete task; task = Q_NULLPTR;
+		return SearchResult(TaskSearchOwner::SO_ERROR, tr("Cannot execute task."));
+	}
 
 	enum TaskSearchOwner::Result taskResult = task->m_result;
 	QString longErrMsg(task->m_isdsLongError);
-	QList<TaskSearchOwner::BoxEntry> foundBoxes(task->m_foundBoxes);
+	QList<Isds::DbOwnerInfo> foundBoxes(task->m_foundBoxes);
 
 	delete task; task = Q_NULLPTR;
 
@@ -662,6 +705,56 @@ DlgDsSearch::SearchResult DlgDsSearch::queryBoxNormal(const QString &boxId,
 	return SearchResult(taskResult, QString());
 }
 
+DlgDsSearch::SearchResultFt DlgDsSearch::searchDataBoxFulltext(void)
+{
+	enum Isds::Type::FulltextSearchType target =
+	    Isds::Type::FST_GENERAL;
+	switch (m_ui->fulltextTargetCBox->currentData(CBoxModel::ROLE_VALUE).toInt()) {
+	case CBOX_TARGET_ALL:
+		target = Isds::Type::FST_GENERAL;
+		break;
+	case CBOX_TARGET_ADDRESS:
+		target = Isds::Type::FST_ADDRESS;
+		break;
+	case CBOX_TARGET_IC:
+		target = Isds::Type::FST_IC;
+		break;
+	case CBOX_TARGET_BOX_ID:
+	default:
+		target = Isds::Type::FST_BOX_ID;
+		break;
+	}
+
+	enum TaskSearchOwnerFulltext::BoxType boxType =
+	    TaskSearchOwnerFulltext::BT_ALL;
+	switch (m_ui->dataBoxTypeCBox->currentData(CBoxModel::ROLE_VALUE).toInt()) {
+	case CBOX_TYPE_FO:
+		boxType = TaskSearchOwnerFulltext::BT_FO;
+		break;
+	case CBOX_TYPE_PFO:
+		boxType = TaskSearchOwnerFulltext::BT_PFO;
+		break;
+	case CBOX_TYPE_PO:
+		boxType = TaskSearchOwnerFulltext::BT_PO;
+		break;
+	case CBOX_TYPE_OVM:
+		boxType = TaskSearchOwnerFulltext::BT_OVM;
+		break;
+	case CBOX_TYPE_ALL:
+	default:
+		boxType = TaskSearchOwnerFulltext::BT_ALL;
+		break;
+	}
+
+	return queryBoxFulltextAll(target, boxType, m_ui->textLineEdit->text());
+}
+
+void DlgDsSearch::startFulltextSearchThread(void)
+{
+	m_contactTableModel.removeAllRows();
+	m_fulltextThread.start();
+}
+
 void DlgDsSearch::displaySearchResult(const SearchResult &searchResult)
 {
 	switch (searchResult.result) {
@@ -694,12 +787,12 @@ void DlgDsSearch::displaySearchResult(const SearchResult &searchResult)
 }
 
 DlgDsSearch::SearchResultFt DlgDsSearch::queryBoxFulltextPage(
-    enum TaskSearchOwnerFulltext::FulltextTarget target,
+    enum Isds::Type::FulltextSearchType target,
     enum TaskSearchOwnerFulltext::BoxType boxType,
     const QString &phrase, qint64 pageNum)
 {
 	TaskSearchOwnerFulltext *task =
-	    new (std::nothrow) TaskSearchOwnerFulltext(m_userName, phrase,
+	    new (::std::nothrow) TaskSearchOwnerFulltext(m_acntId, phrase,
 	        target, boxType, pageNum, false);
 	if (Q_UNLIKELY(task == Q_NULLPTR)) {
 		Q_ASSERT(0);
@@ -707,11 +800,15 @@ DlgDsSearch::SearchResultFt DlgDsSearch::queryBoxFulltextPage(
 		    QString(), false);
 	}
 	task->setAutoDelete(false);
-	globWorkPool.runSingle(task);
+	if (Q_UNLIKELY(!GlobInstcs::workPoolPtr->runSingle(task))) {
+		delete task; task = Q_NULLPTR;
+		return SearchResultFt(TaskSearchOwnerFulltext::SOF_ERROR,
+		    tr("Cannot execute task."), false);
+	}
 
 	enum TaskSearchOwnerFulltext::Result taskResult = task->m_result;
 	QString longErrMsg(task->m_isdsLongError);
-	QList<TaskSearchOwnerFulltext::BoxEntry> foundBoxes(task->m_foundBoxes);
+	QList<Isds::FulltextResult> foundBoxes(task->m_foundBoxes);
 	quint64 totalDb = task->m_totalMatchingBoxes;
 	bool gotLastPage = task->m_gotLastPage;
 
@@ -744,11 +841,9 @@ DlgDsSearch::SearchResultFt DlgDsSearch::queryBoxFulltextPage(
 }
 
 DlgDsSearch::SearchResultFt DlgDsSearch::queryBoxFulltextAll(
-    enum TaskSearchOwnerFulltext::FulltextTarget target,
+    enum Isds::Type::FulltextSearchType target,
     enum TaskSearchOwnerFulltext::BoxType boxType, const QString &phrase)
 {
-	m_contactTableModel.removeRows(0, m_contactTableModel.rowCount());
-
 	m_ui->searchResultText->setText(totalFoundStr(0));
 
 	qint64 pageNum = 0;
@@ -756,7 +851,6 @@ DlgDsSearch::SearchResultFt DlgDsSearch::queryBoxFulltextAll(
 	    QString(), false);
 	while (searchResult = queryBoxFulltextPage(target, boxType, phrase, pageNum),
 	       ((searchResult.result == TaskSearchOwnerFulltext::SOF_SUCCESS) && !searchResult.lastPage)) {
-		//QCoreApplication::processEvents();
 		++pageNum;
 		if (m_breakDownloadLoop) {
 			m_breakDownloadLoop = false;
