@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,13 +21,13 @@
  * the two.
  */
 
-#ifndef _DLG_DS_SEARCH_H_
-#define _DLG_DS_SEARCH_H_
+#pragma once
 
 #include <QDialog>
-#include <QItemSelection>
 #include <QThread>
 
+#include "src/datovka_shared/identifiers/account_id.h"
+#include "src/datovka_shared/isds/types.h"
 #include "src/models/combo_box_model.h"
 #include "src/models/data_box_contacts_model.h"
 #include "src/models/sort_filter_proxy_model.h"
@@ -45,6 +45,8 @@ class FulltextSearchThread : public QThread {
 public:
 	/*!
 	 * @brief Constructor.
+	 *
+	 * @param[in] dlg Pointer to dialogue owning the thread.
 	 */
 	explicit FulltextSearchThread(DlgDsSearch *dlg);
 
@@ -82,7 +84,7 @@ private:
 	/*!
 	 * @brief Constructor.
 	 */
-	DlgDsSearch(const QString &userName, const QString &dbType,
+	DlgDsSearch(const AcntId &acntId, const QString &dbType,
 	    bool dbEffectiveOVM, bool dbOpenAddressing,
 	    QStringList *dbIdList = Q_NULLPTR, QWidget *parent = Q_NULLPTR);
 
@@ -99,25 +101,25 @@ public:
 	 * @return List of data-box identifiers.
 	 */
 	static
-	QStringList search(const QString &userName, const QString &dbType,
+	QStringList search(const AcntId &acntId, const QString &dbType,
 	    bool dbEffectiveOVM, bool dbOpenAddressing,
 	    QWidget *parent = Q_NULLPTR);
 
+protected:
+	/*!
+	 * @brief Check window geometry and set table columns width.
+	 *
+	 * @param[in] event Widget show event.
+	 */
+	virtual
+	void showEvent(QShowEvent *event) Q_DECL_OVERRIDE;
+
 private slots:
 	/*!
 	 * @brief Activates confirmation button.
 	 */
 	void enableOkButton(void);
 
-	/*!
-	 * @brief Set first column containing checkboxes active.
-	 *
-	 * @param[in] selected Newly selected items.
-	 * @param[in] deselected Deselected items.
-	 */
-	void setFirstColumnActive(const QItemSelection &selected,
-	    const QItemSelection &deselected);
-
 	/*!
 	 * @brief Check input fields sanity and activate search button.
 	 */
@@ -155,7 +157,7 @@ private slots:
 	/*!
 	 * @brief Displays elements relevant for normal or full-text search.
 	 *
-	 * @param[in] fulltextCheckState STate of full-text checkbox state.
+	 * @param[in] fulltextCheckState State of full-text checkbox state.
 	 */
 	void makeSearchElelementsVisible(int fulltextState);
 
@@ -228,24 +230,10 @@ private:
 	SearchResultFt searchDataBoxFulltext(void);
 
 	/*!
-	 * @brief Full-text search for data boxes according given criteria
-	 *     run in background.
+	 * @brief Clear results and start full-text search for data boxes
+	 *     according given criteria in a separate thread run in background.
 	 */
-	void searchDataBoxFulltextThread(void);
-
-	/*!
-	 * @brief Encapsulates query.
-	 *
-	 * @param[in] boxId Data box identifier.
-	 * @param[in] boxTye Type of sought data box.
-	 * @param[in] ic Identifier number.
-	 * @param[in] name Name to search for.
-	 * @param[in] zipCode ZIP code.
-	 * @return Search result.
-	 */
-	SearchResult queryBoxNormal(const QString &boxId,
-	    enum TaskSearchOwner::BoxType boxType, const QString &ic,
-	    const QString &name, const QString &zipCode);
+	void startFulltextSearchThread(void);
 
 	/*!
 	 * @brief Display message about search result.
@@ -264,7 +252,7 @@ private:
 	 * @return Search result.
 	 */
 	SearchResultFt queryBoxFulltextPage(
-	    enum TaskSearchOwnerFulltext::FulltextTarget target,
+	    enum Isds::Type::FulltextSearchType target,
 	    enum TaskSearchOwnerFulltext::BoxType boxType,
 	    const QString &phrase, qint64 pageNum);
 
@@ -276,7 +264,7 @@ private:
 	 * @param[in] phrase Text phrase to search for.
 	 */
 	SearchResultFt queryBoxFulltextAll(
-	    enum TaskSearchOwnerFulltext::FulltextTarget target,
+	    enum Isds::Type::FulltextSearchType target,
 	    enum TaskSearchOwnerFulltext::BoxType boxType,
 	    const QString &phrase);
 
@@ -304,7 +292,7 @@ private:
 
 	FulltextSearchThread m_fulltextThread; /*!< Tread that's waiting for incoming full-text search result. */
 
-	const QString m_userName; /*!< User name used for searching. */
+	const AcntId m_acntId; /*!< Account used for searching. */
 	const QString m_dbType; /*!< Data box type used for searching.  */
 	const bool m_dbEffectiveOVM; /*!< True if box has OVM status. */
 	const bool m_dbOpenAddressing; /*!< True if open addressing is enabled. */
@@ -313,6 +301,7 @@ private:
 	                                               * Used for message
 	                                               * sorting and filtering.
 	                                               */
+	QString m_dfltFilerLineStyleSheet; /*!< Used to remember default line edit style. */
 	BoxContactsModel m_contactTableModel; /*!< Model of found data boxes. */
 	CBoxModel m_boxTypeCBoxModel; /*!< Data box type combo box model. */
 	CBoxModel m_fulltextCBoxModel; /*!< Full-text combo box model. */
@@ -323,5 +312,3 @@ private:
 
 	bool m_showInfoLabel; /*!< Controls the notification about limited search results. */
 };
-
-#endif /* DLG_DS_SEARCH_H */
