@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -29,32 +29,44 @@
 DlgYesNoCheckbox::DlgYesNoCheckbox(const QString &title,
     const QString &questionText, const QString &checkBoxText,
     const QString &detailText, QWidget *parent)
-    : QMessageBox(parent),
-    m_yesButton(Q_NULLPTR)
+    : QMessageBox(parent)
 {
 	this->setWindowTitle(title);
 
 	this->setIcon(QMessageBox::Icon::Question);
 
-	m_yesButton = this->addButton(QMessageBox::Yes);
+	this->addButton(QMessageBox::Yes);
 	this->addButton(QMessageBox::No);
 	this->setDefaultButton(QMessageBox::No);
 
 	this->setText(questionText);
 
-	this->setCheckBox(new (std::nothrow) QCheckBox(checkBoxText, this));
+	this->setCheckBox(new (::std::nothrow) QCheckBox(checkBoxText, this));
 
 	this->setInformativeText(detailText);
-
-	connect(this, SIGNAL(buttonClicked(QAbstractButton *)),
-	    this, SLOT(buttonClicked(QAbstractButton *)));
 }
 
-void DlgYesNoCheckbox::buttonClicked(QAbstractButton *button)
+int DlgYesNoCheckbox::exec(void)
 {
-	if (m_yesButton == button) {
-		done(this->checkBox()->isChecked() ? YesChecked : YesUnchecked);
-	} else {
-		done(No);
+	enum RetVal ret = NoUnchecked;
+
+	/*
+	 * We cannot directly intercept the already connected signal from the
+	 * button box. Calling done() from a custom slot causes troubles in
+	 * various versions of Qt.
+	 * Therefore the original exec() is encapsulated and the received
+	 * return value is translated.
+	 */
+	switch (this->QMessageBox::exec()) {
+	case QMessageBox::Yes:
+		ret = this->checkBox()->isChecked() ? YesChecked : YesUnchecked;
+		break;
+	case QMessageBox::No:
+		ret = this->checkBox()->isChecked() ? NoChecked : NoUnchecked;
+		break;
+	default:
+		break;
 	}
+
+	return ret;
 }
