@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,8 +21,7 @@
  * the two.
  */
 
-#ifndef _DLG_YES_NO_CHECKBOX_H_
-#define _DLG_YES_NO_CHECKBOX_H_
+#pragma once
 
 #include <QMessageBox>
 
@@ -34,9 +33,10 @@ public:
 	 * @brief Value returned by exec().
 	 */
 	enum RetVal {
-		No = 0,
-		YesUnchecked = 1,
-		YesChecked = 2
+		NoUnchecked = 0,
+		NoChecked = 1,
+		YesUnchecked = 2,
+		YesChecked = 3
 	};
 
 	/*!
@@ -52,14 +52,12 @@ public:
 	    const QString &checkBoxText, const QString &detailText,
 	    QWidget *parent = Q_NULLPTR);
 
-private slots:
+public slots:
 	/*!
-	 * @brief Check whether the dialogue has been accepted, calls done().
+	 * @brief Execute the back-up operation and show modal dialogue.
+	 *
+	 * @return Value of enum RetVal.
 	 */
-	void buttonClicked(QAbstractButton *button);
-
-private:
-	const QAbstractButton *m_yesButton; /*!< Pointer to accept button. */
+	virtual
+	int exec(void) Q_DECL_OVERRIDE;
 };
-
-#endif /* _DLG_YES_NO_CHECKBOX_H_ */
