@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -23,10 +23,11 @@
 
 #include <QMessageBox>
 
+#include "src/datovka_shared/log/log.h"
+#include "src/datovka_shared/worker/pool.h"
+#include "src/global.h"
 #include "src/common.h"
 #include "src/io/imports.h"
-#include "src/log/log.h"
-#include "src/worker/pool.h"
 #include "src/worker/task_import_message.h"
 #include "src/worker/task_import_zfo.h"
 
@@ -35,19 +36,22 @@ void Imports::importDbMsgsIntoDatabase(MessageDbSet &dbSet,
 {
 	debugFuncCall();
 
-	TaskImportMessage *task;
-	task = new (std::nothrow) TaskImportMessage(userName, &dbSet,
-	    dbFileList, dbId);
+	TaskImportMessage *task = new (::std::nothrow) TaskImportMessage(
+	    userName, &dbSet, dbFileList, dbId);
+	if (Q_UNLIKELY(task == Q_NULLPTR)) {
+		return;
+	}
 	task->setAutoDelete(true);
-	globWorkPool.assignLo(task);
+	GlobInstcs::workPoolPtr->assignLo(task);
 }
 
 void Imports::importZfoIntoDatabase(const QStringList &fileList,
-    const QList<Task::AccountDescr> &databaseList, enum Type zfoType,
+    const QList<AcntIdDb> &databaseList, enum Type zfoType,
     bool authenticate, QSet<QString> &zfoFilesToImport,
     QList< QPair<QString, QString> > &zfoFilesInvalid,
     int &numFilesToImport, QString &errTxt)
 {
+#define STORE_RAW_INTO_DB true /* Set to false if to store as files. TODO -- determine according to VoDZ. */
 	debugFuncCall();
 
 	QPair<QString, QString> impZFOInfo;
@@ -90,20 +94,24 @@ void Imports::importZfoIntoDatabase(const QStringList &fileList,
 
 	/* First, import messages. */
 	foreach (const QString &fileName, messageZfoFiles) {
-
-		TaskImportZfo *task;
-		task = new (std::nothrow) TaskImportZfo(databaseList, fileName,
-		    TaskImportZfo::ZT_MESSAGE, authenticate);
-		task->setAutoDelete(true);
-		globWorkPool.assignLo(task);
+		TaskImportZfo *task = new (::std::nothrow) TaskImportZfo(
+		    databaseList, fileName, TaskImportZfo::ZT_MESSAGE,
+		    STORE_RAW_INTO_DB, authenticate);
+		if (task != Q_NULLPTR) {
+			task->setAutoDelete(true);
+			GlobInstcs::workPoolPtr->assignLo(task);
+		}
 	}
 	/* Second, import delivery information. */
 	foreach (const QString &fileName, deliveryZfoFiles) {
 
-		TaskImportZfo *task;
-		task = new (std::nothrow) TaskImportZfo(databaseList, fileName,
-		    TaskImportZfo::ZT_DELIVERY_INFO, authenticate);
-		task->setAutoDelete(true);
-		globWorkPool.assignLo(task);
+		TaskImportZfo *task = new (::std::nothrow) TaskImportZfo(
+		    databaseList, fileName, TaskImportZfo::ZT_DELIVERY_INFO,
+		    STORE_RAW_INTO_DB, authenticate);
+		if (task != Q_NULLPTR) {
+			task->setAutoDelete(true);
+			GlobInstcs::workPoolPtr->assignLo(task);
+		}
 	}
+#undef STORE_RAW_INTO_DB
 }
