@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,10 +21,9 @@
  * the two.
  */
 
-#ifndef _IMPORTS_H_
-#define _IMPORTS_H_
+#pragma once
 
-#include "src/worker/task.h"
+#include "src/identifiers/account_id_db.h"
 
 /*!
  * @brief Provides zfo and message import to local database.
@@ -68,9 +67,8 @@ public:
 	 */
 	static
 	void importZfoIntoDatabase(const QStringList &fileList,
-	    const QList<Task::AccountDescr> &databaseList,
-	    enum Type zfoType, bool authenticate,
-	    QSet<QString> &zfoFilesToImport,
+	    const QList<AcntIdDb> &databaseList, enum Type zfoType,
+	    bool authenticate, QSet<QString> &zfoFilesToImport,
 	    QList< QPair<QString, QString> > &zfoFilesInvalid,
 	    int &numFilesToImport, QString &errTxt);
 
@@ -82,5 +80,3 @@ private:
 	 */
 	Imports(void);
 };
-
-#endif /* _IMPORTS_H_ */
