@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,47 +21,55 @@
  * the two.
  */
 
-#ifndef _MESSAGE_DB_SET_H_
-#define _MESSAGE_DB_SET_H_
+#pragma once
 
 #include <QDateTime>
 #include <QMap>
+#include <QObject>
 #include <QString>
 #include <QStringList>
 
+#include "src/identifiers/message_id.h"
 #include "src/io/message_db.h"
 
 #define DB_SUFFIX ".db"
 #define PRIMARY_KEY_RE "[^_]+"
 #define SINGLE_FILE_SEC_KEY ""
 #define YEARLY_SEC_KEY_RE "[0-9][0-9][0-9][0-9]"
-#define YEARLY_SEC_KEY_INVALID INVALID_YEAR
-
-/*
- * Flags used when creating new database file.
- */
-#define MDS_FLG_TESTING         0x01 /*!< Create a testing database. */
-#define MDS_FLG_CREATE_FILE     0x02 /*!< Create file if does not exist. */
-#define MDS_FLG_CHECK_QUICK     0x04 /*!< Perform a quick database check. */
-#define MDS_FLG_CHECK_INTEGRITY 0x08 /*!< Perform a full integrity check. */
-
-/*
- * Error codes returned when accessing/creating new database file.
- */
-#define MDS_ERR_OK       0 /*!< No error. */
-#define MDS_ERR_MISSFILE 1 /*!< Database file does not exist. */
-#define MDS_ERR_NOTAFILE 2 /*!< Database file is not a file. */
-#define MDS_ERR_ACCESS   3 /*!< Error reading/writing database file. */
-#define MDS_ERR_CREATE   4 /*!< Error creating database file. */
-#define MDS_ERR_DATA     5 /*!< Data corrupted or not a database file. */
-#define MDS_ERR_MULTIPLE 6 /*!< Multiple differently organised databases reside in the same location. */
+#define YEARLY_SEC_KEY_INVALID MessageDb::invalidYearName
 
 /*!
  * Organises database files according to secondary keys.
  */
-class MessageDbSet : private QMap<QString, MessageDb *> {
+class MessageDbSet : public QObject, private QMap<QString, MessageDb *> {
+	Q_OBJECT
 
 public:
+	/*!
+	 * Flags used when accessing (creating new) database files.
+	 */
+	enum AccessFlag {
+		NO_OPTIONS = 0x00, /*!< No option specified. */
+		TESTING_ACCOUNT = 0x01, /*!< Create a testing database. */
+		CREATE_FILE = 0x02, /*!< Create file if does not exist. */
+		CHECK_QUICK = 0x04, /*!< Perform a quick database check. */
+		CHECK_INTEGRITY = 0x08 /*!< Perform a full integrity check. */
+	};
+	Q_DECLARE_FLAGS(AccessFlags, AccessFlag)
+
+	/*!
+	 * Error codes returned when accessing/creating new database file.
+	 */
+	enum Error {
+		MDSERR_OK = 0, /*!< No error. */
+		MDSERR_MISSFILE = 1, /*!< Database file does not exist. */
+		MDSERR_NOTAFILE = 2, /*!< Database file is not a file. */
+		MDSERR_ACCESS = 3, /*!< Error reading/writing database file. */
+		MDSERR_CREATE = 4, /*!< Error creating database file. */
+		MDSERR_DATA = 5, /*!< Data corrupted or not a database file. */
+		MDSERR_MULTIPLE = 6 /*!< Multiple differently organised databases reside in the same location. */
+	};
+
 	/*!
 	 * Style of database organisation.
 	 */
@@ -122,7 +130,7 @@ public:
 	 * @brief Creates a copy of the current database into a given new
 	 *     directory.
 	 *
-	 * @param[in] newLocDir  New location directory.
+	 * @param[in] newLocDir New location directory.
 	 * @return True if database was copied and re-opened.
 	 */
 	bool copyToLocation(const QString &newLocDir);
@@ -160,11 +168,23 @@ public:
 	 * @brief Computes the secondary key from supplied date according to
 	 *     container settings.
 	 *
-	 * @param time Time to be converted.
+	 * @param[in] time Time to be converted.
 	 * @return Secondary key or null string on error.
 	 */
 	QString secondaryKey(const QDateTime &time) const;
 
+	/*!
+	 * @brief Computes the secondary key fitting the supplied date range
+	 *     according to container settings.
+	 *
+	 * @param[in] fromTime Start time, ignored when invalid.
+	 * @param[in] toTime End time, ignored when converted.
+	 * @return List of secondary keys or empty list on error. Returns all
+	 *     secondary keys when both limits are invalid.
+	 */
+	QStringList secondaryKeys(const QDateTime &fromTime,
+	    const QDateTime &toTime) const;
+
 	/*!
 	 * @brief Access already existent message database.
 	 *
@@ -182,29 +202,76 @@ public:
 	 */
 	MessageDb *accessMessageDb(const QDateTime &deliveryTime, bool writeNew);
 
+	/*!
+	 * @brief Returns location where database files reside.
+	 *
+	 * @return Path do database directory, empty string if database is in memory.
+	 */
+	QString locationDirectory(void) const;
+
 	/*!
 	 * @brief Returns list of file paths where the database files reside.
 	 *
 	 * @note If the database is stored in memory, then a list containing
-	 * a single string indicating memory location is returned.
-	 * @return List of file location.
+	 *     a single string indicating memory location is returned.
+	 *
+	 * @return List of file locations.
 	 */
 	QStringList fileNames(void) const;
 
+	/*!
+	 * @brief Return list of directory paths associated files reside.
+	 *
+	 * @note If the database is stored in memory, then a list containing
+	 *     a single string indicating memory location is returned.
+	 *
+	 * @return List of directory locations.
+	 */
+	QStringList assocFileDirNames(void) const;
+
+	/*!
+	 * @brief Change the primary key for the database.
+	 *
+	 * @param[in] newPrimaryKey Primary key, usually the username.
+	 */
+	bool changePrimaryKey(const QString &newPrimaryKey);
+
+	/*!
+	 * @brief Return list of existing database files in location that match
+	 *    the criteria.
+	 *
+	 * @param[in] locDir Location directory.
+	 * @param[in] primaryKey Primary key, usually the username.
+	 * @param[in] organisation Database organisation type,
+	 *                         use DO_UNKNOWN if you don't care.
+	 * @param[in] testing Set true if account is in testing environment.
+	 */
+	static
+	QStringList fileNamesAvailable(const QString &locDir,
+	    const QString &primaryKey, enum Organisation organisation,
+	    bool testing);
+
+	/*!
+	 * @brief Return list of existing associated directories in location
+	 *    that match the criteria.
+	 *
+	 * @param[in] locDir Location directory.
+	 * @param[in] primaryKey Primary key, usually the username.
+	 * @param[in] organisation Database organisation type,
+	 *                         use DO_UNKNOWN if you don't care.
+	 * @param[in] testing Set true if account is in testing environment.
+	 */
+	static
+	QStringList assocFileDirNamesAvailable(const QString &locDir,
+	    const QString &primaryKey, enum Organisation organisation,
+	    bool testing);
+
 	/*!
 	 * @brief Returns the database organisation.
 	 * @return Database organisation.
 	 */
 	Organisation organisation(void) const;
 
-	/*!
-	 * @brief Returns the size of the underlying database files.
-	 *
-	 * @param sc Determines the way how to compute the database size.
-	 * @return Database size. May return 0 on error or when no such file exists.
-	 */
-	qint64 underlyingFileSize(enum SizeComputation sc) const;
-
 	/*!
 	 * @brief Returns the year identifier from supplied time.
 	 *
@@ -217,19 +284,20 @@ public:
 	/*!
 	 * @brief Creates a new object.
 	 *
-	 * @param[in] locDir           Directory that holds the database files.
-	 * @param[in] primaryKey       Primary key, usually the user name.
-	 * @param[in] testing          True if this is a testing account.
-	 * @param[in] organisation     How to organise the database files.
+	 * @param[in] locDir Directory that holds the database files.
+	 * @param[in] primaryKey Primary key, usually the username.
+	 * @param[in] testing True if this is a testing account.
+	 * @param[in] organisation How to organise the database files.
 	 * @param[in] connectionPrefix Prefix of the connection name.
-	 * @param[in] manner           How to treat files when opening database.
-	 * @return Pointer to new container or zero pointer on error.
+	 * @param[in] onDisk Use false to create database in memory.
+	 * @param[in] manner How to treat files when opening database.
+	 * @return Pointer to new container or Q_NULLPTR on error.
 	 */
 	static
 	MessageDbSet *createNew(const QString &locDir,
 	    const QString &primaryKey, bool testing,
-	    enum Organisation organisation, const QString &connectionPrefix,
-	    enum CreationManner manner);
+	    enum Organisation organisation, bool onDisk,
+	    const QString &connectionPrefix, enum CreationManner manner);
 
 	/*!
 	 * @brief Get the organisation type of the database.
@@ -288,13 +356,14 @@ public:
 	 * @brief Check existing databases for basic faults.
 	 *
 	 * @param[in] primaryKey ISDS user name.
-	 * @param[in] locDir     Directory where to store the file.
-	 * @param[in] flags      Flags to be passed.
+	 * @param[in] locDir Directory where to store the file.
+	 * @param[in] flags Flags to be passed.
+	 * @param[in] onDisk Use false to reference database in memory.
 	 * @return Error code.
 	 */
 	static
-	int checkExistingDbFile(const QString &locDir,
-	    const QString &primaryKey, int flags);
+	enum Error checkExistingDbFile(const QString &locDir,
+	    const QString &primaryKey, AccessFlags flags, bool onDisk);
 
 	/*!
 	 * @brief Returns file name according to primary and secondary key.
@@ -311,18 +380,142 @@ public:
 	    const QString &primaryKey, const QString &secondaryKey,
 	    bool testing, enum Organisation organisation);
 
+	/*!
+	 * @brief Check database filename validity.
+	 *
+	 * @param[in]  fileName Database filename.
+	 * @param[out] dbUserName Username.
+	 * @paran[out] dbYear Year entry if exists, null string else.
+	 * @paran[out] dbTestingFlag True when account is testing, false else.
+	 * @paran[out] errMsg Error message which can be displayed to the user.
+	 * @return True if database filename is correct.
+	 */
+	static
+	bool isValidDbFileName(const QString &fileName, QString &dbUserName,
+	    QString &dbYear, bool &dbTestingFlag, QString &errMsg);
+
+signals:
+	/*!
+	 * @brief Emitted when a database file has been added into the container.
+	 *
+	 * @note Added database may or may not be already opened.
+	 *
+	 * @param[in] primaryKey Primary key, usually the username.
+	 * @param[in] testing Whether the database is related to a testing account.
+	 */
+	void madeAvailable(const QString &primaryKey, bool testing);
+
+	/*!
+	 * @brief Emitted when a database file is opened.
+	 *
+	 * @param[in] primaryKey Primary key, usually the username.
+	 * @param[in] testing Whether the database is related to a testing account.
+	 */
+	void opened(const QString &primaryKey, bool testing);
+
+	/*!
+	 * @brief Emitted when the locally read flag changes.
+	 *
+	 * @param[in] primaryKey Primary key, usually the username.
+	 * @param[in] testing Whether the database is related to a testing account.
+	 * @param[in] msgIds Message identifiers.
+	 * @param[in] read New locally read status.
+	 */
+	void changedLocallyRead(const QString &primaryKey, bool testing,
+	    const QList<MsgId> &msgIds, bool read);
+
+	/*!
+	 * @brief Emitted when the process state changes.
+	 *
+	 * @param[in] primaryKey Primary key, usually the username.
+	 * @param[in] testing Whether the database is related to a testing account.
+	 * @param[in] msgIds Message identifiers.
+	 * @param[in] state New message state.
+	 */
+	void changedProcessState(const QString &primaryKey, bool testing,
+	    const QList<MsgId> &msgIds, enum MessageDb::MessageProcessState state);
+
+	/*!
+	 * @brief Emitted when complete message written into the database.
+	 *
+	 * @param[in] primaryKey Primary key, usually the username.
+	 * @param[in] testing Whether the database is related to a testing account.
+	 * @param[in] msgId Message identifier.
+	 * @param[in] direction Sent or received messages
+	 *                      (enum MessageDirection).
+	 */
+	void messageInserted(const QString &primaryKey, bool testing,
+	    const MsgId &msgId, int direction);
+
+	/*!
+	 * @brief Emitted when all message data deleted from database.
+	 *
+	 * @param[in] primaryKey Primary key, usually the username.
+	 * @param[in] testing Whether the database is related to a testing account.
+	 * @param[in] msgId Message identifier.
+	 */
+	void messageDataDeleted(const QString &primaryKey, bool testing,
+	    const MsgId &msgId);
+
+private slots:
+	/*!
+	 * @brief This slot must be connected to every database created inside
+	 *     this container.
+	 *
+	 * @param[in] fileName Database file name.
+	 */
+	void watchOpened(const QString &fileName);
+
+	/*!
+	 * @brief This slot must be connected to every database created inside
+	 *     this container.
+	 *
+	 * @param[in] msgIds Message identifiers.
+	 * @param[in] read New locally read status.
+	 */
+	void watchChangedLocallyRead(const QList<MsgId> &msgIds, bool read);
+
+	/*!
+	 * @brief This slot must be connected to every database created inside
+	 *     this container.
+	 *
+	 * @param[in] msgIds Message identifiers.
+	 * @param[in] state New message state.
+	 */
+	void watchChangedProcessState(const QList<MsgId> &msgIds,
+	    enum MessageDb::MessageProcessState state);
+
+	/*!
+	 * @brief This slot must be connected to every database created inside
+	 *     this container.
+	 *
+	 * @param[in] msgId Message identifier.
+	 * @param[in] direction Sent or received messages
+	 *                      (enum MessageDirection).
+	 */
+	void watchMessageInserted(const MsgId &msgId, int direction);
+
+	/*!
+	 * @brief This slot must be connected to every database created inside
+	 *     this container.
+	 *
+	 * @param[in] msgId Message identifier.
+	 */
+	void watchMessageDataDeleted(const MsgId &msgId);
+
 private:
 	/*!
 	 * @brief Creates a database set.
 	 *
-	 * @param[in] locDir           Directory that holds the database files.
-	 * @param[in] primaryKey       Primary key, usually the user name.
-	 * @param[in] testing          True if this is a testing account.
-	 * @param[in] organisation     How to organise the database files.
+	 * @param[in] locDir Directory that holds the database files.
+	 * @param[in] primaryKey Primary key, usually the user name.
+	 * @param[in] testing True if this is a testing account.
+	 * @param[in] organisation How to organise the database files.
+	 * @param[in] onDisk Use false to create database in memory.
 	 * @param[in] connectionPrefix Prefix of the connection name.
 	 */
 	MessageDbSet(const QString &locDir, const QString &primaryKey,
-	    bool testing, enum Organisation organisation,
+	    bool testing, enum Organisation organisation, bool onDisk,
 	    const QString &connectionPrefix);
 
 	/*!
@@ -330,7 +523,7 @@ private:
 	 *
 	 * @param[in] secondaryKey Secondary key.
 	 * @param[in] create       Whether to create the file.
-	 * @return Message database or zero pointer on error.
+	 * @return Message database or Q_NULLPTR on error.
 	 */
 	MessageDb *_accessMessageDb(const QString &secondaryKey, bool create);
 
@@ -338,24 +531,63 @@ private:
 	 * @brief Test given file.
 	 *
 	 * @param[in] filePath Full file path.
-	 * @param[in] flags    Flags.
+	 * @param[in] flags Flags.
 	 * @return Error code.
 	 */
 	static
-	int checkGivenDbFile(const QString &filePath, int flags);
+	enum Error checkGivenDbFile(const QString &filePath, AccessFlags flags);
 
 	const QString m_connectionPrefix; /*!< Used for connection naming. */
-	const QString m_primaryKey; /*!< Used for accessing the database. */
+	QString m_primaryKey; /*!< Used for accessing the database. */
 	const bool m_testing; /*!< Whether those are a testing databases. */
 	QString m_locDir; /*!< Directory where the files reside. */
 	enum Organisation m_organisation; /*!< How the database is organised. */
+	const bool m_onDisk; /*!< True if database is generated within a file system, false if it is held within memory. */
 
-public: /* Database function that have been delegate to the container. */
+public: /* Database function that have been delegated to the container. */
 	/*!
 	 * @brief Call vacuum on all managed databases.
 	 */
 	bool vacuum(void);
 
+	/*!
+	 * @brief Backs up the database into a file.
+	 *
+	 * @param[in] dirName Directory name where to store the files into.
+	 *                    The name must contain slashes ('/') as directory
+	 *                    separators.
+	 * @return False on error.
+	 */
+	bool backup(const QString &dirName);
+
+	/*!
+	 * @brief Return sum of sizes of all managed databases;
+	 *
+	 * @param[in] forceAccess Whether to connect all databases before
+	 *                        querying their size.
+	 *
+	 * @return Size in bytes or -1 on error.
+	 */
+	qint64 dbSize(bool forceAccess = false);
+
+	/*!
+	 * @brief Returns the size of the underlying database files.
+	 *
+	 * @param sc Determines the way how to compute the database size.
+	 * @return Database size. Returns -1 if database resides in memory.
+	 *     Returns 0 if not files exist or have not been written yet.
+	 */
+	qint64 fileSize(enum SizeComputation sc) const;
+
+	/*!
+	 * @brief Check whether message with given ID is a high-volume message.
+	 *
+	 * @param[in]  dmId Message ID.
+	 * @param[out] ok Set to true on success.
+	 * @return True if message message entry found and is VoDZ.
+	 */
+	bool isVodz(qint64 dmId, bool *ok = Q_NULLPTR) const;
+
 	/*!
 	 * @brief Return entries for all received messages.
 	 *
@@ -413,11 +645,19 @@ public: /* Database function that have been delegate to the container. */
 	 *
 	 * @param[in] type Whether to obtain sent or received messages.
 	 * @param[in] year Year number.
-	 * @return Number of unread messages, -1 on error.
+	 * @return Number of unread messages, -1 on error, -2 if database is not opened.
 	 */
 	int msgsUnreadInYear(enum MessageDb::MessageType type,
 	    const QString &year) const;
 
+	/*!
+	 * @brief Returns the number of messages which are in an unaccepted state.
+	 *
+	 * @param[in] type Whether to obtain sent or received messages.
+	 * @return Number of messages in unaccepted state.
+	 */
+	int msgsUnaccepted(enum MessageDb::MessageType type) const;
+
 	/*!
 	 * @brief Return entries for all sent messages.
 	 *
@@ -473,7 +713,8 @@ public: /* Database function that have been delegate to the container. */
 	 * @param[in] state  Message state to be set.
 	 * @return True if operation successful.
 	 */
-	bool msgSetAllReceivedProcessState(enum MessageProcessState state);
+	bool setReceivedMessagesProcessState(
+	    enum MessageDb::MessageProcessState state);
 
 	/*!
 	 * @brief Set process state of received messages in given year.
@@ -483,7 +724,7 @@ public: /* Database function that have been delegate to the container. */
 	 * @return True if operation successful.
 	 */
 	bool smsgdtSetReceivedYearProcessState(const QString &year,
-	    enum MessageProcessState state);
+	    enum MessageDb::MessageProcessState state);
 
 	/*!
 	 * @brief Set process state of recently received messages.
@@ -492,12 +733,12 @@ public: /* Database function that have been delegate to the container. */
 	 * @return True if operation successful.
 	 */
 	bool smsgdtSetWithin90DaysReceivedProcessState(
-	    enum MessageProcessState state);
+	    enum MessageDb::MessageProcessState state);
 
 	/*!
 	 * @brief Returns message identifier of message with given id number.
 	 *
-	 * @note This method is quire expensive as it scans all database files,
+	 * @note This method is quite expensive as it scans all database files,
 	 *     don't use it if not absolutely necessary.
 	 *
 	 * @paran[in] dmId Message identification number.
@@ -505,22 +746,30 @@ public: /* Database function that have been delegate to the container. */
 	 *     If no such message is found then a message identifier
 	 *     containing -1 dmId returned.
 	 */
-	MessageDb::MsgId msgsMsgId(qint64 dmId) const;
+	MsgId msgsMsgId(qint64 dmId) const;
 
 	/*!
 	 * @brief Return contacts from message db.
 	 *
 	 * @return List of vectors containing recipientId, recipientName,
-	 *     recipentAddress.
+	 *     recipientAddress.
 	 */
 	QList<MessageDb::ContactEntry> uniqueContacts(void) const;
 
 	/*!
-	 * @brief Return all message ID from database.
+	 * @brief Return all message IDs from database.
 	 *
-	 * @return message id list.
+	 * @return MsgId set.
 	 */
-	QList<MessageDb::MsgId> getAllMessageIDsFromDB(void) const;
+	QSet<MsgId> getAllMsgIds(void) const;
+
+	/*!
+	 * @brief Return all message IDs from database.
+	 *
+	 * @param[in] messageType Specifies sent or received messages.
+	 * @return MsgId set.
+	 */
+	QSet<MsgId> getAllMsgIds(enum MessageDb::MessageType messageType) const;
 
 	/*!
 	 * @brief Return all IDs of messages without attachment.
@@ -541,36 +790,36 @@ public: /* Database function that have been delegate to the container. */
 	 * @brief Return list of message ids corresponding to given date
 	 *     interval.
 	 *
-	 * @param[in] fromDate  Start date.
-	 * @param[in] toDate    Stop date.
-	 * @param[in] sent      True for sent messages, false for received.
+	 * @param[in] fromDate Start date. Use invalid value to ignore this limit.
+	 * @param[in] toDate Stop date. Use invalid value to ignore this limit.
+	 * @param[in] messageType Specifies sent or received messages.
 	 * @return List of message ids. Empty list on error.
 	 */
-	QList<MessageDb::MsgId> msgsDateInterval(const QDate &fromDate,
-	    const QDate &toDate, enum MessageDirection msgDirect) const;
+	QList<MsgId> msgsDateInterval(const QDate &fromDate,
+	    const QDate &toDate, enum MessageDb::MessageType messageType) const;
 
 	/*!
-	 * @brief Searches for matching messages according to envelope data.
+	 * @brief Message search according to envelope data.
 	 *
-	 * @return message item list pass to search query.
+	 * @param[in] envel Message envelope data to search for.
+	 * @param[in] msgDirect Message orientation.
+	 * @param[in] attachPhrase Phrase to search in attachment names.
+	 * @param[in] logicalAnd Set to true if found messages should match all criteria,
+	 *                       set to false if found messages should match any criteria.
+	 * @param[in] fromDate Start date. Use invalid value to ignore this limit.
+	 * @param[in] toDate Stop date. Use invalid value to ignore this limit.
+	 * @return Found message data.
 	 */
-	QList<MessageDb::SoughtMsg> msgsAdvancedSearchMessageEnvelope(
-	    qint64 dmId, const QString &dmAnnotation,
-	    const QString &dbIDSender, const QString &dmSender,
-	    const QString &dmAddress, const QString &dbIDRecipient,
-	    const QString &dmRecipient, const QString &dmSenderRefNumber,
-	    const QString &dmSenderIdent, const QString &dmRecipientRefNumber,
-	    const QString &dmRecipientIdent, const QString &dmToHands,
-	    const QString &dmDeliveryTime, const QString &dmAcceptanceTime,
-	    enum MessageDirection msgDirect) const;
+	QList<MessageDb::SoughtMsg> msgsSearch(const Isds::Envelope &envel,
+	    enum MessageDirection msgDirect, const QString &attachPhrase,
+	    bool logicalAnd, const QDate &fromDate, const QDate &toDate) const;
 
 	/*!
 	 * @brief Get message envelope data from id.
 	 *
 	 * @return message data for message id.
 	 */
-	MessageDb::SoughtMsg msgsGetMsgDataFromId(
-	    const qint64 msgId) const;
+	MessageDb::SoughtMsg msgsGetMsgDataFromId(const qint64 msgId) const;
 
 private:
 	/*!
@@ -581,13 +830,14 @@ private:
 	 */
 	QStringList _yrly_secKeysIn90Days(void) const;
 
+	inline bool _sf_isVodz(qint64 dmId, bool *ok) const;
+	inline bool _yrly_isVodz(qint64 dmId, bool *ok) const;
+
 	inline QList<MessageDb::RcvdEntry> _sf_msgsRcvdEntries(void) const;
 	inline QList<MessageDb::RcvdEntry> _yrly_msgsRcvdEntries(void) const;
 
 	inline QList<MessageDb::RcvdEntry> _sf_msgsRcvdEntriesWithin90Days(void) const;
 	static
-	inline QList<MessageDb::RcvdEntry> _yrly_2dbs_attach_msgsRcvdEntriesWithin90Days(MessageDb &db, const QString &attachFileName);
-	static
 	inline QList<MessageDb::RcvdEntry> _yrly_2dbs_msgsRcvdEntriesWithin90Days(MessageDb &db0, MessageDb &db1);
 	inline QList<MessageDb::RcvdEntry> _yrly_msgsRcvdEntriesWithin90Days(void) const;
 
@@ -606,13 +856,14 @@ private:
 	inline int _sf_msgsUnreadInYear(enum MessageDb::MessageType type, const QString &year) const;
 	inline int _yrly_msgsUnreadInYear(enum MessageDb::MessageType type, const QString &year) const;
 
+	inline int _sf_msgsUnaccepted(enum MessageDb::MessageType type) const;
+	inline int _yrly_msgsUnaccepted(enum MessageDb::MessageType type) const;
+
 	inline QList<MessageDb::SntEntry> _sf_msgsSntEntries(void) const;
 	inline QList<MessageDb::SntEntry> _yrly_msgsSntEntries(void) const;
 
 	inline QList<MessageDb::SntEntry> _sf_msgsSntEntriesWithin90Days(void) const;
 	static
-	inline QList<MessageDb::SntEntry> _yrly_2dbs_attach_msgsSntEntriesWithin90Days(MessageDb &db, const QString &attachFileName);
-	static
 	inline QList<MessageDb::SntEntry> _yrly_2dbs_msgsSntEntriesWithin90Days(MessageDb &db0, MessageDb &db1);
 	inline QList<MessageDb::SntEntry> _yrly_msgsSntEntriesWithin90Days(void) const;
 
@@ -628,23 +879,26 @@ private:
 	inline bool _sf_smsgdtSetWithin90DaysReceivedLocallyRead(bool read);
 	inline bool _yrly_smsgdtSetWithin90DaysReceivedLocallyRead(bool read);
 
-	inline bool _sf_msgSetAllReceivedProcessState(enum MessageProcessState state);
-	inline bool _yrly_msgSetAllReceivedProcessState(enum MessageProcessState state);
+	inline bool _sf_msgSetAllReceivedProcessState(enum MessageDb::MessageProcessState state);
+	inline bool _yrly_msgSetAllReceivedProcessState(enum MessageDb::MessageProcessState state);
 
-	inline bool _sf_smsgdtSetReceivedYearProcessState(const QString &year, enum MessageProcessState state);
-	inline bool _yrly_smsgdtSetReceivedYearProcessState(const QString &year, enum MessageProcessState state);
+	inline bool _sf_smsgdtSetReceivedYearProcessState(const QString &year, enum MessageDb::MessageProcessState state);
+	inline bool _yrly_smsgdtSetReceivedYearProcessState(const QString &year, enum MessageDb::MessageProcessState state);
 
-	inline bool _sf_smsgdtSetWithin90DaysReceivedProcessState(enum MessageProcessState state);
-	inline bool _yrly_smsgdtSetWithin90DaysReceivedProcessState(enum MessageProcessState state);
+	inline bool _sf_smsgdtSetWithin90DaysReceivedProcessState(enum MessageDb::MessageProcessState state);
+	inline bool _yrly_smsgdtSetWithin90DaysReceivedProcessState(enum MessageDb::MessageProcessState state);
 
-	MessageDb::MsgId _sf_msgsMsgId(qint64 dmId) const;
-	MessageDb::MsgId _yrly_msgsMsgId(qint64 dmId) const;
+	MsgId _sf_msgsMsgId(qint64 dmId) const;
+	MsgId _yrly_msgsMsgId(qint64 dmId) const;
 
 	inline QList<MessageDb::ContactEntry> _sf_uniqueContacts(void) const;
 	inline QList<MessageDb::ContactEntry> _yrly_uniqueContacts(void) const;
 
-	inline QList<MessageDb::MsgId> _sf_getAllMessageIDsFromDB(void) const;
-	inline QList<MessageDb::MsgId> _yrly_getAllMessageIDsFromDB(void) const;
+	inline QSet<MsgId> _sf_getAllMsgIds(void) const;
+	inline QSet<MsgId> _yrly_getAllMsgIds(void) const;
+
+	inline QSet<MsgId> _sf_getAllMsgIds(enum MessageDb::MessageType messageType) const;
+	inline QSet<MsgId> _yrly_getAllMsgIds(enum MessageDb::MessageType messageType) const;
 
 	inline QList<qint64> _sf_getAllMessageIDsWithoutAttach(void) const;
 	inline QList<qint64> _yrly_getAllMessageIDsWithoutAttach(void) const;
@@ -652,20 +906,12 @@ private:
 	inline QList<qint64> _sf_getAllMessageIDs(enum MessageDb::MessageType messageType) const;
 	inline QList<qint64> _yrly_getAllMessageIDs(enum MessageDb::MessageType messageType) const;
 
-	inline QList<MessageDb::MsgId> _sf_msgsDateInterval(const QDate &fromDate, const QDate &toDate, enum MessageDirection msgDirect) const;
-	inline QList<MessageDb::MsgId> _yrly_msgsDateInterval(const QDate &fromDate, const QDate &toDate, enum MessageDirection msgDirect) const;
+	inline QList<MsgId> _sf_msgsDateInterval(const QDate &fromDate, const QDate &toDate, enum MessageDb::MessageType messageType) const;
+	inline QList<MsgId> _yrly_msgsDateInterval(const QDate &fromDate, const QDate &toDate, enum MessageDb::MessageType messageType) const;
 
-	inline QList<MessageDb::SoughtMsg> _sf_msgsAdvancedSearchMessageEnvelope(qint64 dmId, const QString &dmAnnotation, const QString &dbIDSender,
-	    const QString &dmSender, const QString &dmAddress, const QString &dbIDRecipient, const QString &dmRecipient, const QString &dmSenderRefNumber,
-	    const QString &dmSenderIdent, const QString &dmRecipientRefNumber, const QString &dmRecipientIdent, const QString &dmToHands,
-	    const QString &dmDeliveryTime, const QString &dmAcceptanceTime, enum MessageDirection msgDirect) const;
-	inline QList<MessageDb::SoughtMsg> _yrly_msgsAdvancedSearchMessageEnvelope(qint64 dmId, const QString &dmAnnotation, const QString &dbIDSender,
-	    const QString &dmSender, const QString &dmAddress, const QString &dbIDRecipient, const QString &dmRecipient, const QString &dmSenderRefNumber,
-	    const QString &dmSenderIdent, const QString &dmRecipientRefNumber, const QString &dmRecipientIdent, const QString &dmToHands,
-	    const QString &dmDeliveryTime, const QString &dmAcceptanceTime, enum MessageDirection msgDirect) const;
+	inline QList<MessageDb::SoughtMsg> _sf_msgsSearch(const Isds::Envelope &envel, enum MessageDirection msgDirect, const QString &attachPhrase, bool logicalAnd, const QDate &fromDate, const QDate &toDate) const;
+	inline QList<MessageDb::SoughtMsg> _yrly_msgsSearch(const Isds::Envelope &envel, enum MessageDirection msgDirect, const QString &attachPhrase, bool logicalAnd, const QDate &fromDate, const QDate &toDate) const;
 
 	inline MessageDb::SoughtMsg _sf_msgsGetMsgDataFromId(const qint64 msgId) const;
 	inline MessageDb::SoughtMsg _yrly_msgsGetMsgDataFromId(const qint64 msgId) const;
 };
-
-#endif /* _MESSAGE_DB_SET_H_ */
