@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -23,12 +23,12 @@
 
 #include <QFileInfo>
 
-#include "message_db_single.h"
+#include "src/io/message_db.h"
+#include "src/io/message_db_single.h"
 
 MessageDbSingle::MessageDbSingle(void)
-    : m_db(NULL)
+    : m_db(Q_NULLPTR)
 {
-
 }
 
 MessageDbSingle::~MessageDbSingle(void)
@@ -39,34 +39,31 @@ MessageDbSingle::~MessageDbSingle(void)
 MessageDbSingle *MessageDbSingle::createNew(const QString &filePath,
     const QString &connectionPrefix)
 {
-	MessageDb *db = NULL;
-	MessageDbSingle *dbSingle = NULL;
-
-	if (filePath.isEmpty()) {
-		return NULL;
+	if (Q_UNLIKELY(filePath.isEmpty())) {
+		return Q_NULLPTR;
 	}
 
-	if (!QFileInfo::exists(filePath)) {
-		return NULL;
+	if (Q_UNLIKELY(!QFileInfo::exists(filePath))) {
+		return Q_NULLPTR;
 	}
 
 	QString connectionName(connectionPrefix + "_SINGLE_FILE");
 
-	db = new(std::nothrow) MessageDb(connectionName);
-	if (NULL == db) {
+	MessageDb *db = new (::std::nothrow) MessageDb(connectionName);
+	if (Q_UNLIKELY(Q_NULLPTR == db)) {
 		Q_ASSERT(0);
-		return NULL;
+		return Q_NULLPTR;
 	}
 
-	if (!db->openDb(filePath)) {
+	if (Q_UNLIKELY(!db->openDb(filePath, SQLiteDb::CREATE_MISSING))) {
 		delete db;
-		return NULL;
+		return Q_NULLPTR;
 	}
 
-	dbSingle = new(std::nothrow) MessageDbSingle();
-	if (NULL == dbSingle) {
+	MessageDbSingle *dbSingle = new (::std::nothrow) MessageDbSingle();
+	if (Q_UNLIKELY(Q_NULLPTR == dbSingle)) {
 		delete db;
-		return NULL;
+		return Q_NULLPTR;
 	}
 
 	/* Assign database. */
@@ -77,13 +74,32 @@ MessageDbSingle *MessageDbSingle::createNew(const QString &filePath,
 
 /* Methods delegated from MessageDb and made public via this class. */
 
-QList<MessageDb::MsgId> MessageDbSingle::getAllMessageIDsFromDB(void) const
+QSet<MsgId> MessageDbSingle::getAllMsgIds(void) const
 {
-	return m_db->getAllMessageIDsFromDB();
+	return m_db->getAllMsgIds();
 }
 
 bool MessageDbSingle::isRelevantMsgForImport(qint64 msgId,
-    const QString databoxId) const
+    const QString &dbId) const
 {
-	return m_db->isRelevantMsgForImport(msgId, databoxId);
+	return m_db->isRelevantMsgForImport(msgId, dbId);
+}
+
+int MessageDbSingle::getMessageType(qint64 dmId)
+{
+	return m_db->getMessageType(dmId);
+}
+
+bool MessageDbSingle::insertOrReplaceCompleteMessage(bool storeRawInDb,
+    const Isds::Message &msg,
+    enum MessageDirection msgDirect, bool verified, bool transaction)
+{
+	return m_db->insertOrReplaceCompleteMessage(storeRawInDb, msg, msgDirect,
+	    verified, transaction);
+}
+
+bool MessageDbSingle::insertOrReplaceDeliveryInfoRaw(bool storeRawInDb,
+    qint64 dmId, const QByteArray &raw)
+{
+	return m_db->insertOrReplaceDeliveryInfoRaw(storeRawInDb, dmId, raw);
 }
