@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,18 +21,17 @@
  * the two.
  */
 
-#ifndef _ACCOUNTS_MODEL_H_
-#define _ACCOUNTS_MODEL_H_
+#pragma once
 
 #include <QAbstractItemModel>
 #include <QList>
 #include <QMap>
 #include <QObject>
-#include <QSettings>
 #include <QString>
 #include <QVariant>
 
-#include "src/settings/accounts.h"
+#include "src/datovka_shared/identifiers/account_id.h"
+#include "src/models/account_unique_numid.h"
 
 /* Login method descriptors. */
 #define LIM_USERNAME "username"
@@ -41,13 +40,29 @@
 #define LIM_HOTP "hotp"
 #define LIM_TOTP "totp"
 
+class AccountsMap; /* Forward declaration. */
+class QMimeData; /* Forward declaration. */
+
 /*!
  * @brief Account hierarchy.
  */
-class AccountModel: public QAbstractItemModel {
-    Q_OBJECT
+class AccountModel : public QAbstractItemModel {
+	Q_OBJECT
 
 public:
+	/*!
+	 * @brief Additional roles.
+	 */
+	enum UserRoles {
+		ROLE_PLAIN_DISPLAY = (Qt::UserRole + 1), /*!< Low level data. */
+		ROLE_COLLAPSED_BIT = 0x0200 /* Actually not a role value, but a bit mask. */
+	};
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 5, 0))
+	Q_ENUM(UserRoles)
+#else /* < Qt-5.5 */
+	Q_ENUMS(UserRoles)
+#endif /* >= Qt-5.5 */
+
 	/*
 	 * nodeRoot (Invisible.)
 	 * |
@@ -103,12 +118,29 @@ public:
 		DESCENDING
 	};
 
+	/*!
+	 * @brief Unread message yearly counter.
+	 */
+	class YearCounter {
+	public:
+		YearCounter(void)
+		    : dbOpened(false), unread(0)
+		{ }
+
+		YearCounter(bool o, unsigned u)
+		    : dbOpened(o), unread(u)
+		{ }
+
+		bool dbOpened; /*!< True if underlying database has been opened. */
+		unsigned unread; /*!< Number of unread messages. */
+	};
+
 	/*!
 	 * @brief Constructor.
 	 *
 	 * @param[in] parent Pointer to parent object.
 	 */
-	explicit AccountModel(QObject *parent = 0);
+	explicit AccountModel(QObject *parent = Q_NULLPTR);
 
 	/*!
 	 * @brief Return index specified by supplied parameters.
@@ -164,73 +196,130 @@ public:
 	/*!
 	 * @brief Returns header data in given location under given role.
 	 *
-	 * @brief[in] section     Header position.
-	 * @brief[in] orientation Header orientation.
-	 * @brief[in] role        Data role.
+	 * @param[in] section     Header position.
+	 * @param[in] orientation Header orientation.
+	 * @param[in] role        Data role.
 	 * @return Header data from model.
 	 */
 	virtual
 	QVariant headerData(int section, Qt::Orientation orientation,
 	    int role = Qt::DisplayRole) const Q_DECL_OVERRIDE;
 
+	/*!
+	 * @brief Move rows.
+	 *
+	 * @param[in] sourceParent Source parent.
+	 * @param[in] sourceRow Source row.
+	 * @param[in] count Number of rows to be moved.
+	 * @param[in] destinationParent Destination parent.
+	 * @param[in] destinationChild Row to move data into.
+	 * @return If move performed.
+	 */
+	virtual
+	bool moveRows(const QModelIndex &sourceParent, int sourceRow,
+	    int count, const QModelIndex &destinationParent,
+	    int destinationChild) Q_DECL_OVERRIDE;
+
+	/*!
+	 * @brief Remove rows.
+	 *
+	 * @param[in] row Starting row.
+	 * @param[in] count Number of rows to be removed.
+	 * @param[in] parent Parent item the row is relative to.
+	 * @return True if the rows were successfully removed.
+	 */
+	virtual
+	bool removeRows(int row, int count,
+	    const QModelIndex &parent = QModelIndex()) Q_DECL_OVERRIDE;
+
+	/*!
+	 * @brief Returns the drop actions supported by this model.
+	 *
+	 * @return Supported drop actions.
+	 */
+	virtual
+	Qt::DropActions supportedDropActions(void) const Q_DECL_OVERRIDE;
+
 	/*!
 	 * @brief Returns item flags for given index.
 	 *
-	 * @brief[in] index Index specifying the item.
+	 * @param[in] index Index specifying the item.
 	 * @return Item flags.
 	 */
 	virtual
 	Qt::ItemFlags flags(const QModelIndex &index) const Q_DECL_OVERRIDE;
 
 	/*!
-	 * @brief Load data from supplied settings.
+	 * @brief Returns the list of allowed MIME types.
 	 *
-	 * @param[in] settings Settings structure to be read.
+	 * @return List of MIME types.
 	 */
-	void loadFromSettings(const QSettings &settings);
+	virtual
+	QStringList mimeTypes(void) const Q_DECL_OVERRIDE;
 
 	/*!
-	 * @brief Store data to settings structure.
+	 * @brief Returns object containing serialised attachment data.
 	 *
-	 * @param[out] settings Setting structure to store the model content
-	 *     into.
+	 * @param[in] indexes List of indexes.
+	 * @return Pointer to newly allocated MIME data object, Q_NULLPTR on error.
 	 */
-	void saveToSettings(QSettings &settings) const;
+	virtual
+	QMimeData *mimeData(
+	    const QModelIndexList &indexes) const Q_DECL_OVERRIDE;
 
 	/*!
-	 * @brief Add account.
+	 * @brief Returns whether the model accepts drops of given MIME data.
 	 *
-	 * @patam[in]  acntSettings Settings data to be added into the model.
-	 * @param[out] idx          Index of newly added account if specified.
-	 * @return -2 if account already exists,
-	 *         -1 if account could not be added,
-	 *          0 if account was added.
+	 * @param[in] data Data to be dropped.
+	 * @param[in] action Type of drop action.
+	 * @param[in] row Target row.
+	 * @param[in] column Target column.
+	 * @param[in] parent Parent index.
+	 * @return True if drop is accepted.
 	 */
-	int addAccount(const AcntSettings &acntSettings, QModelIndex *idx = 0);
+	virtual
+	bool canDropMimeData(const QMimeData *data, Qt::DropAction action,
+	    int row, int column,
+	    const QModelIndex &parent) const Q_DECL_OVERRIDE;
 
 	/*!
-	 * @brief Delete account.
+	 * @brief Handles data supplied by drop operation.
 	 *
-	 * @param[in] userName User name.
+	 * @param[in] data Data to be dropped.
+	 * @param[in] action Type of drop action.
+	 * @param[in] row Target row.
+	 * @param[in] column Target column.
+	 * @param[in] parent Parent index.
+	 * @return True if data are handled by the model.
 	 */
-	void deleteAccount(const QString &userName);
+	virtual
+	bool dropMimeData(const QMimeData *data, Qt::DropAction action,
+	    int row, int column,
+	    const QModelIndex &parent) Q_DECL_OVERRIDE;
 
 	/*!
-	 * @brief Returns user name for given node.
+	 * @brief Load content.
+	 *
+	 * @param[in] accounts Accounts.
+	 */
+	void loadContent(AccountsMap *accounts);
+
+	/*!
+	 * @brief Returns account identifier for given node.
 	 *
 	 * @param[in] index Data index.
-	 * @return User name of the account which the node belongs to
-	 *     or null string on error.
+	 * @return Account identifier which the node belongs to or a null
+	 *     identifier on error.
 	 */
-	QString userName(const QModelIndex &index) const;
+	const AcntId &acntId(const QModelIndex &index) const;
 
 	/*!
 	 * @brief Returns top node related to user name.
 	 *
-	 * @param[in] userName Sought user name.
+	 * @param[in] acntId Account identifier.
 	 * @return Top node index or invalid index if no such name found.
 	 */
-	QModelIndex topAcntIndex(const QString &userName) const;
+	QModelIndex topAcntIndex(const AcntId &acntId) const;
 
 	/*!
 	 * @brief Returns node type.
@@ -242,71 +331,78 @@ public:
 	enum NodeType nodeType(const QModelIndex &index);
 
 	/*!
-	 * @brief Checks whether node type is received.
+	 * @brief Checks whether node type refers to a received type.
 	 *
 	 * @param[in] index Data index.
 	 * @return True if node type is received.
 	 */
 	static
-	bool nodeTypeIsReceived(const QModelIndex &index);
+	bool nodeTypeIsReceived(enum NodeType nodeType);
 
 	/*!
-	 * @brief Checks whether node type is sent.
+	 * @brief Checks whether node type refers to a sent type.
 	 *
 	 * @param[in] index Data index.
 	 * @return True if node type is received.
 	 */
 	static
-	bool nodeTypeIsSent(const QModelIndex &index);
+	bool nodeTypeIsSent(enum NodeType nodeType);
+
+	/*!
+	 * @brief Checks whether node is of received type.
+	 *
+	 * @param[in] index Data index.
+	 * @return True if node type is received.
+	 */
+	static
+	bool nodeIsReceived(const QModelIndex &index);
+
+	/*!
+	 * @brief Checks whether node is of sent type.
+	 *
+	 * @param[in] index Data index.
+	 * @return True if node type is received.
+	 */
+	static
+	bool nodeIsSent(const QModelIndex &index);
 
 	/*!
 	 * @brief Set number of unread messages in recent model nodes.
 	 *
-	 * @param[in] userName   User name.
-	 * @param[in] nodeType   May be nodeRecentReceived or nodeRecentSent.
+	 * @param[in] acntId Account identifier.
+	 * @param[in] nodeType May be nodeRecentReceived or nodeRecentSent.
 	 * @param[in] unreadMsgs Number of unread messages.
 	 * @return True on success.
 	 */
-	bool updateRecentUnread(const QString &userName,
+	bool updateRecentUnread(const AcntId &acntId,
 	    enum NodeType nodeType, unsigned unreadMsgs = 0);
 
-	/*!
-	 * @brief Append year node into account.
-	 *
-	 * @param[in] userName   User name.
-	 * @param[in] nodeType   May be nodeReceivedYear or nodeSentYear.
-	 * @param[in] year       Year string.
-	 * @param[in] unreadMsgs Number of unread messages.
-	 * @return True on success.
-	 */
-	bool appendYear(const QString &userName, enum NodeType nodeType,
-	    const QString &year, unsigned unreadMsgs = 0);
-
 	/*!
 	 * @brief Update year nodes.
 	 *
-	 * @param[in] userName         User name.
-	 * @param[in] nodeType         May be nodeReceivedYear or nodeSentYear.
+	 * @param[in] acntId Account identifier.
+	 * @param[in] nodeType May be nodeReceivedYear or nodeSentYear.
 	 * @param[in] yearlyUnreadList List of paired years and unread messages
 	 *                             numbers.
-	 * @param[in] sorting          Sorting.
+	 * @param[in] sorting Sorting.
+	 * @param[in] prohibitYearRemoval Set to true if year nodes should not be removed.
 	 * @return True on success.
 	 */
-	bool updateYearNodes(const QString &userName, enum NodeType nodeType,
-	    const QList< QPair<QString, unsigned> > &yearlyUnreadList,
-	    enum Sorting sorting);
+	bool updateYearNodes(const AcntId &acntId, enum NodeType nodeType,
+	    const QList< QPair<QString, YearCounter> > &yearlyUnreadList,
+	    enum Sorting sorting, bool prohibitYearRemoval = false);
 
 	/*!
 	 * @brief Update existing year node in account.
 	 *
-	 * @param[in] userName   User name.
-	 * @param[in] nodeType   May be nodeReceivedYear or nodeSentYear.
-	 * @param[in] year       Year string.
-	 * @param[in] unreadMsgs Number of unread messages.
+	 * @param[in] acntId Account identifier.
+	 * @param[in] nodeType May be nodeReceivedYear or nodeSentYear.
+	 * @param[in] year Year string.
+	 * @param[in] yCounter Whether database is opened and number of unread messages.
 	 * @return True on success.
 	 */
-	bool updateYear(const QString &userName, enum NodeType nodeType,
-	    const QString &year, unsigned unreadMsgs = 0);
+	bool updateYear(const AcntId &acntId, enum NodeType nodeType,
+	    const QString &year, const YearCounter &yCounter);
 
 	/*!
 	 * @brief Delete all year-related nodes in model.
@@ -316,21 +412,54 @@ public:
 	/*!
 	 * @brief Move related data by given number of of positions.
 	 *
-	 * @param[in] userName User name.
-	 * @param[in] shunt    Amount of positions the account should be moved.
-	 *                     Negative values move towards begin positive to
-	 *                     the end.
+	 * @param[in] acntId Account identifier.
+	 * @param[in] shunt Amount of positions the account should be moved.
+	 *                  Negative values move towards begin positive to
+	 *                  the end.
 	 * @return True on success.
 	 */
-	bool changePosition(const QString &userName, int shunt);
+	bool changePosition(const AcntId &acntId, int shunt);
 
-private slots:
+private Q_SLOTS:
 	/*!
-	 * @brief This slot handles changes of account data.
-	 *
-	 * @param[in] userName User name defining the changed account.
+	 * @brief Notify before an account is going to be added.
 	 */
-	void handleAccountDataChange(const QString &userName);
+	void watchAccountAboutToBeAdded(const AcntId &acntId, int row);
+
+	/*!
+	 * @brief Update the model after an account has been added.
+	 */
+	void watchAccountAdded(const AcntId &acntId, int row);
+
+	/*!
+	 * @brief Notify before an account is going to be removed.
+	 */
+	void watchAccountAboutToBeRemoved(const AcntId &acntId, int row);
+
+	/*!
+	 * @brief Update the model after an account has been removed.
+	 */
+	void watchAccountRemoved(const AcntId &acntId, int row);
+
+	/*!
+	 * @brief Notify before accounts are going to be moved.
+	 */
+	void watchAccountsAboutToBeMoved(int srcFrst, int srcLst, int dst);
+
+	/*!
+	 * @brief Update the model after accounts have been moved.
+	 */
+	void watchAccountsMoved(int srcFrst, int srcLst, int dst);
+
+	/*!
+	 * @brief This slot handles changes of username.
+	 */
+	void watchUsernameChanged(const AcntId &oldId, const AcntId &newId, int index);
+
+	/*!
+	 * @brief Update the model after account has been modified.
+	 */
+	void watchAcountChanged(const AcntId &acntId);
 
 private:
 	/*!
@@ -364,43 +493,14 @@ private:
 	/*!
 	 * @brief Returns the row of the top node related to the account.
 	 *
-	 * @param[in] userName Sought user name.
+	 * @param[in] acntId Account identifier.
 	 * @return Top node row or -1 if no such name found.
 	 */
-	int topAcntRow(const QString &userName) const;
+	int topAcntRow(const AcntId &acntId) const;
 
-	/*!
-	 * @brief Determines node type by traversing node structure.
-	 *
-	 * @param[in] index Data index.
-	 * @return Node type related to the supplied index.
-	 */
-	static
-	enum NodeType nodeTypeTraversed(const QModelIndex &index);
+	AccountsMap *m_accountsPtr; /*!< Pointer to account data container. */
 
-	/*
-	 * Model indexes hold a value that is effectively the index into this
-	 * list. This value serves for accessing model data. Therefore, the
-	 * ordering of the list must not change during the lifespan of the
-	 * model.
-	 *
-	 * Should an entry be deleted then only the value at the corresponding
-	 * position should be set to a null string.
-	 * Addition of new entries is performed by appending new values.
-	 */
-	QList<QString> m_userNames; /*!<
-	                             * List of user names that are used to
-	                             * access the global accounts.
-	                             */
-	/*
-	 * The index into the mapping list is the actual top account row.
-	 * The value stored in the field is the index into the list of user
-	 * names.
-	 *
-	 * The size of the list is the actual number of account stored in the
-	 * model.
-	 */
-	QList<int> m_row2UserNameIdx; /*!< Mapping from row to user name. */
+	AcntNumId m_numIdMapping; /*!< Provides int <-> AcntId mapping for this model. */
 
 	/*!
 	 * @brief Holds additional information about the displayed data.
@@ -419,13 +519,11 @@ private:
 
 		unsigned unreadRecentReceived; /*!< Number of unread recent received messages. */
 		unsigned unreadRecentSent; /*!< Number of unread recent sent messages. */
-		QList<QString> receivedGroups; /*!< Group names and numbers of unread messages. */
+		QList<QString> receivedGroups; /*!< Groups of unread messages. */
 		QList<QString> sentGroups; /*!< Groups of unread messages. */
-		QMap<QString, unsigned> unreadReceivedGroups; /*< Number of unread messages. */
-		QMap<QString, unsigned> unreadSentGroups; /*!< Number of unread messages. */
+		QMap<QString, YearCounter> unreadReceivedGroups; /*< Number of unread messages. */
+		QMap<QString, YearCounter> unreadSentGroups; /*!< Number of unread messages. */
 	};
 
-	QMap<QString, AccountCounters> m_countersMap; /*!< Unread counters. */
+	QMap<AcntId, AccountCounters> m_countersMap; /*!< Unread counters. */
 };
-
-#endif /* _ACCOUNTS_MODEL_H_ */
