@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,8 +21,7 @@
  * the two.
  */
 
-#ifndef _MESSAGES_MODEL_H_
-#define _MESSAGES_MODEL_H_
+#pragma once
 
 #include <QIcon>
 #include <QList>
@@ -30,10 +29,18 @@
 #include <QVariant>
 #include <QVector>
 
-#include "src/common.h" /* enum MessageProcessState */
+#include "src/identifiers/message_id.h"
 #include "src/io/message_db.h"
 #include "src/models/table_model.h"
 
+namespace Json {
+	class Int64StringList; /* Forward declaration. */
+	class TagAssignmentHash; /* Forward declaration. */
+	class TagEntry; /* Forward declaration. */
+	class TagEntryList; /* Forward declaration. */
+	class TagMsgIdList; /* Forward declaration. */
+}
+
 /*!
  * @brief Custom message model class.
  *
@@ -45,20 +52,47 @@ class DbMsgsTblModel : public TblModel {
     Q_OBJECT
 
 public:
+	/*!
+	 * @brief Additional roles.
+	 */
+	enum UserRoles {
+		ROLE_PLAIN_DISPLAY = (Qt::UserRole + 1), /*!< Low level data. */
+		ROLE_DB_ENTRY_TYPE, /*!< Used to determine the database data type of the column. */
+		ROLE_PROXYSORT /*!< Allows fine-grained sorting of boolean values which may be displayed as icons or check boxes. */
+	};
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 5, 0))
+	Q_ENUM(UserRoles)
+#else /* < Qt-5.5 */
+	Q_ENUMS(UserRoles)
+#endif /* >= Qt-5.5 */
+
 	/*!
 	 * @brief Identifies the column index.
+	 *
+	 * @Note Non-negative values are directly derived from the message
+	 *     database. Columns identified by negative values are added and
+	 *     are kept in separate database files.
+	 *
+	 * @todo Processing state should be separated from the message database
+	 *     and should be kept in a separate database in a similar fashion
+	 *     as tags or records management information.
 	 */
 	enum ColumnNumbers {
 		DMID_COL = 0, /* Message identifier. */
-		ANNOT_COL = 1, /* Annotation column. */
-		/* 2 stands for sender or recipient name. */
-		DELIVERY_COL = 3, /* Delivery time column. */
-		ACCEPT_COL = 4, /* Acceptance time column. */
-		READLOC_COL = 5, /* Read locally. */
-		ATTDOWN_COL = 6, /* Attachments downloaded. */
-		PROCSNG_COL = 7, /* Processing state. */
-		REC_MGMT_NEG_COL = -2, /* Records management service. */
-		TAGS_NEG_COL = -1 /* Tags. */
+		PERSDELIV_COL = 1, /* Personal delivery. */
+		ANNOT_COL = 2, /* Annotation column. */
+		SENDER_COL = 3, /* Sender name column. */
+		RECIP_COL = 4, /* Recipient name column. */
+		DELIVERY_COL = 5, /* Delivery time column. */
+		ACCEPT_COL = 6, /* Acceptance time column. */
+		READLOC_COL = 7, /* Read locally. */
+		MSGSTAT_COL = 8, /* Message status. */
+		ATTDOWN_COL = 9, /* Attachments downloaded. */
+		PROCSNG_COL = 10, /* Processing state. */
+		RECMGMT_NEG_COL = -2, /* 11 */ /* Records management service. */
+		TAGS_NEG_COL = -1, /* 12 */ /* Tags. */
+		BASIC_COLNUM = 11, /* Number of basic columns (convenience value). */
+		MAX_COLNUM = 13 /* Maximal number of columns (convenience value). */
 	};
 
 	/*!
@@ -67,8 +101,8 @@ public:
 	 * @note Dummies are used to fake empty models.
 	 */
 	enum Type {
-		WORKING_RCVD = 0, /*!< Ordinary model created from SQL query result. */
-		WORKING_SNT /*!< Ordinary model created from SQL query result. */
+		RCVD_MODEL = 0, /*!< Model created from SQL query result. */
+		SNT_MODEL /*!< Model created from SQL query result. */
 	};
 
 	/*!
@@ -96,14 +130,14 @@ public:
 	 * @param[in] type   Type of the table model.
 	 * @param[in] parent Parent object.
 	 */
-	explicit DbMsgsTblModel(enum Type type = WORKING_RCVD,
+	explicit DbMsgsTblModel(enum Type type = RCVD_MODEL,
 	    QObject *parent = Q_NULLPTR);
 
 	/*!
 	 * @brief Returns the data stored under the given role.
 	 *
 	 * @param[in] index Position.
-	 * @param[in] role  Role if the position.
+	 * @param[in] role  Role of the position.
 	 * @return Data or invalid QVariant if no matching data found.
 	 */
 	virtual
@@ -123,21 +157,21 @@ public:
 	    int role = Qt::DisplayRole) const Q_DECL_OVERRIDE;
 
 	/*!
-	 * @brief Appends received entry data into the model.
+	 * @brief Set received entry data into the model.
 	 *
 	 * @param[in] entryList List of entries to append into the model.
 	 * @param[in] appendedColsNum Number of added empty columns.
 	 */
-	void appendData(const QList<MessageDb::RcvdEntry> &entryList,
+	void assignData(const QList<MessageDb::RcvdEntry> &entryList,
 	    int appendedColsNum);
 
 	/*!
-	 * @brief Appends sent entry data into the model.
+	 * @brief Set sent entry data into the model.
 	 *
 	 * @param[in] entryList List of entries to append into the model.
 	 * @param[in] appendedColsNum Number of added empty columns.
 	 */
-	void appendData(const QList<MessageDb::SntEntry> &entryList,
+	void assignData(const QList<MessageDb::SntEntry> &entryList,
 	    int appendedColsNum);
 
 	/*!
@@ -151,18 +185,26 @@ public:
 	bool setType(enum Type type);
 
 	/*!
-	 * @brief Set header data for received model.
+	 * @brief Get model type.
 	 *
-	 * @return False on error.
+	 * @return Model type.
 	 */
-	bool setRcvdHeader(const QList<AppendedCol> &appendedCols);
+	enum Type type(void) const;
 
 	/*!
-	 * @Brief Set header data for sent model.
+	 * @brief Set header data.
 	 *
 	 * @return False on error.
 	 */
-	bool setSntHeader(const QList<AppendedCol> &appendedCols);
+	bool setHeader(const QList<AppendedCol> &appendedCols);
+
+	/*!
+	 * @brief Remove row containing entry with corresponding ID.
+	 *
+	 * @param[in] dmId Message ID.
+	 * @return True on success.
+	 */
+	bool remove(qint64 dmId);
 
 	/*!
 	 * @brief Override message as being read.
@@ -175,6 +217,17 @@ public:
 	 */
 	bool overrideRead(qint64 dmId, bool forceRead = true);
 
+	/*!
+	 * @brief Override message as being read.
+	 *
+	 * @note Emits dataChanged signal.
+	 *
+	 * @param[in] msgIds Message identifiers.
+	 * @param[in] forceRead Set whether to force read state.
+	 * @return True on success.
+	 */
+	bool overrideRead(const QList<MsgId> &msgIds, bool forceRead = true);
+
 	/*!
 	 * @brief Override message as having its attachments having downloaded.
 	 *
@@ -197,29 +250,60 @@ public:
 	 * @return True on success.
 	 */
 	bool overrideProcessing(qint64 dmId,
-	    enum MessageProcessState forceState);
+	    enum MessageDb::MessageProcessState forceState);
+
+	/*!
+	 * @brief Override message processing state.
+	 *
+	 * @note Emits dataChanged signal.
+	 *
+	 * @param[in] msgIds Message identifiers.
+	 * @param[in] forceState Set forced value.
+	 * @return True on success.
+	 */
+	bool overrideProcessing(const QList<MsgId> &msgIds,
+	    enum MessageDb::MessageProcessState forceState);
+
+	/*!
+	 * @brief Get listing of all message identifiers in the model.
+	 *
+	 * @param[in] testEnv testing environment.
+	 * @return List of message identifiers.
+	 */
+	Json::TagMsgIdList msgIdList(bool testEnv) const;
 
 	/*!
 	 * @brief Fills the model with tag information.
 	 *
-	 * @param[in] userName Account user name.
-	 * @param[in] col      Negative number specifying the column to write
-	 *                     into.
+	 * @param[in] testEnv testing environment.
+	 * @param[in] col Negative number specifying the column to write into.
+	 * @param[in] assignments Tag assignments.
 	 * @return True on success.
 	 */
-	bool fillTagsColumn(const QString &userName, int col);
+	bool setTagAssignments(bool testEnv, int col,
+	    const Json::TagAssignmentHash &assignments);
 
 	/*!
-	 * @brief Reload tags in given rows.
+	 * @brief Update all occurrences of tag with matching identifier to new
+	 *     content.
 	 *
-	 * @param[in] userName Account user name.
-	 * @param[in] dmIds    List of message ids for which to load tags.
-	 * @param[in] col      Negative number specifying the column to write
-	 *                     into.
-	 * @return True on success.
+	 * @param[in] entry New tag content.
 	 */
-	bool refillTagsColumn(const QString &userName,
-	    const QList<qint64> &dmIds, int col);
+	void updateTag(const Json::TagEntry &entry);
+
+	/*!
+	 * @brief Delete all occurrences of tags with matching identifiers.
+	 *
+	 * @param[in] ids Identifiers of tags to be deleted.
+	 */
+	void deleteTags(const Json::Int64StringList &ids);
+
+	/*!
+	 * @brief Set new tag entries for given message.
+	 * @param[in] dmId Message id for which to set the tags.
+	 * @param[in] entries New tag entries.
+	 */
+	void updateTagAssignment(qint64 dmId, const Json::TagEntryList &entries);
 
 	/*!
 	 * @brief Sets records management service notification icon.
@@ -227,6 +311,20 @@ public:
 	 */
 	bool setRecordsManagementIcon(void);
 
+	/*!
+	 * @brief Set amount of days to show message deletion notification.
+	 *
+	 * @param[in] days Number of days to show deletion notification.
+	 */
+	void setMsgDeletionNotifyAheadDays(int days);
+
+	/*!
+	 * @brief Set colour to show message deletion notification.
+	 *
+	 * @param[in] colour Colour to be used.
+	 */
+	void setMsgDeletionNotifyAheadColour(const QColor &colour);
+
 	/*!
 	 * @brief Fills the model with records management service information.
 	 *
@@ -238,29 +336,46 @@ public:
 	/*!
 	 * @brief Reload records management service information in given rows.
 	 *
-	 * @param[in] dmIds List of message ids for which to load tags.
+	 * @param[in] dmId Message id for which to update data.
+	 * @param[in] locations Locations in the records management hierarchy.
 	 * @param[in] col Negative number specifying the column to write into.
 	 * @return True on success.
 	 */
-	bool refillRecordsManagementColumn(const QList<qint64> &dmIds, int col);
+	bool updateRecordsManagementColumn(qint64 dmId,
+	    const QStringList &locations, int col);
 
 private:
-	enum Type m_type; /*!<
-	                   * Whether this is a model dummy or contains data.
-	                   */
+	/*!
+	 * @brief Return records management model data.
+	 *
+	 * @param[in] index Position.
+	 * @param[in] role Role of the position.
+	 * @return Data to be displayed.
+	 */
+	QVariant recMgmtData(const QModelIndex &index, int role) const;
 
-	static
-	const int rcvdMsgsColCnt; /*<
-	                           * Number of columns when displaying received
-	                           * messages (without added columns).
-	                           */
-	static
-	const int sntMsgsColCnt; /*!<
-	                          * Number of columns when displaying sent
-	                          * messages (without added columns).
-	                          */
+	/*!
+	 * @brief Return tags model data.
+	 *
+	 * @param[in] index Position.
+	 * @param[in] role Role of the position.
+	 * @return Data to be displayed.
+	 */
+	QVariant tagsData(const QModelIndex &index, int role) const;
 
-	QIcon m_dsIco; /*!< Records management icon. */
+	/*!
+	 * @brief Get message ID related to supplied index.
+	 *
+	 * @param[in] index Index which specifies the row used to take message
+	 *                  identifier from.
+	 * @return Message ID.
+	 */
+	qint64 getDmId(const QModelIndex &index) const;
+
+	enum Type m_type; /*!< Specifies type of data held in the model. */
+
+	QIcon m_rmIco; /*!< Records management icon. */
+
+	int m_msgDltNotifyAheadDays; /*!< Amount of days to show message deletion notification. */
+	QColor m_msgDltNotifyColour; /*!< Colour to be used to highlight messages to be deleted. */
 };
-
-#endif /* _MESSAGES_MODEL_H_ */
