@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,8 +21,7 @@
  * the two.
  */
 
-#ifndef _TABLE_MODEL_H_
-#define _TABLE_MODEL_H_
+#pragma once
 
 #include <QAbstractTableModel>
 #include <QMap>
@@ -30,6 +29,9 @@
 #include <QVariant>
 #include <QVector>
 
+/*!
+ * @brief Custom table model.
+ */
 class TblModel : public QAbstractTableModel {
     Q_OBJECT
 
@@ -124,6 +126,11 @@ public:
 	bool removeRows(int row, int count,
 	    const QModelIndex &parent = QModelIndex()) Q_DECL_OVERRIDE;
 
+	/*!
+	 * @brief Clears the model content.
+	 */
+	void removeAllRows(void);
+
 protected:
 	/*!
 	 * @brief Sets the content of the model according to the supplied query.
@@ -211,5 +218,3 @@ private:
 	 */
 	QMap< int, QMap<int, QVariant> > m_headerData; /*!< Header data. */
 };
-
-#endif /* _TABLE_MODEL_H_ */
