@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -25,19 +25,17 @@
 #include <QUrl>
 
 #include "src/common.h"
-#include "src/log/log.h"
+#include "src/datovka_shared/compat/compiler.h" /* macroStdMove */
+#include "src/datovka_shared/log/log.h"
+#include "src/datovka_shared/settings/prefs.h"
 #include "src/settings/proxy.h"
 
 #define HTTP_PROXY_VARMAME "http_proxy"
 #define HTTPS_PROXY_VARMAME "https_proxy"
 
-ProxiesSettings globProxSet;
-
 #define NO_PROXY_STR "None"
 #define AUTO_PROXY_STR "-1"
 
-#define NO_PORT -1
-
 const QByteArray ProxiesSettings::httpProxyEnvVar(qgetenv(HTTP_PROXY_VARMAME));
 const QByteArray ProxiesSettings::httpsProxyEnvVar(qgetenv(HTTPS_PROXY_VARMAME));
 
@@ -45,8 +43,8 @@ ProxiesSettings::ProxySettings::ProxySettings(void)
     : usage(NO_PROXY), /* Default is no proxy. */
     userName(),
     password(),
-    hostName(QLatin1String(NO_PROXY_STR)),
-    port(NO_PORT)
+    hostName(NO_PROXY_STR),
+    port(PROXY_NO_PORT)
 {
 }
 
@@ -88,30 +86,24 @@ ProxiesSettings::ProxiesSettings(void)
  * @brief Set up configuration strings.
  *
  * @param[in] type Whether to use HTTP or HTTPS settings.
- * @param[in] full Whether to prefix settings group name.
  * @param[out] connProxy Proxy entry name.
  * @param[out] connProxyUser Proxy user name entry.
  * @param[out] connProxyPwd Proxy password entry.
  */
 static
-void settingsStringSetUp(enum ProxiesSettings::Type type, bool full,
+void settingsStringSetUp(enum ProxiesSettings::Type type,
     QString &connProxy, QString &connProxyUser, QString &connProxyPwd)
 {
-	QString prefix;
-	if (full) {
-		prefix = QLatin1String("connection/");
-	}
-
 	switch (type) {
 	case ProxiesSettings::HTTP:
-		connProxy = prefix + QLatin1String("http_proxy");
-		connProxyUser = prefix + QLatin1String("http_proxy_username");
-		connProxyPwd = prefix + QLatin1String("http_proxy_password");
+		connProxy = QStringLiteral("network.proxy.http");
+		connProxyUser = QStringLiteral("network.proxy.http.username");
+		connProxyPwd = QStringLiteral("network.proxy.http.password.base64");
 		break;
 	case ProxiesSettings::HTTPS:
-		connProxy = prefix + QLatin1String("https_proxy");
-		connProxyUser = prefix + QLatin1String("https_proxy_username");
-		connProxyPwd = prefix + QLatin1String("https_proxy_password");
+		connProxy = QStringLiteral("network.proxy.https");
+		connProxyUser = QStringLiteral("network.proxy.https.username");
+		connProxyPwd = QStringLiteral("network.proxy.https.password.base64");
 		break;
 	default:
 		Q_ASSERT(0);
@@ -120,26 +112,27 @@ void settingsStringSetUp(enum ProxiesSettings::Type type, bool full,
 }
 
 /*!
- * @brief Reads proxy configuration from settings,
+ * @brief Reads proxy configuration from preferences.
  *
- * @param[in] settings Settings structure.
+ * @param[in] prefs Preferences.
  * @param[in] type Whether to read HTTP or HTTPS proxy settings,
  * @return Proxy settings structure.
  */
 static
-ProxiesSettings::ProxySettings loadProxySettings(const QSettings &settings,
+ProxiesSettings::ProxySettings loadProxySettings(const Prefs &prefs,
     enum ProxiesSettings::Type type)
 {
 	QString connProxy, connProxyUser, connProxyPwd;
-	settingsStringSetUp(type, true, connProxy, connProxyUser, connProxyPwd);
+	settingsStringSetUp(type, connProxy, connProxyUser, connProxyPwd);
 
 	ProxiesSettings::ProxySettings proxy;
 
-	QString auxStr = settings.value(connProxy).toString();
-	if (auxStr.isEmpty() || (auxStr == QLatin1String(NO_PROXY_STR))) {
+	QString auxStr;
+	prefs.strVal(connProxy, auxStr);
+	if (auxStr.isEmpty() || (auxStr == QStringLiteral(NO_PROXY_STR))) {
 		/* Defaults. */
 		proxy.usage = ProxiesSettings::ProxySettings::NO_PROXY;
-	} else if (auxStr == QLatin1String(AUTO_PROXY_STR)) {
+	} else if (auxStr == QStringLiteral(AUTO_PROXY_STR)) {
 		proxy.usage = ProxiesSettings::ProxySettings::AUTO_PROXY;
 	} else if (auxStr.contains(":")) {
 		QString hostName(auxStr.section(":", 0, -2));
@@ -147,7 +140,7 @@ ProxiesSettings::ProxySettings loadProxySettings(const QSettings &settings,
 		int port = auxStr.section(":", -1, -1).toInt(&ok, 10);
 		if (ok) {
 			proxy.usage = ProxiesSettings::ProxySettings::DEFINED_PROXY;
-			proxy.hostName = hostName;
+			proxy.hostName = macroStdMove(hostName);
 			proxy.port = port;
 		}
 	} else {
@@ -155,71 +148,68 @@ ProxiesSettings::ProxySettings loadProxySettings(const QSettings &settings,
 		proxy.usage = ProxiesSettings::ProxySettings::NO_PROXY;
 	}
 
-	proxy.userName = settings.value(connProxyUser).toString();
-	proxy.password = fromBase64(settings.value(connProxyPwd).toString());
+	QString val;
+	prefs.strVal(connProxyUser, val);
+	proxy.userName = val;
+	val.clear();
+	prefs.strVal(connProxyPwd, val);
+	proxy.password = fromBase64(val);
 
 	return proxy;
 }
 
-void ProxiesSettings::loadFromSettings(const QSettings &settings)
+void ProxiesSettings::loadFromPrefs(const Prefs &prefs)
 {
-	http = loadProxySettings(settings, HTTP);
-	https = loadProxySettings(settings, HTTPS);
+	http = loadProxySettings(prefs, HTTP);
+	https = loadProxySettings(prefs, HTTPS);
 }
 
 /*!
- * @brief Writes proxy configuration to settings.
+ * @brief Writes proxy configuration to preferences.
  *
- * @param[out] settings Settings structure.
+ * @param[out] prefs Preferences.
  * @param[in] type Whether to read HTTP or HTTPS proxy settings,
  * @param[in] proxy Proxy settings structure.
  */
 static
-void saveProxySettings(QSettings &settings, enum ProxiesSettings::Type type,
+void saveProxySettings(Prefs &prefs, enum ProxiesSettings::Type type,
     const ProxiesSettings::ProxySettings &proxy)
 {
 	QString connProxy, connProxyUser, connProxyPwd;
-	settingsStringSetUp(type, false, connProxy, connProxyUser,
-	    connProxyPwd);
+	settingsStringSetUp(type, connProxy, connProxyUser, connProxyPwd);
 
 	switch (proxy.usage) {
 	case ProxiesSettings::ProxySettings::NO_PROXY:
 		/* No proxy is default. */
-		//settings.setValue(connProxy, QLatin1String(NO_PROXY_STR));
+		//prefs.setStrVal(connProxy, QStringLiteral(NO_PROXY_STR));
+		prefs.setStrVal(connProxy, QString());
 		return;
 		break;
 	case ProxiesSettings::ProxySettings::AUTO_PROXY:
-		settings.setValue(connProxy, QLatin1String(AUTO_PROXY_STR));
+		prefs.setStrVal(connProxy, QStringLiteral(AUTO_PROXY_STR));
 		break;
 	case ProxiesSettings::ProxySettings::DEFINED_PROXY:
 		if (proxy.hostName.isEmpty() ||
 		    proxy.port < 0 || proxy.port > 65535) {
 			/* Default to no proxy. */
+			prefs.setStrVal(connProxy, QString());
 			return;
 		}
-		settings.setValue(connProxy, proxy.hostName + ":" + QString::number(proxy.port, 10));
+		prefs.setStrVal(connProxy, proxy.hostName + ":" + QString::number(proxy.port, 10));
 		break;
 	default:
 		Q_ASSERT(0);
 		break;
 	}
 
-	if (!proxy.userName.isEmpty()) {
-		settings.setValue(connProxyUser, proxy.userName);
-	}
-	if (!proxy.password.isEmpty()) {
-		settings.setValue(connProxyPwd, toBase64(proxy.password));
-	}
+	prefs.setStrVal(connProxyUser, proxy.userName);
+	prefs.setStrVal(connProxyPwd, toBase64(proxy.password));
 }
 
-void ProxiesSettings::saveToSettings(QSettings &settings) const
+void ProxiesSettings::saveToPrefs(Prefs &prefs) const
 {
-	settings.beginGroup("connection");
-
-	saveProxySettings(settings, HTTP, http);
-	saveProxySettings(settings, HTTPS, https);
-
-	settings.endGroup();
+	saveProxySettings(prefs, HTTP, http);
+	saveProxySettings(prefs, HTTPS, https);
 }
 
 QByteArray ProxiesSettings::detectEnvironment(enum ProxiesSettings::Type type)
@@ -239,8 +229,8 @@ QByteArray ProxiesSettings::detectEnvironment(enum ProxiesSettings::Type type)
 
 #if !defined(Q_OS_UNIX) || defined(Q_OS_MAC)
 	QNetworkProxyQuery npq(QUrl((type == HTTP) ?
-	    QLatin1String("http://www.nic.cz") :
-	    QLatin1String("https://www.nic.cz")));
+	    QStringLiteral("http://www.nic.cz") :
+	    QStringLiteral("https://www.nic.cz")));
 
 	QList<QNetworkProxy> listOfProxies(
 	    QNetworkProxyFactory::systemProxyForQuery(npq));
@@ -251,7 +241,7 @@ QByteArray ProxiesSettings::detectEnvironment(enum ProxiesSettings::Type type)
 		    "Multiple HTTPS proxies detected. Using first.");
 	}
 
-	foreach (const QNetworkProxy &p, listOfProxies) {
+	for (const QNetworkProxy &p : listOfProxies) {
 		if (!p.hostName().isEmpty()) {
 			proxyEnvVar = p.hostName().toUtf8();
 			proxyEnvVar += ":" + QByteArray::number(p.port(), 10);
@@ -270,7 +260,16 @@ QByteArray ProxiesSettings::detectEnvironment(enum ProxiesSettings::Type type)
 	return proxyEnvVar;
 }
 
-bool ProxiesSettings::addUserPwdIfMissing(QByteArray &envVal,
+/*!
+ * @brief Adds user and password, if not specified in environment value.
+ *
+ * @param[in,out] envVal Environment value.
+ * @param[in]     user User name.
+ * @param[in]     pwd User password.
+ * @return True if added, false else.
+ */
+static
+bool addUserPwdIfMissing(QByteArray &envVal,
     const QString &user, const QString &pwd)
 {
 	int aux;
@@ -279,7 +278,7 @@ bool ProxiesSettings::addUserPwdIfMissing(QByteArray &envVal,
 		return false;
 	}
 
-	if (user.isEmpty() || pwd.isEmpty()) {
+	if (Q_UNLIKELY(user.isEmpty() || pwd.isEmpty())) {
 		return false;
 	}
 
@@ -320,7 +319,7 @@ QByteArray adjustedDetectedEnvironment(enum ProxiesSettings::Type type,
 	}
 
 	if (!proxy.userName.isEmpty() && !proxy.password.isEmpty()) {
-		bool ret = ProxiesSettings::addUserPwdIfMissing(val,
+		bool ret = addUserPwdIfMissing(val,
 		    proxy.userName, proxy.password);
 		if (ret) {
 			logWarningNL(
@@ -344,7 +343,7 @@ bool setProxyEnvVar(enum ProxiesSettings::Type type,
     const ProxiesSettings::ProxySettings &proxy)
 {
 	const char *proxyVarName = NULL;
-	const QByteArray *proxyStartUp = 0;
+	const QByteArray *proxyStartUp = Q_NULLPTR;
 	switch (type) {
 	case ProxiesSettings::HTTP:
 		proxyVarName = HTTP_PROXY_VARMAME;
@@ -360,7 +359,7 @@ bool setProxyEnvVar(enum ProxiesSettings::Type type,
 		break;
 	}
 	Q_ASSERT(proxyVarName != NULL);
-	Q_ASSERT(proxyStartUp != 0);
+	Q_ASSERT(proxyStartUp != Q_NULLPTR);
 
 	QByteArray proxyOld(qgetenv(proxyVarName));
 
@@ -418,7 +417,7 @@ static
 ProxiesSettings::ProxySettings fromEnvVal(QByteArray proxyEnv,
     ProxiesSettings::ProxySettings::Usage usage)
 {
-	ProxiesSettings::ProxySettings settings; /* noProxyStr, NO_PORT */
+	ProxiesSettings::ProxySettings settings; /* noProxyStr, PROXY_NO_PORT */
 
 	int aux;
 
@@ -477,7 +476,7 @@ ProxiesSettings::ProxySettings fromEnvVal(QByteArray proxyEnv,
 ProxiesSettings::ProxySettings ProxiesSettings::proxySettings(
     enum ProxiesSettings::Type type) const
 {
-	const ProxiesSettings::ProxySettings *proxy = 0;
+	const ProxiesSettings::ProxySettings *proxy = Q_NULLPTR;
 
 	switch (type) {
 	case HTTP:
@@ -491,7 +490,7 @@ ProxiesSettings::ProxySettings ProxiesSettings::proxySettings(
 		return ProxiesSettings::ProxySettings();
 		break;
 	}
-	Q_ASSERT(proxy != 0);
+	Q_ASSERT(proxy != Q_NULLPTR);
 
 	ProxiesSettings::ProxySettings returnedVal;
 
@@ -515,3 +514,30 @@ ProxiesSettings::ProxySettings ProxiesSettings::proxySettings(
 
 	return returnedVal;
 }
+
+QNetworkProxy ProxiesSettings::networkProxy(enum Type type) const
+{
+	const ProxySettings pSettings = proxySettings(type);
+
+	/* If something detected or set up. */
+	if (ProxiesSettings::ProxySettings::NO_PROXY != pSettings.usage) {
+		QNetworkProxy proxy;
+
+		proxy.setHostName(pSettings.hostName);
+		proxy.setPort(pSettings.port);
+//		proxy.setType(QNetworkProxy::DefaultProxy);
+//		proxy.setType(QNetworkProxy::Socks5Proxy);
+		proxy.setType(QNetworkProxy::HttpProxy);
+		if (!pSettings.userName.isEmpty()) {
+			proxy.setUser(pSettings.userName);
+		}
+		if (!pSettings.password.isEmpty()) {
+			proxy.setPassword(pSettings.password);
+		}
+
+		return proxy;
+	}
+
+	/* Return default proxy. */
+	return QNetworkProxy();
+}
