@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,12 +21,15 @@
  * the two.
  */
 
-#ifndef _PROXY_H_
-#define _PROXY_H_
+#pragma once
 
-#include <QSettings>
+#include <QNetworkProxy>
 #include <QString>
 
+#define PROXY_NO_PORT -1
+
+class Prefs; /* Forward declaration. */
+
 class ProxiesSettings {
 public:
 	/*!
@@ -60,7 +63,7 @@ public:
 
 		enum Usage usage; /*!< How to interpret the proxy data.  */
 
-		QString userName; /*!< Proxy user name. */
+		QString userName; /*!< Proxy username. */
 		QString password; /*!< Proxy password. */
 
 		QString hostName; /*!< Proxy host. */
@@ -83,22 +86,19 @@ public:
 	 */
 	ProxiesSettings(void);
 
-	ProxySettings https; /*!< HTTPS proxy settings. */
-	ProxySettings http; /*!< HTTP proxy settings. */
+	/*!
+	 * @brief Load data from supplied preferences.
+	 *
+	 * @param[in] prefs Preferences.
+	 */
+	void loadFromPrefs(const Prefs &prefs);
 
 	/*!
-	 * @brief Load data from supplied settings.
+	 * @brief Store data to preferences.
 	 *
-	 * @param[in] settings Settings structure.
+	 * @param[out] prefs Preferences.
 	 */
-	void loadFromSettings(const QSettings &settings);
-
-	/*!
-	 * @brief Store data to settings structure.
-	 *
-	 * @param[out] settings Settings structure.
-	 */
-	void saveToSettings(QSettings &settings) const;
+	void saveToPrefs(Prefs &prefs) const;
 
 	/*
 	 * Libcurl automatically uses the variables http_proxy and https_proxy
@@ -112,7 +112,7 @@ public:
 	 *
 	 * @note Takes values from stored environmental values or from system.
 	 *
-	 * @param[in] type Whether to obtain HPPT or HTTPS settings.
+	 * @param[in] type Whether to obtain HTTP or HTTPS settings.
 	 * @returns Values as they would be stored in a http(s)_proxy variable (e.g.
 	 *     'http://a:aaa@127.0.0.1:3128', 'b:bb@192.168.1.1:3128',
 	 *     '172.0.0.1:3128').
@@ -120,18 +120,6 @@ public:
 	static
 	QByteArray detectEnvironment(enum Type type);
 
-	/*!
-	 * @brief Adds user and password, if not specified in environment value.
-	 *
-	 * @param[in,out] envVal Environment value.
-	 * @param[in]     user User name.
-	 * @param[in]     pwd User password.
-	 * @return True if added, false else.
-	 */
-	static
-	bool addUserPwdIfMissing(QByteArray &envVal, const QString &user,
-	    const QString &pwd);
-
 	/*!
 	 * @brief Sets environmental variables according to proxy settings.
 	 *
@@ -142,14 +130,20 @@ public:
 	/*!
 	 * @brief Returns proxy settings as they are configures.
 	 *
+	 * @param[in] type Whether to obtain HTTP or HTTPS settings.
 	 * @return Proxy settings.
 	 */
 	ProxySettings proxySettings(enum Type type) const;
+
+	/*!
+	 * @brief Returns network proxy object according to stored settings.
+	 *
+	 * @param[in] type Whether to obtain HTTP or HTTPS settings.
+	 * @return QNetworkProxy::HttpProxy proxy if proxy set,
+	 *     QNetworkProxy::DefaultProxy otherwise.
+	 */
+	QNetworkProxy networkProxy(enum Type type) const;
+
+	ProxySettings https; /*!< HTTPS proxy settings. */
+	ProxySettings http; /*!< HTTP proxy settings. */
 };
-
-/*!
- * @brief Global instance of the structure.
- */
-extern ProxiesSettings globProxSet;
-
-#endif /* _PROXY_H_ */
