@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -25,7 +25,9 @@
 #include <QDropEvent>
 #include <QProxyStyle>
 
-#include "src/models/files_model.h"
+#include "src/datovka_shared/log/log.h"
+#include "src/models/attachments_model.h"
+#include "src/models/progress_proxy_model.h"
 #if 0
 #include <QApplication>
 #include <QDrag>
@@ -34,12 +36,11 @@
 #include <QPixmap>
 
 #include "src/common.h"
-#include "src/log/log.h"
 #include "src/models/files_model.h"
 #endif
-#include "src/log/log.h"
 #include "src/views/attachment_table_view.h"
 
+#if 0
 /*!
  * @brief Style used to override default drop indicator.
  */
@@ -82,12 +83,20 @@ void AttachmentViewStyle::drawPrimitive(PrimitiveElement element,
 	}
 	QProxyStyle::drawPrimitive(element, option, painter, widget);
 }
+#endif
 
 AttachmentTableView::AttachmentTableView(QWidget *parent)
     : LoweredTableView(parent)
 {
-	/* Override default style. */
+#if 0
+	/*
+	 * Override default style.
+	 *
+	 * This style modification isn't used because it complicates the dark
+	 * mode style..
+	 */
 	setStyle(new AttachmentViewStyle(style()));
+#endif
 
 	setDragEnabled(true);
 }
@@ -95,13 +104,13 @@ AttachmentTableView::AttachmentTableView(QWidget *parent)
 #if 0
 void AttachmentTableView::dragEnterEvent(QDragEnterEvent *event)
 {
-	if (0 == event) {
+	if (Q_NULLPTR == event) {
 		Q_ASSERT(0);
 		return;
 	}
 
 	const QMimeData *mimeData = event->mimeData();
-	if (0 == mimeData) {
+	if (Q_NULLPTR == mimeData) {
 		Q_ASSERT(0);
 		return;
 	}
@@ -118,7 +127,7 @@ void AttachmentTableView::dragEnterEvent(QDragEnterEvent *event)
 #if 0
 void AttachmentTableView::dragMoveEvent(QDragMoveEvent *event)
 {
-	if (0 == event) {
+	if (Q_NULLPTR == event) {
 		Q_ASSERT(0);
 		return;
 	}
@@ -138,8 +147,13 @@ void AttachmentTableView::dropEvent(QDropEvent *event)
 	 */
 
 	/* TODO -- Access the model in a more transparent way. */
-	DbFlsTblModel *attachmentModel = qobject_cast<DbFlsTblModel *>(model());
-	if (attachmentModel == Q_NULLPTR) {
+	QAbstractItemModel *attachmentModel =
+	    qobject_cast<AttachmentTblModel *>(model());
+	if (Q_NULLPTR == attachmentModel) {
+		attachmentModel = qobject_cast<ProgressBarProxyModel *>(model());
+	}
+	if (Q_UNLIKELY(Q_NULLPTR == attachmentModel)) {
+		logErrorNL("%s", "Unknown model type in attachment table view.");
 		return;
 	}
 
@@ -154,7 +168,7 @@ void AttachmentTableView::dropEvent(QDropEvent *event)
 #if 0
 void AttachmentTableView::mouseMoveEvent(QMouseEvent *event)
 {
-	if (0 == event) {
+	if (Q_NULLPTR == event) {
 		Q_ASSERT(0);
 		return;
 	}
@@ -166,20 +180,23 @@ void AttachmentTableView::mouseMoveEvent(QMouseEvent *event)
 	}
 
 	/* TODO -- Access the model in a more transparent way. */
-	DbFlsTblModel *attachmentModel =
-	    qobject_cast<DbFlsTblModel *>(model());
-	if (0 == attachmentModel) {
+	QAbstractItemModel *attachmentModel =
+	    qobject_cast<AttachmentTblModel *>(model());
+	if (Q_NULLPTR == attachmentModel) {
+		attachmentModel = qobject_cast<ProgressBarProxyModel *>(model());
+	}
+	if (Q_UNLIKELY(Q_NULLPTR == attachmentModel)) {
 		return;
 	}
 
-	QDrag *drag = new (std::nothrow) QDrag(this);
-	if (0 == drag) {
+	QDrag *drag = new (::std::nothrow) QDrag(this);
+	if (Q_NULLPTR == drag) {
 		return;
 	}
 
 	QModelIndexList rowIdxs(this->selectionModel()->selectedRows(0));
 	QMimeData *mimeData = attachmentModel->mimeData(rowIdxs);
-	if (0 == mimeData) {
+	if (Q_NULLPTR == mimeData) {
 		delete drag;
 		return;
 	}
@@ -209,7 +226,7 @@ void AttachmentTableView::mouseMoveEvent(QMouseEvent *event)
 #if 0
 void AttachmentTableView::mousePressEvent(QMouseEvent *event)
 {
-	if (0 == event) {
+	if (Q_NULLPTR == event) {
 		Q_ASSERT(0);
 		return;
 	}
@@ -227,8 +244,12 @@ void AttachmentTableView::shuffleOnDrop(QDropEvent *event)
 	Q_ASSERT(event->source() == this);
 
 	/* TODO -- Access the model in a more transparent way. */
-	DbFlsTblModel *attachmentModel = qobject_cast<DbFlsTblModel *>(model());
-	if (attachmentModel == Q_NULLPTR) {
+	QAbstractItemModel *attachmentModel =
+	    qobject_cast<AttachmentTblModel *>(model());
+	if (Q_NULLPTR == attachmentModel) {
+		attachmentModel = qobject_cast<ProgressBarProxyModel *>(model());
+	}
+	if (Q_UNLIKELY(Q_NULLPTR == attachmentModel)) {
 		Q_ASSERT(0);
 		return;
 	}
@@ -251,7 +272,7 @@ void AttachmentTableView::shuffleOnDrop(QDropEvent *event)
 	}
 
 	const QModelIndexList selectedSorted(
-	    DbFlsTblModel::sortedUniqueLineIndexes(
+	    AttachmentTblModel::sortedUniqueLineIndexes(
 	        selectionModel()->selectedIndexes(), 0));
 
 	if (selectedSorted.isEmpty()) {
@@ -303,11 +324,17 @@ bool AttachmentTableView::dropOn(QDropEvent *event, int *dropRow, int *dropCol,
 		return false;
 	}
 
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+	const QPoint pos = event->position().toPoint();
+#else /* < Qt-6.0.0 */
+	const QPoint pos = event->pos();
+#endif /* >= Qt-6.0.0 */
+
 	QModelIndex index;
 	/* rootIndex() (i.e. the viewport) might be a valid index. */
-	if (viewport()->rect().contains(event->pos())) {
-		index = indexAt(event->pos());
-		if (!index.isValid() || !visualRect(index).contains(event->pos())) {
+	if (viewport()->rect().contains(pos)) {
+		index = indexAt(pos);
+		if (!index.isValid() || !visualRect(index).contains(pos)) {
 			index = rootIndex();
 		}
 	}
@@ -317,9 +344,9 @@ bool AttachmentTableView::dropOn(QDropEvent *event, int *dropRow, int *dropCol,
 		int row = -1;
 		int col = -1;
 		if (index != rootIndex()) {
-			//dropIndicatorPosition = position(event->pos(), visualRect(index), index);
+			//dropIndicatorPosition = position(pos, visualRect(index), index);
 			//switch (dropIndicatorPosition) {
-			switch (position(event->pos(), visualRect(index), index)) {
+			switch (position(pos, visualRect(index), index)) {
 			case QAbstractItemView::AboveItem:
 				row = index.row();
 				col = index.column();
@@ -375,5 +402,5 @@ QAbstractItemView::DropIndicatorPosition AttachmentTableView::position(
 		    QAbstractItemView::AboveItem : QAbstractItemView::BelowItem;
 	}
 
-    return r;
+	return r;
 }
