@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,8 +21,7 @@
  * the two.
  */
 
-#ifndef _ATTACHMENT_TABLE_VIEW_H_
-#define _ATTACHMENT_TABLE_VIEW_H_
+#pragma once
 
 #if 0
 #include <QList>
@@ -123,5 +122,3 @@ private:
 	QAbstractItemView::DropIndicatorPosition position(const QPoint &pos,
 	    const QRect &rect, const QModelIndex &index) const;
 };
-
-#endif /* _ATTACHMENT_TABLE_VIEW_H_ */
