@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,8 +21,7 @@
  * the two.
  */
 
-#ifndef _LOWERED_TABLE_VIEW_H_
-#define _LOWERED_TABLE_VIEW_H_
+#pragma once
 
 #include <QTableView>
 
@@ -44,6 +43,12 @@ public:
 	 * @brief Sets narrowed row size.
 	 */
 	void setNarrowedLineHeight(void) const;
-};
 
-#endif /* _LOWERED_TABLE_VIEW_H_ */
+	/*!
+	 * @brief Sets column width corresponding to line height in order
+	 *     to display a square-shaped icon.
+	 *
+	 * @param[in] column Column number.
+	 */
+	void setSquareColumnWidth(int column);
+};
