@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -33,6 +33,12 @@ LoweredTableWidget::LoweredTableWidget(QWidget *parent)
 
 void LoweredTableWidget::setNarrowedLineHeight(void) const
 {
+	QStyleOptionViewItem option;
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+	initViewItemOption(&option);
+#else /* < Qt-6.0 */
+	option = viewOptions();
+#endif /* >= Qt-6.0 */
 	verticalHeader()->setDefaultSectionSize(
-	    Dimensions::tableLineHeight(viewOptions()));
+	    Dimensions::tableLineHeight(option));
 }
