@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,14 +21,13 @@
  * the two.
  */
 
-#ifndef _TABLE_SPACE_SELECTION_FILTER_H_
-#define _TABLE_SPACE_SELECTION_FILTER_H_
+#pragma once
 
 #include <QObject>
 
 /*!
- * @brief This object is used as to tweak the behaviour of the QTableWidget
- *    when selecting data boxes via pressing space bar.
+ * @brief This object is used as to tweak the behaviour of the QAbstractItemView
+ *     and QTableWidget when selecting data boxes via pressing space bar.
  */
 class TableSpaceSelectionFilter : public QObject {
     Q_OBJECT
@@ -37,9 +36,10 @@ public:
 	/*!
 	 * @brief Constructor.
 	 *
+	 * @param[in] checkCol Column which should be treated as checkable.
 	 * @param[in] parent Parent object.
 	 */
-	TableSpaceSelectionFilter(QObject *parent = 0);
+	TableSpaceSelectionFilter(int checkCol, QObject *parent = Q_NULLPTR);
 
 	/*!
 	 * @brief Destructor.
@@ -50,14 +50,15 @@ public:
 	 * @brief Event filter function.
 	 *
 	 * @note The function catches the Space key and performs selection.
-	 *     It only applies to  QTableWidget objects.
+	 *     It applies to QTableView and QTableWidget objects.
 	 *
 	 * @param[in,out] object View object.
-	 * @param[in]     event  Caught event.
+	 * @param[in]     event Caught event.
 	 * @return True when filter applied.
 	 */
 	virtual
 	bool eventFilter(QObject *object, QEvent *event) Q_DECL_OVERRIDE;
-};
 
-#endif /* _TABLE_SPACE_SELECTION_FILTER_H_ */
+private:
+	const int m_checkCol; /*!< Column that should be treated as checkable. */
+};
