@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,11 +21,15 @@
  * the two.
  */
 
-#ifndef _MESSAGE_EMITTER_H_
-#define _MESSAGE_EMITTER_H_
+#pragma once
 
 #include <QObject>
 
+#include "src/identifiers/message_id.h" /* MsgId */
+
+class AcntId; /* Forward declaration. */
+class AcntIdDb; /* Forward declaration. */
+
 /*!
  * @brief Message processing status emitter.
  */
@@ -36,35 +40,24 @@ signals:
 	/*!
 	 * @brief Emitted when download message finishes.
 	 *
-	 * @param[in] usrName       Account identifier (user login name).
+	 * @param[in] acntId Account identifier.
 	 * @param[in] msgId         Message identifier.
 	 * @param[in] deliveryTime  Message delivery time.
 	 * @param[in] result        Operation outcome
 	 *                          (enum TaskDownloadMessage::Result).
 	 * @param[in] errDesc       Error description string.
-	 * @param[in] listScheduled True if ran from download message list.
+	 * @param[in] processFlags Message processing flags.
+	 * @param[in] recMgmtHierarchyId Predefined target record management hierarchy id.
 	 */
-	void downloadMessageFinished(const QString &usrName, qint64 msgId,
+	void downloadMessageFinished(const AcntId &acntId, qint64 msgId,
 	   const QDateTime &deliveryTime, int result, const QString &errDesc,
-	   bool listScheduled);
-
-	/*!
-	 * @brief Emitted when download message finishes - webdatovka.
-	 *
-	 * @param[in] usrName       Account identifier (user login name).
-	 * @param[in] msgId         Message identifier.
-	 * @param[in] result        Operation outcome
-	 *                          (enum TaskDownloadMessage::Result).
-	 * @param[in] errDesc       Error description string.
-	 * @param[in] listScheduled True if ran from download message list.
-	 */
-	void downloadMessageFinishedMojeId(const QString &usrName, qint64 msgId,
-	   int result, const QString &errDesc, bool listScheduled);
+	   int processFlags, const QString &recMgmtHierarchyId);
 
 	/*!
 	 * @brief Emitted when download message finishes.
 	 *
-	 * @param[in] usrName   Account identifier (user login name).
+	 * @param[in] rAcntIdDb Regular account identifier.
+	 * @param[in] sAcntId Shadow account identifier.
 	 * @param[in] direction Sent or received messages
 	 *                      (enum MessageDirection).
 	 * @param[in] result    Operation outcome
@@ -76,9 +69,20 @@ signals:
 	 * @param[in] st        Number of sent messages on server.
 	 * @param[in] sn        Number of new sent messages (locally unknown).
 	 */
-	void downloadMessageListFinished(const QString &usrName,
-	    int direction, int result, const QString &errDesc,
-	    bool add, int rt, int rn, int st, int sn);
+	void downloadMessageListFinished(const AcntIdDb &rAcntIdDb,
+	    const AcntId &sAcntId, int direction, int result,
+	    const QString &errDesc, bool add, int rt, int rn, int st, int sn);
+
+	/*!
+	 * @brief Emitted when unlisted messages found.
+	 *
+	 * @param[in] rAcntIdDb Regular account identifier.
+	 * @param[in] direction Sent or received messages
+	 *                      (enum MessageDirection).
+	 * @param[in] unlistedMsgIds List of unlisted message identifiers.
+	 */
+	void foundDownloadedUnlistedMessages(const AcntIdDb &rAcntIdDb,
+	    int direction, const QList<MsgId> &unlistedMsgIds);
 
 	/*!
 	 * @brief Emitted when ZFO import finishes.
@@ -110,17 +114,11 @@ signals:
 	 */
 	void progressChange(const QString &label, int value);
 
-	/*!
-	 * @brief This signal is emitted when account is changed.
-	 *
-	 * @param[in] account username.
-	 */
-	void refreshAccountList(const QString &userName);
-
 	/*!
 	 * @brief Emitted when send message finishes.
 	 *
-	 * @param[in] userName      Account identifier (user login name).
+	 * @param[in] acntId Account identifier.
+	 * @param[in] testing True if this is a testing account.
 	 * @param[in] transactId    Transaction identifier.
 	 * @param[in] result        Operation outcome
 	 *                          (enum TaskSendMessage::Result).
@@ -128,22 +126,16 @@ signals:
 	 * @param[in] dbIDRecipient Recipient identifier.
 	 * @param[in] recipientName Recipient name.
 	 * @param[in] isPDZ         True if message was a PDZ.
+	 * @param[in] isVodz True if message was a high-volume message.
 	 * @param[in] dmId          Message identifier if message has been sent.
+	 * @param[in] processFlags Message processing flags.
+	 * @param[in] recMgmtHierarchyId Predefined target record management hierarchy id.
 	 */
-	void sendMessageFinished(const QString &userName,
+	void sendMessageFinished(const AcntId &acntId,
 	    const QString &transactId, int result, const QString &resultDesc,
 	    const QString &dbIDRecipient, const QString &recipientName,
-	    bool isPDZ, qint64 dmId);
-
-	/*!
-	 * @brief Emitted when mojeid send message finishes.
-	 *
-	 * @param[in] userName      Account username.
-	 * @param[in] resultList    ResultList from webdatovka.
-	 * @param[in] error         Error message from netmanager.
-	 */
-	void sendMessageMojeIdFinished(const QString &userName,
-	    const QStringList &resultList, const QString &errStr);
+	    bool isPDZ, bool isVodz, qint64 dmId, int processFlags,
+	    const QString &recMgmtHierarchyId);
 
 	/*!
 	 * @brief This signal is emitted when status bar text is changed
@@ -151,11 +143,33 @@ signals:
 	 * @param[in] text Status bar text.
 	 */
 	void statusBarChange(const QString &text);
+
+	/*!
+	 * @brief Emitted when records management stored messages task finishes.
+	 */
+	void recordsManagementStoredMessagesFinished(const QString &taskId);
+
+	/*!
+	 * @brief Emitted when data uploaded.
+	 *
+	 * @param[in] acntId Account identifier.
+	 * @param[in] transactId Transaction identifier.
+	 * @param[in] uploadTotal Expected total upload.
+	 * @param[in] uploadCurrent Cumulative current upload progress.
+	 */
+	void uploadProgress(const AcntId &acntId,
+	    const QString &transactId, qint64 uploadTotal, qint64 uploadCurrent);
+
+	/*!
+	 * @brief Emitted when upload finishes.
+	 *
+	 * @param[in] acntId Account identifier.
+	 * @param[in] transactId Transaction identifier.
+	 * @param[in] result Return value, depends on the emitter.
+	 * @param[in] resultDesc Result description, often the error description.
+	 * @param[in] resultVal Result value.
+	 */
+	void uploadProgressFinished(const AcntId &acntId,
+	    const QString &transactId, int result, const QString &resultDesc,
+	    const QVariant &resultVal);
 };
-
-/*!
- * @brief This object is used to emit signals from message processing workers.
- */
-extern MessageProcessingEmitter globMsgProcEmitter;
-
-#endif /* _MESSAGE_EMITTER_H_ */
