@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,12 +21,12 @@
  * the two.
  */
 
-#ifndef _TASK_AUTHENTICATE_MESSAGE_H_
-#define _TASK_AUTHENTICATE_MESSAGE_H_
+#pragma once
 
 #include <QByteArray>
 #include <QString>
 
+#include "src/datovka_shared/identifiers/account_id.h"
 #include "src/worker/task.h"
 
 /*!
@@ -34,6 +34,12 @@
  */
 class TaskAuthenticateMessage : public Task {
 public:
+	enum MsgSizeType {
+		MSG_BASIC, /*!< Always treat as normal message. */
+		MSG_VODZ_SIZE_DEPENDENT, /*!< VoDZ if message exceeds the limit for basic message. */
+		MSG_VODZ /*!< Always treat as VoDZ. */
+	};
+
 	/*!
 	 * @Brief Return state describing what happened.
 	 */
@@ -42,27 +48,26 @@ public:
 		AUTH_DATA_ERROR, /*!< Data to be authenticated are empty. */
 		AUTH_NOT_EQUAL, /*!< Data could not be authenticated. */
 		AUTH_ISDS_ERROR, /*!< Error communicating with ISDS. */
-		AUTH_ERR, /*!< Other error. */
-		AUTH_CANCELLED /*!< Operation cancelled, used outside. */
+		AUTH_ERR /*!< Other error. */
 	};
 
 	/*!
 	 * @brief Constructor, from data.
 	 *
-	 * @param[in] userName Account identifier (user login name).
+	 * @param[in] acntId Account identifier.
 	 * @param[in] fileName Name of file containing message data.
 	 */
-	explicit TaskAuthenticateMessage(const QString &userName,
-	    const QString &fileName);
+	explicit TaskAuthenticateMessage(const AcntId &acntId,
+	    const QString &fileName, enum MsgSizeType sizeType);
 
 	/*!
 	 * @brief Constructor, from file.
 	 *
-	 * @param[in] userName Account identifier (user login name).
-	 * @param[in] data     Message data.
+	 * @param[in] acntId Account identifier.
+	 * @param[in] data Message data.
 	 */
-	explicit TaskAuthenticateMessage(const QString &userName,
-	    const QByteArray &data);
+	explicit TaskAuthenticateMessage(const AcntId &acntId,
+	    const QByteArray &data, enum MsgSizeType sizeType);
 
 	/*!
 	 * @brief Performs action.
@@ -75,15 +80,16 @@ public:
 	 *
 	 * TODO -- This method ought to be protected.
 	 *
-	 * @param[in]  userName  Account identifier (user login name).
-	 * @param[in]  data      Message data.
-	 * @param[out] error     Error description.
+	 * @param[in]  acntId Account identifier.
+	 * @param[in]  data Message data.
+	 * @param[out] error Error description.
 	 * @param[out] longError Long error description.
 	 * @return Authentication result.
 	 */
 	static
-	enum Result authenticateMessage(const QString &userName,
-	    const QByteArray &data, QString &error, QString &longError);
+	enum Result authenticateMessage(const AcntId &acntId,
+	    const QByteArray &data, enum MsgSizeType sizeType,
+	    QString &error, QString &longError);
 
 	enum Result m_result; /*!< Authentication outcome. */
 	QString m_isdsError; /*!< Error description. */
@@ -96,8 +102,7 @@ private:
 	TaskAuthenticateMessage(const TaskAuthenticateMessage &);
 	TaskAuthenticateMessage &operator=(const TaskAuthenticateMessage &);
 
-	const QString m_userName; /*!< Account identifier (user login name). */
+	const AcntId m_acntId; /*!< Account identifier. */
 	QByteArray m_data; /*!< Message data. */
+	enum MsgSizeType m_sizeType; /*!< Message size type. */
 };
-
-#endif /* _TASK_AUTHENTICATE_MESSAGE_H_ */
