@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,11 +21,11 @@
  * the two.
  */
 
-#ifndef _TASK_DOWNLOAD_PASSWORD_INFO_H_
-#define _TASK_DOWNLOAD_PASSWORD_INFO_H_
+#pragma once
 
 #include <QString>
 
+#include "src/datovka_shared/identifiers/account_id.h"
 #include "src/worker/task.h"
 
 /*!
@@ -36,9 +36,9 @@ public:
 	/*!
 	 * @brief Constructor.
 	 *
-	 * @param[in] userName Account identifier (user login name).
+	 * @param[in] acntId Account identifier.
 	 */
-	explicit TaskDownloadPasswordInfo(const QString &userName);
+	explicit TaskDownloadPasswordInfo(const AcntId &acntId);
 
 	/*!
 	 * @brief Performs action.
@@ -60,16 +60,14 @@ private:
 	/*!
 	 * @brief Download credit information from ISDS.
 	 *
-	 * @param[in]  userName  Account identifier (user login name).
-	 * @param[out] error     Error description.
+	 * @param[in]  acntId Account identifier.
+	 * @param[out] error Error description.
 	 * @param[out] longError Long error description.
 	 * @retrun True if operation succeeded.
 	 */
 	static
-	bool downloadPasswordInfoFromISDS(const QString &userName,
+	bool downloadPasswordInfoFromISDS(const AcntId &acntId,
 	    QString &error, QString &longError);
 
-	const QString m_userName; /*!< Account identifier (user login name). */
+	const AcntId m_acntId; /*!< Account identifier. */
 };
-
-#endif /* _TASK_DOWNLOAD_PASSWORD_INFO_H_ */
