@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -24,42 +24,39 @@
 #include <cinttypes>
 #include <QThread>
 
+#include "src/datovka_shared/isds/error.h"
+#include "src/datovka_shared/isds/type_description.h"
+#include "src/datovka_shared/isds/types.h"
+#include "src/datovka_shared/log/log.h"
+#include "src/global.h"
 #include "src/io/isds_sessions.h"
-#include "src/io/message_db.h"
-#include "src/log/log.h"
+#include "src/io/message_db_set.h"
+#include "src/isds/services.h"
 #include "src/worker/message_emitter.h"
 #include "src/worker/task_erase_message.h"
 
-TaskEraseMessage::TaskEraseMessage(const QString &userName, MessageDbSet *dbSet,
-    const MessageDb::MsgId &msgId, enum MessageDirection msgDirect,
-    bool delFromIsds)
+TaskEraseMessage::TaskEraseMessage(const AcntIdDb &acntIdDb, const MsgId &msgId,
+    enum MessageDirection msgDirect, bool delFromIsds)
     : m_result(NOT_DELETED),
     m_isdsError(),
     m_isdsLongError(),
-    m_userName(userName),
-    m_dbSet(dbSet),
+    m_acntIdDb(acntIdDb),
     m_msgId(msgId),
     m_msgDirect(msgDirect),
     m_delFromIsds(delFromIsds)
 {
-	Q_ASSERT(!m_userName.isEmpty());
-	Q_ASSERT(0 != m_dbSet);
-	Q_ASSERT(m_msgId.dmId >= 0);
+	Q_ASSERT(m_acntIdDb.isValid());
+	Q_ASSERT(m_msgId.dmId() >= 0);
 }
 
 void TaskEraseMessage::run(void)
 {
-	if (m_userName.isEmpty()) {
+	if (Q_UNLIKELY(!m_acntIdDb.isValid())) {
 		Q_ASSERT(0);
 		return;
 	}
 
-	if (0 == m_dbSet) {
-		Q_ASSERT(0);
-		return;
-	}
-
-	if (0 > m_msgId.dmId) {
+	if (Q_UNLIKELY(0 > m_msgId.dmId())) {
 		Q_ASSERT(0);
 		return;
 	}
@@ -69,10 +66,10 @@ void TaskEraseMessage::run(void)
 
 	/* ### Worker task begin. ### */
 
-	m_result = eraseMessage(m_userName, m_dbSet, m_msgId, m_msgDirect,
+	m_result = eraseMessage(m_acntIdDb, m_msgId, m_msgDirect,
 	    m_delFromIsds, m_isdsError, m_isdsLongError);
 
-	emit globMsgProcEmitter.progressChange(PL_IDLE, 0);
+	emit GlobInstcs::msgProcEmitterPtr->progressChange(PL_IDLE, 0);
 
 	/* ### Worker task end. ### */
 
@@ -81,62 +78,72 @@ void TaskEraseMessage::run(void)
 }
 
 enum TaskEraseMessage::Result TaskEraseMessage::eraseMessage(
-    const QString &userName, MessageDbSet *dbSet, const MessageDb::MsgId &msgId,
+    const AcntIdDb &acntIdDb, const MsgId &msgId,
     enum MessageDirection msgDirect, bool delFromIsds, QString &error,
     QString &longError)
 {
-	Q_ASSERT(!userName.isEmpty());
-	Q_ASSERT(0 != dbSet);
-	Q_ASSERT(msgId.dmId >= 0);
+#define TRANSACTION_ENABLE true
 
-	isds_error status = IE_SUCCESS;
+	Q_ASSERT(acntIdDb.isValid());
+	Q_ASSERT(msgId.dmId() >= 0);
 
-	MessageDb *messageDb = dbSet->accessMessageDb(msgId.deliveryTime,
-	    false);
-	if (0 == messageDb) {
+	Isds::Error err;
+	err.setCode(Isds::Type::ERR_SUCCESS);
+
+	MessageDb *messageDb = acntIdDb.messageDbSet()->accessMessageDb(
+	    msgId.deliveryTime(), false);
+	if (Q_UNLIKELY(Q_NULLPTR == messageDb)) {
 		Q_ASSERT(0);
 		return NOT_DELETED;
 	}
 
 	/* First delete message in ISDS. */
 	if (delFromIsds) {
-		struct isds_ctx *session = globIsdsSessions.session(userName);
-		if (NULL == session) {
-			Q_ASSERT(0);
+		Isds::Session *session =
+		    GlobInstcs::isdsSessionsPtr->session(acntIdDb.username());
+		if (Q_UNLIKELY(Q_NULLPTR == session)) {
+			logErrorNL("Missing active session for username '%s'.",
+			    acntIdDb.username().toUtf8().constData());
 			return NOT_DELETED;
 		}
 
-		status = isds_delete_message_from_storage(session,
-		    QString::number(msgId.dmId).toUtf8().constData(),
+		err = Isds::Service::eraseMessage(session, msgId.dmId(),
 		    msgDirect == MSG_RECEIVED);
 
-		if (IE_SUCCESS == status) {
+		if (err.code() == Isds::Type::ERR_SUCCESS) {
 			logDebugLv1NL(
 			    "Message '%" PRId64 "' was deleted from ISDS.",
-			    msgId.dmId);
+			    UGLY_QINT64_CAST msgId.dmId());
 		} else {
-			error = isdsStrError(status);
-			longError = isdsLongMessage(session);
+			error = Isds::Description::descrError(err.code());
+			longError = err.longDescr();
 
 			logErrorNL(
 			    "Erasing message '%" PRId64 "'from ISDS returned status '%d': '%s'",
-			    msgId.dmId, status, error.toUtf8().constData());
+			    UGLY_QINT64_CAST msgId.dmId(), err.code(),
+			    error.toUtf8().constData());
 		}
 	}
 
-	if ((IE_SUCCESS != status) && (IE_INVAL != status)) {
+	if ((err.code() != Isds::Type::ERR_SUCCESS) && (err.code() != Isds::Type::ERR_INVAL)) {
 		return NOT_DELETED;
 	}
 
-	if (messageDb->msgsDeleteMessageData(msgId.dmId)) {
+	if (Q_UNLIKELY(GlobInstcs::req_halt)) {
+		return (delFromIsds && (Isds::Type::ERR_SUCCESS == err.code())) ? DELETED_ISDS : NOT_DELETED;
+	}
+
+	if (messageDb->msgsDeleteMessageData(msgId.dmId(), TRANSACTION_ENABLE)) {
 		logDebugLv1NL(
 		    "Message '%" PRId64 "' was deleted from local database.",
-		    msgId.dmId);
-		return (delFromIsds && (IE_SUCCESS == status)) ? DELETED_ISDS_LOCAL : DELETED_LOCAL;
+		    UGLY_QINT64_CAST msgId.dmId());
+		return (delFromIsds && (Isds::Type::ERR_SUCCESS == err.code())) ? DELETED_ISDS_LOCAL : DELETED_LOCAL;
 	} else {
 		logErrorNL(
 		    "Could not delete message '%" PRId64 "' from local database.",
-		    msgId.dmId);
-		return (delFromIsds && (IE_SUCCESS == status)) ? DELETED_ISDS : NOT_DELETED;
+		    UGLY_QINT64_CAST msgId.dmId());
+		return (delFromIsds && (Isds::Type::ERR_SUCCESS == err.code())) ? DELETED_ISDS : NOT_DELETED;
 	}
+
+#undef TRANSACTION_ENABLE
 }
