@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,17 +21,21 @@
  * the two.
  */
 
-#ifndef _TASK_IMPORT_ZFO_H_
-#define _TASK_IMPORT_ZFO_H_
+#pragma once
 
+#include <QCoreApplication> /* Q_DECLARE_TR_FUNCTIONS */
 #include <QString>
 
+#include "src/datovka_shared/isds/message_interface.h"
+#include "src/identifiers/account_id_db.h"
 #include "src/worker/task.h"
 
 /*!
  * @brief Task describing ZFO file import.
  */
 class TaskImportZfo : public Task {
+	Q_DECLARE_TR_FUNCTIONS(TaskImportZfo)
+
 public:
 	/*!
 	 * @brief Return state describing what happened.
@@ -44,6 +48,7 @@ public:
 		IMP_DB_INS_ERR, /*!< Error inserting into database. */
 		IMP_DB_MISSING_MSG, /*!< Related message to delivery info is missing. */
 		IMP_DB_EXISTS, /*!< Message already exists. */
+		IMP_ABORTED, /*!< Operation has been aborted. */
 		IMP_ERR /*!< Other error. */
 	};
 
@@ -59,14 +64,16 @@ public:
 	/*!
 	 * @brief Constructor.
 	 *
-	 * @param[in] accounts     List of account identifiers.
-	 * @param[in] fileName     Full path to ZFO file.
-	 * @param[in] type         ZFO file type; type is determined if unknown.
+	 * @param[in] acntIdDbList List of account identifiers.
+	 * @param[in] fileName Full path to ZFO file.
+	 * @param[in] type ZFO file type; type is determined if unknown.
+	 * @param[in] storeRawInDb Whether to store raw data in database file.
 	 * @param[in] authenticate True if you want to authenticate message
 	 *                         before importing.
 	 */
-	explicit TaskImportZfo(const QList<Task::AccountDescr> &accounts,
-	    const QString &fileName, enum ZfoType type, bool authenticate);
+	explicit TaskImportZfo(const QList<AcntIdDb> &acntIdDbList,
+	    const QString &fileName, enum ZfoType type, bool storeRawInDb,
+	    bool authenticate);
 
 	/*!
 	 * @brief Performs action.
@@ -90,7 +97,7 @@ public:
 //	QString m_isdsLongError; /*!< Long error description. */
 	QString m_resultDesc; /*!<
 	                       * Result description that has mostly nothing
-	                       * to do with libisds.
+	                       * to do with libdatovka.
 	                       */
 
 private:
@@ -104,77 +111,83 @@ private:
 	 * @brief Tries to import a single message ZFO file into a single
 	 *     account.
 	 *
-	 * @param[in]  acnt          Account to try to insert into.
-	 * @param[in]  message       Parsed message.
-	 * @param[in]  dmId          Message identifier.
-	 * @param[in]  deliveryTime  Message delivery time.
-	 * @param[in]  direct        Whether it is sent or received message.
-	 * @param[in]  fileName      If supplied then the file is going to be
-	 *                           authenticated on ISDS server.
-	 * @param[out] isdsError     Error description.
+	 * @param[in]  acntIdDb Account to try to insert into.
+	 * @param[in]  message Parsed message.
+	 * @param[in]  dmId Message identifier.
+	 * @param[in]  deliveryTime Message delivery time.
+	 * @param[in]  direct Whether it is sent or received message.
+	 * @param[in]  fileName If supplied then the file is going to be
+	 *                      authenticated on ISDS server.
+	 * @param[in]  storeRawInDb Whether to store raw data in database file.
+	 * @param[out] isdsError Error description.
 	 * @param[out] isdsLongError Long error description.
-	 * @param[out] resultDesc    Result description.
+	 * @param[out] resultDesc Result description.
 	 * @returns Error identifier.
 	 */
 	static
-	enum Result importMessageZfoSingle(const Task::AccountDescr &acnt,
-	    const struct isds_message *message, qint64 dmId,
+	enum Result importMessageZfoSingle(const AcntIdDb &acntIdDb,
+	    const Isds::Message &message, qint64 dmId,
 	    const QDateTime &deliveryTime, enum MessageDirection direct,
-	    const QString &fileName, QString &isdsError, QString &isdsLongError,
-	    QString &resultDesc);
+	    const QString &fileName, bool storeRawInDb, QString &isdsError,
+	    QString &isdsLongError, QString &resultDesc);
 
 	/*!
 	 * @brief Imports message info database.
 	 *
-	 * @param[in] accounts     List of accounts to try to import data into.
+	 * @param[in] acntIdDbList List of accounts to try to import data into.
 	 * @param[in] fileName     Full name of ZFO file holding the message.
+	 * @param[in] storeRawInDb Whether to store raw data in database file.
 	 * @param[in] authenticate True if data should be authenticated before
 	 *                         inserting.
 	 * @param[in] resultDesc   String holding result description.
 	 * @return Status or error code.
 	 */
 	static
-	enum Result importMessageZfo(const QList<Task::AccountDescr> &accounts,
-	    const QString &fileName, bool authenticate, QString &resultDesc);
+	enum Result importMessageZfo(const QList<AcntIdDb> &acntIdDbList,
+	    const QString &fileName, bool storeRawInDb, bool authenticate,
+	    QString &resultDesc);
 
 	/*!
 	 * @brief Tries to import a single delivery info ZFO file into a single
 	 *     account.
 	 *
-	 * @param[in]  acnt          Account to try to insert into.
-	 * @param[in]  message       Parsed delivery info.
-	 * @param[in]  dmId          Message identifier.
-	 * @param[in]  deliveryTime  Message delivery time.
-	 * @param[in]  fileName      If supplied then the file is going to be
-	 *                           authenticated on ISDS server.
-	 * @param[out] isdsError     Error description.
+	 * @param[in]  acntIdDb Account to try to insert into.
+	 * @param[in]  message Parsed delivery info.
+	 * @param[in]  dmId Message identifier.
+	 * @param[in]  deliveryTime Message delivery time.
+	 * @param[in]  fileName If supplied then the file is going to be
+	 *                      authenticated on ISDS server.
+	 * @param[in]  storeRawInDb Whether to store raw data in database file.
+	 * @param[out] isdsError Error description.
 	 * @param[out] isdsLongError Long error description.
-	 * @param[out] resultDesc    Result description.
+	 * @param[out] resultDesc Result description.
 	 * @returns Error identifier.
 	 */
 	static
-	enum Result importDeliveryZfoSingle(const Task::AccountDescr &acnt,
-	    const struct isds_message *message, qint64 dmId,
+	enum Result importDeliveryZfoSingle(const AcntIdDb &acntIdDb,
+	    const Isds::Message &message, qint64 dmId,
 	    const QDateTime &deliveryTime, const QString &fileName,
-	    QString &isdsError, QString &isdsLongError, QString &resultDesc);
+	    bool storeRawInDb, QString &isdsError, QString &isdsLongError,
+	    QString &resultDesc);
 
 	/*!
 	 * @brief Imports delivery info into databases.
 	 *
-	 * @param[in] accounts     List of accounts to try to import data into.
+	 * @param[in] acntIdDbList List of accounts to try to import data into.
 	 * @param[in] fileName     Full name of ZFO file holding delivery info.
+	 * @param[in] storeRawInDb Whether to store raw data in database file.
 	 * @param[in] authenticate True if data should be authenticated before
 	 *                         inserting.
 	 * @param[in] resultDesc   String holding result description.
 	 * @return Status or error code.
 	 */
 	static
-	enum Result importDeliveryZfo(const QList<Task::AccountDescr> &accounts,
-	    const QString &fileName, bool authenticate, QString &resultDesc);
+	enum Result importDeliveryZfo(const QList<AcntIdDb> &acntIdDbList,
+	    const QString &fileName, bool storeRawInDb, bool authenticate,
+	    QString &resultDesc);
 
-	QList<Task::AccountDescr> m_accounts; /*!< List of accounts to be inserted into. */
+	QList<AcntIdDb> m_acntIdDbList; /*!< List of accounts to be inserted into. */
 	enum ZfoType m_zfoType; /*!< Type of the ZFO file. */
+	const bool m_storeRawInDb; /*!, True whether to store attachment into database, false when to store as files. */
 	const bool m_auth; /*!< True if authentication before importing. */
 };
-
-#endif /* _TASK_IMPORT_ZFO_H_ */
