@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -23,20 +23,21 @@
 
 #include <QThread>
 
+#include "src/datovka_shared/log/log.h"
+#include "src/global.h"
 #include "src/io/isds_sessions.h"
-#include "src/log/log.h"
 #include "src/worker/task_keep_alive.h"
 
-TaskKeepAlive::TaskKeepAlive(const QString &userName)
+TaskKeepAlive::TaskKeepAlive(const AcntId &acntId)
     : m_isAlive(false),
-    m_userName(userName)
+    m_acntId(acntId)
 {
-	Q_ASSERT(!m_userName.isEmpty());
+	Q_ASSERT(m_acntId.isValid());
 }
 
 void TaskKeepAlive::run(void)
 {
-	if (m_userName.isEmpty()) {
+	if (Q_UNLIKELY(!m_acntId.isValid())) {
 		Q_ASSERT(0);
 		return;
 	}
@@ -46,7 +47,7 @@ void TaskKeepAlive::run(void)
 
 	/* ### Worker task begin. ### */
 
-	m_isAlive = globIsdsSessions.isConnectedToIsds(m_userName);
+	m_isAlive = GlobInstcs::isdsSessionsPtr->isConnectedToIsds(m_acntId.username());
 	if (m_isAlive) {
 		logInfo("%s\n", "Connection to ISDS is alive :)");
 	} else {
