@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -21,8 +21,9 @@
  * the two.
  */
 
+#include <QDir>
+#include <QFileInfo>
 #include <QtGlobal> /* QT_VERSION */
-#include <QObject>
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 4, 0))
 #include <QStorageInfo>
 #else /* < Qt-5.4 */
@@ -31,7 +32,9 @@
 #endif /* >= Qt-5.4 */
 #include <QThread>
 
-#include "src/log/log.h"
+#include "src/datovka_shared/log/log.h"
+#include "src/global.h"
+#include "src/io/message_db_set.h"
 #include "src/worker/message_emitter.h"
 #include "src/worker/task_vacuum_db_set.h"
 
@@ -44,7 +47,7 @@ TaskVacuumDbSet::TaskVacuumDbSet(MessageDbSet *dbSet)
 
 void TaskVacuumDbSet::run(void)
 {
-	if (0 == m_dbSet) {
+	if (Q_UNLIKELY(Q_NULLPTR == m_dbSet)) {
 		Q_ASSERT(0);
 		return;
 	}
@@ -56,7 +59,7 @@ void TaskVacuumDbSet::run(void)
 
 	m_success = vacuumDbSet(m_dbSet, m_error);
 
-	emit globMsgProcEmitter.progressChange(PL_IDLE, 0);
+	emit GlobInstcs::msgProcEmitterPtr->progressChange(PL_IDLE, 0);
 
 	/* ### Worker task end. ### */
 
@@ -64,14 +67,21 @@ void TaskVacuumDbSet::run(void)
 	    (void *) QThread::currentThreadId());
 }
 
-QString TaskVacuumDbSet::storagePlace(MessageDbSet *dbSet)
+/*!
+ * @brief Returns full path to database file directory.
+ *
+ * @param[in,out] dbSet Non-null pointer to database container.
+ * @return Null string on error.
+ */
+static
+QString storagePlace(MessageDbSet *dbSet)
 {
-	Q_ASSERT(0 != dbSet);
+	Q_ASSERT(Q_NULLPTR != dbSet);
 
 	QString place;
 
 	const QStringList fileNames(dbSet->fileNames());
-	foreach (const QString &fileName, fileNames) {
+	for (const QString &fileName : fileNames) {
 		const QFileInfo fInfo(fileName);
 		if (!fInfo.isFile()) {
 			return QString();
@@ -89,9 +99,16 @@ QString TaskVacuumDbSet::storagePlace(MessageDbSet *dbSet)
 	return place;
 }
 
+/*!
+ * @brief Check whether free space in directory is available.
+ *
+ * @param storagePlace Full path to database directory.
+ * @param spaceSize Size of space requested.
+ * @return True if requested amount of space is available.
+ */
+static
+bool haveStorageSpace(const QString &storagePlace, qint64 spaceSize)
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 4, 0))
-bool TaskVacuumDbSet::haveStorageSpace(const QString &storagePlace,
-    qint64 spaceSize)
 {
 	Q_ASSERT(!storagePlace.isEmpty());
 	Q_ASSERT(spaceSize > 0);
@@ -106,8 +123,6 @@ bool TaskVacuumDbSet::haveStorageSpace(const QString &storagePlace,
 	return sInfo.bytesAvailable() > spaceSize;
 }
 #else /* < Qt-5.4 */
-bool TaskVacuumDbSet::haveStorageSpace(const QString &storagePlace,
-    qint64 spaceSize)
 {
 	Q_ASSERT(!storagePlace.isEmpty());
 	Q_ASSERT(spaceSize > 0);
@@ -138,9 +153,12 @@ bool TaskVacuumDbSet::haveStorageSpace(const QString &storagePlace,
 
 bool TaskVacuumDbSet::vacuumDbSet(MessageDbSet *dbSet, QString &error)
 {
-	Q_ASSERT(0 != dbSet);
+	if (Q_UNLIKELY(Q_NULLPTR == dbSet)) {
+		Q_ASSERT(0);
+		return false;
+	}
 
-	qint64 maxDbSize = dbSet->underlyingFileSize(MessageDbSet::SC_LARGEST);
+	qint64 maxDbSize = dbSet->fileSize(MessageDbSet::SC_LARGEST);
 	if (maxDbSize <= 0) {
 		/* Nothing to do. */
 		return true;
@@ -150,20 +168,20 @@ bool TaskVacuumDbSet::vacuumDbSet(MessageDbSet *dbSet, QString &error)
 	maxDbSize += (qint64)(maxDbSize * 0.02);
 
 	const QString dbDir(storagePlace(dbSet));
-	if (dbDir.isEmpty()) {
-		error = QObject::tr("Could not determine database directory");
+	if (Q_UNLIKELY(dbDir.isEmpty())) {
+		error = tr("Could not determine database directory");
 		return false;
 	}
 
 	/* Test for available space. */
 	if (!haveStorageSpace(dbDir, maxDbSize)) {
-		error = QObject::tr("Not enough space on device where '%1' resides.").arg(dbDir);
+		error = tr("Not enough space on device where '%1' resides.").arg(dbDir);
 		return false;
 	}
 
 	bool ret = dbSet->vacuum();
 	if (!ret) {
-		error = QObject::tr("Calling vacuum on database set failed.");
+		error = tr("Calling vacuum on database set failed.");
 	}
 
 	return ret;
