@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 CZ.NIC
+ * Copyright (C) 2014-2024 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -25,7 +25,7 @@
 
 #include <QDialog>
 
-#include "src/records_management/json/service_info.h"
+#include "src/datovka_shared/records_management/json/service_info.h"
 
 namespace Ui {
 	class DialogueServiceInfo;
@@ -38,9 +38,10 @@ public:
 	/*!
 	 * @brief Dialogue constructor.
 	 */
-	explicit DialogueServiceInfo(const ServiceInfoResp &sir,
+	explicit DialogueServiceInfo(const RecMgmt::ServiceInfoResp &sir,
 	    QWidget *parent = Q_NULLPTR);
 
+	virtual
 	~DialogueServiceInfo(void);
 
 private:
